@@ -19,6 +19,8 @@
      // by-one bug in the kernel) the usable bytes are only 30.
 #    define SHM_NAME_MAX 30
 #  endif
+#elif defined(Q_OS_WINDOWS)
+#  include "qt_windows.h"
 #endif
 
 #if QT_CONFIG(sharedmemory) || QT_CONFIG(systemsemaphore)
@@ -213,7 +215,7 @@ QNativeIpcKey QtIpcCommon::platformSafeKey(const QString &key, QtIpcCommon::IpcT
             }
 
             QString result = prefix + mid + payload;
-#ifdef MAX_PATH
+#ifdef Q_OS_WINDOWS
             result.truncate(MAX_PATH);
 #endif
             k.setNativeKey(result);
@@ -242,6 +244,8 @@ QNativeIpcKey QtIpcCommon::platformSafeKey(const QString &key, QtIpcCommon::IpcT
     \since 6.6
     \brief The QNativeIpcKey class holds a native key used by QSystemSemaphore and QSharedMemory.
 
+    \compares equality
+
     The \l QSharedMemory and \l QSystemSemaphore classes identify their
     resource using a system-wide identifier known as a "key". The low-level key
     value as well as the key type are encapsulated in Qt using the \l
@@ -420,9 +424,7 @@ void QNativeIpcKey::destroy_internal() noexcept
 
 /*!
     \fn QNativeIpcKey::swap(QNativeIpcKey &other) noexcept
-
-    Swaps the native IPC key and type \a other with this object.
-    This operation is very fast and never fails.
+    \memberswap{native IPC key and type}
 */
 
 /*!
@@ -495,15 +497,8 @@ void QNativeIpcKey::setNativeKey_internal(const QString &)
 }
 
 /*!
-    \fn size_t QNativeIpcKey::qHash(const QNativeIpcKey &ipcKey) noexcept
-
-    Returns the hash value for \a ipcKey, using a default seed of \c 0.
-*/
-
-/*!
-    \fn size_t QNativeIpcKey::qHash(const QNativeIpcKey &ipcKey, size_t seed) noexcept
-
-    Returns the hash value for \a ipcKey, using \a seed to seed the calculation.
+    \fn size_t QNativeIpcKey::qHash(const QNativeIpcKey &key, size_t seed)
+    \qhash{QNativeIpcKey}
 */
 size_t qHash(const QNativeIpcKey &ipcKey, size_t seed) noexcept
 {
