@@ -6,7 +6,6 @@
 #include "qcoreapplication_p.h"
 #include "qstringlist.h"
 #include "qfileinfo.h"
-#include "qcorecmdlineargs_p.h"
 #ifndef QT_NO_QOBJECT
 #include "qmutex.h"
 #include <private/qthread_p.h>
@@ -21,7 +20,7 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-Q_CORE_EXPORT QString qAppFileName()                // get application file name
+QString qAppFileName()                // get application file name
 {
     /*
       GetModuleFileName() returns the length of the module name, when it has
