@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtCore module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qmetaobjectbuilder_p.h"
 
@@ -89,7 +53,7 @@ Q_CORE_EXPORT bool isBuiltinType(const QByteArray &type)
 } // namespace QtPrivate
 
 // copied from qmetaobject.cpp
-[[maybe_unused]] static inline const QMetaObjectPrivate *priv(const uint* data)
+[[maybe_unused]] static inline const QMetaObjectPrivate *qmobPriv(const uint* data)
 { return reinterpret_cast<const QMetaObjectPrivate*>(data); }
 
 class QMetaMethodBuilderPrivate
@@ -195,6 +159,7 @@ public:
 
     QByteArray name;
     QByteArray enumName;
+    QMetaType metaType;
     bool isFlag;
     bool isScoped;
     QList<QByteArray> keys;
@@ -633,6 +598,7 @@ QMetaEnumBuilder QMetaObjectBuilder::addEnumerator(const QMetaEnum &prototype)
 {
     QMetaEnumBuilder en = addEnumerator(prototype.name());
     en.setEnumName(prototype.enumName());
+    en.setMetaType(prototype.metaType());
     en.setIsFlag(prototype.isFlag());
     en.setIsScoped(prototype.isScoped());
     int count = prototype.keyCount();
@@ -740,7 +706,7 @@ void QMetaObjectBuilder::addMetaObject(const QMetaObject *prototype,
     }
 
     if ((members & RelatedMetaObjects) != 0) {
-        Q_ASSERT(priv(prototype->d.data)->revision >= 2);
+        Q_ASSERT(qmobPriv(prototype->d.data)->revision >= 2);
         const auto *objects = prototype->d.relatedMetaObjects;
         if (objects) {
             while (*objects != nullptr) {
@@ -751,7 +717,7 @@ void QMetaObjectBuilder::addMetaObject(const QMetaObject *prototype,
     }
 
     if ((members & StaticMetacall) != 0) {
-        Q_ASSERT(priv(prototype->d.data)->revision >= 6);
+        Q_ASSERT(qmobPriv(prototype->d.data)->revision >= 6);
         if (prototype->d.static_metacall)
             setStaticMetacallFunction(prototype->d.static_metacall);
     }
@@ -1062,6 +1028,9 @@ int QMetaObjectBuilder::indexOfClassInfo(const QByteArray &name)
 }
 
 // Align on a specific type boundary.
+#ifdef ALIGN
+#  undef ALIGN
+#endif
 #define ALIGN(size,type)    \
     (size) = ((size) + sizeof(type) - 1) & ~(sizeof(type) - 1)
 
@@ -1119,7 +1088,7 @@ static void writeString(char *out, int i, const QByteArray &str,
     memcpy(out + 2 * i * sizeof(uint), &offsetLen, 2 * sizeof(uint));
 
     memcpy(out + offset, str.constData(), size);
-    out[offsetOfStringdataMember + stringdataOffset + size] = '\0';
+    out[offset + size] = '\0';
 
     stringdataOffset += size + 1;
 }
@@ -1193,8 +1162,8 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
     }
 
     // Populate the QMetaObjectPrivate structure.
-    QMetaObjectPrivate *pmeta
-        = reinterpret_cast<QMetaObjectPrivate *>(buf + size);
+    QMetaObjectPrivate *pmeta = buf ? reinterpret_cast<QMetaObjectPrivate *>(buf + size)
+                                    : nullptr;
     //int pmetaSize = size;
     dataIndex = MetaObjectPrivateFieldCount;
     int methodParametersDataSize =
@@ -1203,7 +1172,7 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
             - int(d->methods.size())       // return "parameters" don't have names
             - int(d->constructors.size()); // "this" parameters don't have names
     if constexpr (mode == Construct) {
-        static_assert(QMetaObjectPrivate::OutputRevision == 10, "QMetaObjectBuilder should generate the same version as moc");
+        static_assert(QMetaObjectPrivate::OutputRevision == 12, "QMetaObjectBuilder should generate the same version as moc");
         pmeta->revision = QMetaObjectPrivate::OutputRevision;
         pmeta->flags = d->flags.toInt();
         pmeta->className = 0;   // Class name is always the first string.
@@ -1281,8 +1250,8 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
 
     // Output the methods in the class.
     Q_ASSERT(!buf || dataIndex == pmeta->methodData);
-    // + 1 for metatype of this metaobject
-    int parameterMetaTypesIndex = int(d->properties.size()) + 1;
+    // property count + enum count  + 1 for metatype of this metaobject
+    int parameterMetaTypesIndex = int(d->properties.size()) + int(d->enumerators.size()) + 1;
     for (const auto &method : d->methods) {
         [[maybe_unused]] int name = strings.enter(method.name());
         int argc = method.parameterCount();
@@ -1439,41 +1408,46 @@ static int buildMetaObject(QMetaObjectBuilderPrivate *d, char *buf,
         size += sizeof(SuperData) * (d->relatedMetaObjects.size() + 1);
     }
 
-    if (d->properties.size() > 0 || d->methods.size() > 0 || d->constructors.size() > 0) {
-        ALIGN(size, QtPrivate::QMetaTypeInterface *);
-        auto types = reinterpret_cast<QtPrivate::QMetaTypeInterface **>(buf + size);
-        if constexpr (mode == Construct) {
-            meta->d.metaTypes = types;
-            for (const auto &prop : d->properties) {
-                QMetaType mt = prop.metaType;
-                *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
-                types++;
-            }
-            // add metatype interface for this metaobject - must be null
-            // as we can't know our metatype
-            *types = nullptr;
+    ALIGN(size, QtPrivate::QMetaTypeInterface *);
+    auto types = reinterpret_cast<const QtPrivate::QMetaTypeInterface **>(buf + size);
+    if constexpr (mode == Construct) {
+        meta->d.metaTypes = types;
+        for (const auto &prop : d->properties) {
+            QMetaType mt = prop.metaType;
+            *types = mt.iface();
             types++;
-            for (const auto &method: d->methods) {
-                QMetaType mt(QMetaType::fromName(method.returnType).id());
-                *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+        }
+        // add metatypes for enumerators
+        for (const auto &enumerator: d->enumerators) {
+            QMetaType mt = enumerator.metaType;
+            mt.registerType();
+            *types = mt.iface();
+            types++;
+        }
+        // add metatype interface for this metaobject - must be null
+        // as we can't know our metatype
+        *types = nullptr;
+        types++;
+        for (const auto &method: d->methods) {
+            QMetaType mt(QMetaType::fromName(method.returnType).id());
+            *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
+            types++;
+            for (const auto &parameterType: method.parameterTypes()) {
+                QMetaType mt = QMetaType::fromName(parameterType);
+                *types = mt.iface();
+                types++;
+            }
+        }
+        for (const auto &constructor : d->constructors) {
+            for (const auto &parameterType : constructor.parameterTypes()) {
+                QMetaType mt = QMetaType::fromName(parameterType);
+                *types = mt.iface();
                 types++;
-                for (const auto &parameterType: method.parameterTypes()) {
-                    QMetaType mt = QMetaType::fromName(parameterType);
-                    *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
-                    types++;
-                }
-            }
-            for (const auto &constructor : d->constructors) {
-                for (const auto &parameterType : constructor.parameterTypes()) {
-                    QMetaType mt = QMetaType::fromName(parameterType);
-                    *types = reinterpret_cast<QtPrivate::QMetaTypeInterface *&>(mt);
-                    types++;
-                }
             }
         }
-        // parameterMetaTypesIndex is equal to the total number of metatypes
-        size += sizeof(QMetaType) * parameterMetaTypesIndex;
     }
+    // parameterMetaTypesIndex is equal to the total number of metatypes
+    size += sizeof(QMetaType) * parameterMetaTypesIndex;
 
     // Align the final size and return it.
     ALIGN(size, void *);
@@ -1725,7 +1699,7 @@ int QMetaMethodBuilder::attributes() const
 {
     QMetaMethodBuilderPrivate *d = d_func();
     if (d)
-        return (d->attributes >> 4);
+        return (d->attributes >> 4) & 0x7;
     else
         return 0;
 }
@@ -1738,8 +1712,10 @@ int QMetaMethodBuilder::attributes() const
 void QMetaMethodBuilder::setAttributes(int value)
 {
     QMetaMethodBuilderPrivate *d = d_func();
-    if (d)
-        d->attributes = ((d->attributes & 0x0f) | (value << 4));
+    if (d) {
+        d->attributes &= ~0x70;
+        d->attributes |= (value & 0x7) << 4;
+    }
 }
 
 /*!
@@ -2340,6 +2316,31 @@ void QMetaEnumBuilder::setEnumName(const QByteArray &alias)
         d->enumName = alias;
 }
 
+/*!
+    Returns the meta type of the enumerator.
+
+    \since 6.6
+*/
+QMetaType QMetaEnumBuilder::metaType() const
+{
+    if (QMetaEnumBuilderPrivate *d = d_func())
+        return d->metaType;
+    return QMetaType();
+}
+
+/*!
+    Sets this enumerator to have the given \c metaType.
+
+    \since 6.6
+    \sa metaType()
+*/
+void QMetaEnumBuilder::setMetaType(QMetaType metaType)
+{
+    QMetaEnumBuilderPrivate *d = d_func();
+    if (d)
+        d->metaType = metaType;
+}
+
 /*!
     Returns \c true if this enumerator is used as a flag; otherwise returns
     false.
