@@ -53,11 +53,16 @@ static QJsonValue::Type convertFromCborType(QCborValue::Type type) noexcept
     \inmodule QtCore
     \ingroup json
     \ingroup shared
+    \ingroup qtserialization
     \reentrant
     \since 5.0
 
     \brief The QJsonValue class encapsulates a value in JSON.
 
+    \compares equality
+    \compareswith equality QJsonValueConstRef QJsonValueRef
+    \endcompareswith
+
     A value in JSON can be one of 6 basic types:
 
     JSON is a format to store structured data. It has 6 basic data types:
@@ -95,7 +100,7 @@ static QJsonValue::Type convertFromCborType(QCborValue::Type type) noexcept
     \li \l {QJsonObject}::operator[](const QString & key) const
     \endlist
 
-    \sa {JSON Support in Qt}, {JSON Save Game Example}
+    \sa {JSON Support in Qt}, {Saving and Loading a Game}
 */
 
 /*!
@@ -197,7 +202,7 @@ QJsonValue::QJsonValue(const QString &s)
  */
 
 /*!
-    Creates a value of type String, with value \a s.
+    Creates a value of type String, with the Latin-1 string viewed by \a s.
  */
 QJsonValue::QJsonValue(QLatin1StringView s)
     : value(s)
@@ -247,15 +252,12 @@ QJsonValue::~QJsonValue() = default;
 /*!
     Creates a copy of \a other.
  */
-QJsonValue::QJsonValue(const QJsonValue &other)
-    : value(other.value)
-{
-}
+QJsonValue::QJsonValue(const QJsonValue &other) noexcept = default;
 
 /*!
     Assigns the value stored in \a other to this object.
  */
-QJsonValue &QJsonValue::operator =(const QJsonValue &other)
+QJsonValue &QJsonValue::operator =(const QJsonValue &other) noexcept
 {
     QJsonValue copy(other);
     swap(copy);
@@ -290,8 +292,7 @@ void QJsonValue::swap(QJsonValue &other) noexcept
 /*!
     \fn void QJsonValue::swap(QJsonValue &other)
     \since 5.10
-
-    Swaps the value \a other with this. This operation is very fast and never fails.
+    \memberswap{value}
 */
 
 /*!
@@ -347,6 +348,7 @@ void QJsonValue::swap(QJsonValue &other) noexcept
     error cases as e.g. accessing a non existing key in a QJsonObject.
  */
 
+#ifndef QT_NO_VARIANT
 /*!
     Converts \a variant to a QJsonValue and returns it.
 
@@ -589,6 +591,7 @@ QVariant QJsonValue::toVariant() const
                     error condition, when trying to read an out of bounds value
                     in an array or a non existent key in an object.
 */
+#endif // !QT_NO_VARIANT
 
 /*!
     Returns the type of the value.
@@ -823,35 +826,37 @@ const QJsonValue QJsonValue::operator[](qsizetype i) const
 }
 
 /*!
-    Returns \c true if the value is equal to \a other.
- */
-bool QJsonValue::operator==(const QJsonValue &other) const
+    \fn bool QJsonValue::operator==(const QJsonValue &lhs, const QJsonValue &rhs)
+
+    Returns \c true if the \a lhs value is equal to \a rhs value, \c false otherwise.
+*/
+bool comparesEqual(const QJsonValue &lhs, const QJsonValue &rhs)
 {
-    if (value.type() != other.value.type()) {
-        if (isDouble() && other.isDouble()) {
+    if (lhs.value.type() != rhs.value.type()) {
+        if (lhs.isDouble() && rhs.isDouble()) {
             // One value Cbor integer, one Cbor double, should interact as doubles.
-            return toDouble() == other.toDouble();
+            return lhs.toDouble() == rhs.toDouble();
         }
         return false;
     }
 
-    switch (value.type()) {
+    switch (lhs.value.type()) {
     case QCborValue::Undefined:
     case QCborValue::Null:
     case QCborValue::True:
     case QCborValue::False:
         break;
     case QCborValue::Double:
-        return toDouble() == other.toDouble();
+        return lhs.toDouble() == rhs.toDouble();
     case QCborValue::Integer:
-        return QJsonPrivate::Value::valueHelper(value)
-                == QJsonPrivate::Value::valueHelper(other.value);
+        return QJsonPrivate::Value::valueHelper(lhs.value)
+                == QJsonPrivate::Value::valueHelper(rhs.value);
     case QCborValue::String:
-        return toString() == other.toString();
+        return lhs.toString() == rhs.toString();
     case QCborValue::Array:
-        return toArray() == other.toArray();
+        return lhs.toArray() == rhs.toArray();
     case QCborValue::Map:
-        return toObject() == other.toObject();
+        return lhs.toObject() == rhs.toObject();
     default:
         return false;
     }
@@ -859,12 +864,10 @@ bool QJsonValue::operator==(const QJsonValue &other) const
 }
 
 /*!
-    Returns \c true if the value is not equal to \a other.
- */
-bool QJsonValue::operator!=(const QJsonValue &other) const
-{
-    return !(*this == other);
-}
+    \fn bool QJsonValue::operator!=(const QJsonValue &lhs, const QJsonValue &rhs)
+
+    Returns \c true if the \a lhs value is not equal to \a rhs value, \c false otherwise.
+*/
 
 /*!
     \class QJsonValueRef
@@ -938,10 +941,12 @@ QJsonValueRef &QJsonValueRef::operator =(const QJsonValueRef &ref)
     return assignToRef(*this, d->valueAt(index), is_object);
 }
 
+#ifndef QT_NO_VARIANT
 QVariant QJsonValueConstRef::toVariant() const
 {
     return concrete(*this).toVariant();
 }
+#endif // !QT_NO_VARIANT
 
 QJsonArray QJsonValueConstRef::toArray() const
 {
@@ -1013,7 +1018,7 @@ QJsonValue QJsonValueConstRef::concrete(QJsonValueConstRef self) noexcept
 QString QJsonValueConstRef::objectKey(QJsonValueConstRef self)
 {
     Q_ASSERT(self.is_object);
-    Q_ASSUME(self.is_object);
+    Q_ASSERT(self.is_object);
     const QCborContainerPrivate *d = QJsonPrivate::Value::container(self);
     qsizetype index = QJsonPrivate::Value::indexHelper(self);
 
@@ -1100,8 +1105,7 @@ size_t qHash(const QJsonValue &value, size_t seed)
     case QJsonValue::Undefined:
         return seed;
     }
-    Q_UNREACHABLE();
-    return 0;
+    Q_UNREACHABLE_RETURN(0);
 }
 
 #if !defined(QT_NO_DEBUG_STREAM) && !defined(QT_JSON_READONLY)
