@@ -14,6 +14,7 @@ static const int QTEXTSTREAM_BUFFERSIZE = 16384;
 
     \ingroup io
     \ingroup string-processing
+    \ingroup qtserialization
     \reentrant
 
     QTextStream can operate on a QIODevice, a QByteArray or a
@@ -195,6 +196,8 @@ static const int QTEXTSTREAM_BUFFERSIZE = 16384;
 #include "qnumeric.h"
 #include "qvarlengtharray.h"
 #include <private/qdebug_p.h>
+#include <private/qnumeric_p.h>
+#include <private/qtools_p.h>
 
 #include <locale.h>
 #include "private/qlocale_p.h"
@@ -244,6 +247,12 @@ static const int QTEXTSTREAM_BUFFERSIZE = 16384;
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
+using namespace QtMiscUtils;
+
+#ifndef QT_NO_QOBJECT
+QDeviceClosedNotifier::~QDeviceClosedNotifier()
+    = default;
+#endif
 
 //-------------------------------------------------------------------
 
@@ -356,6 +365,7 @@ bool QTextStreamPrivate::fillReadBuffer(qint64 maxBytes)
     if (bytesRead <= 0)
         return false;
 
+#ifndef QT_BOOTSTRAPPED
     if (autoDetectUnicode) {
         autoDetectUnicode = false;
 
@@ -370,6 +380,7 @@ bool QTextStreamPrivate::fillReadBuffer(qint64 maxBytes)
 #if defined (QTEXTSTREAM_DEBUG)
     qDebug("QTextStreamPrivate::fillReadBuffer(), using %s encoding", QStringConverter::nameForEncoding(encoding));
 #endif
+#endif
 
 #if defined (QTEXTSTREAM_DEBUG)
     qDebug("QTextStreamPrivate::fillReadBuffer(), device->read(\"%s\", %d) == %d",
@@ -996,7 +1007,10 @@ QTextStream::QTextStream(FILE *fileHandle, OpenMode openMode)
            fileHandle, int(openMode));
 #endif
     QFile *file = new QFile;
-    file->open(fileHandle, openMode);
+    // Discarding the return value of open; even if it failed
+    // (and the file is not open), QTextStream still reports `Ok`
+    // for closed QIODevices, so there's nothing really to do here.
+    (void)file->open(fileHandle, openMode);
 
     Q_D(QTextStream);
     d->device = file;
@@ -1399,7 +1413,8 @@ QTextStream::RealNumberNotation QTextStream::realNumberNotation() const
 /*!
     Sets the precision of real numbers to \a precision. This value
     describes the number of fraction digits QTextStream should
-    write when generating real numbers.
+    write when generating real numbers (FixedNotation, ScientificNotation), or
+    the maximum number of significant digits (SmartNotation).
 
     The precision cannot be a negative value. The default value is 6.
 
@@ -1418,7 +1433,9 @@ void QTextStream::setRealNumberPrecision(int precision)
 
 /*!
     Returns the current real number precision, or the number of fraction
-    digits QTextStream will write when generating real numbers.
+    digits QTextStream will write when generating real numbers
+    (FixedNotation, ScientificNotation), or the maximum number of significant
+    digits (SmartNotation).
 
     \sa setRealNumberNotation(), realNumberNotation(), numberFlags(), integerBase()
 */
@@ -1682,7 +1699,7 @@ QTextStreamPrivate::NumberParsingStatus QTextStreamPrivate::getNumber(qulonglong
         int ndigits = 0;
         while (getChar(&dig)) {
             int n = dig.toLower().unicode();
-            if (n >= '0' && n <= '7') {
+            if (isOctalDigit(n)) {
                 val *= 8;
                 val += n - '0';
             } else {
@@ -1746,13 +1763,10 @@ QTextStreamPrivate::NumberParsingStatus QTextStreamPrivate::getNumber(qulonglong
         // Parse digits
         int ndigits = 0;
         while (getChar(&dig)) {
-            int n = dig.toLower().unicode();
-            if (n >= '0' && n <= '9') {
+            const int h = fromHex(dig.unicode());
+            if (h != -1) {
                 val <<= 4;
-                val += n - '0';
-            } else if (n >= 'a' && n <= 'f') {
-                val <<= 4;
-                val += 10 + (n - 'a');
+                val += h;
             } else {
                 ungetChar(dig);
                 break;
@@ -1908,13 +1922,13 @@ bool QTextStreamPrivate::getReal(double *f)
     // nan/+inf/-inf, so here we also check for uppercase and mixed
     // case versions.
     if (!qstricmp(buf, "nan") || !qstricmp(buf, "+nan") || !qstricmp(buf, "-nan")) {
-        *f = qQNaN();
+        *f = qt_qnan();
         return true;
     } else if (!qstricmp(buf, "+inf") || !qstricmp(buf, "inf")) {
-        *f = qInf();
+        *f = qt_inf();
         return true;
     } else if (!qstricmp(buf, "-inf")) {
-        *f = -qInf();
+        *f = -qt_inf();
         return true;
     }
     bool ok;
@@ -1946,7 +1960,7 @@ QTextStream &QTextStream::operator>>(QChar &c)
     \overload
 
     Reads a character from the stream and stores it in \a c. The
-    character from the stream is converted to ISO-5589-1 before it is
+    character from the stream is converted to ISO-8859-1 before it is
     stored.
 
     \sa QChar::toLatin1()
@@ -2819,7 +2833,7 @@ QTextStream &center(QTextStream &stream)
     \snippet code/src_corelib_io_qtextstream.cpp 9
 
     Note: On Windows, all '\\n' characters are written as '\\r\\n' if
-    QTextStream's device or string is opened using the QIODevice::Text flag.
+    QTextStream's device or string is opened using the \l QIODeviceBase::Text flag.
 
     \since 5.14
 
