@@ -6,9 +6,14 @@
 #include "qdeadlinetimer.h"
 #include "qcoreapplication.h"
 
+#include <QtCore/qpointer.h>
+
 #include <algorithm>
+#include <climits> // For INT_MAX
 #include <memory>
 
+using namespace std::chrono_literals;
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -87,7 +92,7 @@ void QThreadPoolThread::run()
             if (manager->queue.isEmpty())
                 break;
 
-            QueuePage *page = manager->queue.first();
+            QueuePage *page = manager->queue.constFirst();
             r = page->pop();
 
             if (page->isFinished()) {
@@ -210,7 +215,7 @@ void QThreadPoolPrivate::tryToStartMoreThreads()
 {
     // try to push tasks on the queue to any available threads
     while (!queue.isEmpty()) {
-        QueuePage *page = queue.first();
+        QueuePage *page = queue.constFirst();
         if (!tryStart(page->first()))
             break;
 
@@ -256,7 +261,7 @@ void QThreadPoolPrivate::startThread(QRunnable *runnable)
 /*!
     \internal
 
-    Helper function only to be called from waitForDone(int)
+    Helper function only to be called from waitForDone()
 
     Deletes all current threads.
 */
@@ -283,22 +288,17 @@ void QThreadPoolPrivate::reset()
 /*!
     \internal
 
-    Helper function only to be called from waitForDone(int)
+    Helper function only to be called from the public waitForDone()
 */
 bool QThreadPoolPrivate::waitForDone(const QDeadlineTimer &timer)
 {
+    QMutexLocker locker(&mutex);
     while (!(queue.isEmpty() && activeThreads == 0) && !timer.hasExpired())
         noActiveThreads.wait(&mutex, timer);
 
-    return queue.isEmpty() && activeThreads == 0;
-}
-
-bool QThreadPoolPrivate::waitForDone(int msecs)
-{
-    QMutexLocker locker(&mutex);
-    QDeadlineTimer timer(msecs);
-    if (!waitForDone(timer))
+    if (!queue.isEmpty() || activeThreads)
         return false;
+
     reset();
     // New jobs might have started during reset, but return anyway
     // as the active thread and task count did reach 0 once, and
@@ -474,6 +474,28 @@ QThreadPool *QThreadPool::globalInstance()
     return theInstance;
 }
 
+/*!
+    Returns the QThreadPool instance for Qt Gui.
+    \internal
+*/
+QThreadPool *QThreadPoolPrivate::qtGuiInstance()
+{
+    Q_CONSTINIT static QPointer<QThreadPool> guiInstance;
+    Q_CONSTINIT static QBasicMutex theMutex;
+    const static bool runtime_disable = qEnvironmentVariableIsSet("QT_NO_GUI_THREADPOOL");
+    if (runtime_disable)
+        return nullptr;
+    const QMutexLocker locker(&theMutex);
+    if (guiInstance.isNull() && !QCoreApplication::closingDown()) {
+        guiInstance = new QThreadPool();
+        // Limit max thread to avoid too many parallel threads.
+        // We are not optimized for much more than 4 or 8 threads.
+        if (guiInstance && guiInstance->maxThreadCount() > 4)
+            guiInstance->setMaxThreadCount(qBound(4, guiInstance->maxThreadCount() / 2, 8));
+    }
+    return guiInstance;
+}
+
 /*!
     Reserves a thread and uses it to run \a runnable, unless this thread will
     make the current thread count exceed maxThreadCount().  In that case,
@@ -502,20 +524,21 @@ void QThreadPool::start(QRunnable *runnable, int priority)
 }
 
 /*!
+    \fn template<typename Callable, QRunnable::if_callable<Callable>> void QThreadPool::start(Callable &&callableToRun, int priority)
     \overload
     \since 5.15
 
-    Reserves a thread and uses it to run \a functionToRun, unless this thread will
+    Reserves a thread and uses it to run \a callableToRun, unless this thread will
     make the current thread count exceed maxThreadCount().  In that case,
-    \a functionToRun is added to a run queue instead. The \a priority argument can
+    \a callableToRun is added to a run queue instead. The \a priority argument can
     be used to control the run queue's order of execution.
+
+    \note This function participates in overload resolution only if \c Callable
+    is a function or function object which can be called with zero arguments.
+
+    \note In Qt version prior to 6.6, this function took std::function<void()>,
+    and therefore couldn't handle move-only callables.
 */
-void QThreadPool::start(std::function<void()> functionToRun, int priority)
-{
-    if (!functionToRun)
-        return;
-    start(QRunnable::create(std::move(functionToRun)), priority);
-}
 
 /*!
     Attempts to reserve a thread to run \a runnable.
@@ -547,30 +570,21 @@ bool QThreadPool::tryStart(QRunnable *runnable)
 }
 
 /*!
+    \fn template<typename Callable, QRunnable::if_callable<Callable>> bool QThreadPool::tryStart(Callable &&callableToRun)
     \overload
     \since 5.15
-    Attempts to reserve a thread to run \a functionToRun.
+    Attempts to reserve a thread to run \a callableToRun.
 
     If no threads are available at the time of calling, then this function
-    does nothing and returns \c false.  Otherwise, \a functionToRun is run immediately
+    does nothing and returns \c false.  Otherwise, \a callableToRun is run immediately
     using one available thread and this function returns \c true.
+
+    \note This function participates in overload resolution only if \c Callable
+    is a function or function object which can be called with zero arguments.
+
+    \note In Qt version prior to 6.6, this function took std::function<void()>,
+    and therefore couldn't handle move-only callables.
 */
-bool QThreadPool::tryStart(std::function<void()> functionToRun)
-{
-    if (!functionToRun)
-        return false;
-
-    Q_D(QThreadPool);
-    QMutexLocker locker(&d->mutex);
-    if (!d->allThreads.isEmpty() && d->areAllThreadsActive())
-        return false;
-
-    QRunnable *runnable = QRunnable::create(std::move(functionToRun));
-    if (d->tryStart(runnable))
-        return true;
-    delete runnable;
-    return false;
-}
 
 /*! \property QThreadPool::expiryTimeout
     \brief the thread expiry timeout value in milliseconds.
@@ -589,16 +603,22 @@ bool QThreadPool::tryStart(std::function<void()> functionToRun)
 
 int QThreadPool::expiryTimeout() const
 {
+    using namespace std::chrono;
     Q_D(const QThreadPool);
-    return d->expiryTimeout;
+    QMutexLocker locker(&d->mutex);
+    if (d->expiryTimeout == decltype(d->expiryTimeout)::max())
+        return -1;
+    return duration_cast<milliseconds>(d->expiryTimeout).count();
 }
 
 void QThreadPool::setExpiryTimeout(int expiryTimeout)
 {
     Q_D(QThreadPool);
-    if (d->expiryTimeout == expiryTimeout)
-        return;
-    d->expiryTimeout = expiryTimeout;
+    QMutexLocker locker(&d->mutex);
+    if (expiryTimeout < 0)
+        d->expiryTimeout = decltype(d->expiryTimeout)::max();
+    else
+        d->expiryTimeout = expiryTimeout * 1ms;
 }
 
 /*! \property QThreadPool::maxThreadCount
@@ -616,6 +636,7 @@ void QThreadPool::setExpiryTimeout(int expiryTimeout)
 int QThreadPool::maxThreadCount() const
 {
     Q_D(const QThreadPool);
+    QMutexLocker locker(&d->mutex);
     return d->requestedMaxThreadCount;
 }
 
@@ -685,12 +706,14 @@ void QThreadPool::reserveThread()
 void QThreadPool::setStackSize(uint stackSize)
 {
     Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
     d->stackSize = stackSize;
 }
 
 uint QThreadPool::stackSize() const
 {
     Q_D(const QThreadPool);
+    QMutexLocker locker(&d->mutex);
     return d->stackSize;
 }
 
@@ -711,12 +734,14 @@ uint QThreadPool::stackSize() const
 void QThreadPool::setThreadPriority(QThread::Priority priority)
 {
     Q_D(QThreadPool);
+    QMutexLocker locker(&d->mutex);
     d->threadPriority = priority;
 }
 
 QThread::Priority QThreadPool::threadPriority() const
 {
     Q_D(const QThreadPool);
+    QMutexLocker locker(&d->mutex);
     return d->threadPriority;
 }
 
@@ -777,30 +802,39 @@ void QThreadPool::startOnReservedThread(QRunnable *runnable)
 }
 
 /*!
+    \fn template<typename Callable, QRunnable::if_callable<Callable>> void QThreadPool::startOnReservedThread(Callable &&callableToRun)
     \overload
     \since 6.3
 
     Releases a thread previously reserved with reserveThread() and uses it
-    to run \a functionToRun.
-*/
-void QThreadPool::startOnReservedThread(std::function<void()> functionToRun)
-{
-    if (!functionToRun)
-        return releaseThread();
+    to run \a callableToRun.
 
-    startOnReservedThread(QRunnable::create(std::move(functionToRun)));
-}
+    \note This function participates in overload resolution only if \c Callable
+    is a function or function object which can be called with zero arguments.
+
+    \note In Qt version prior to 6.6, this function took std::function<void()>,
+    and therefore couldn't handle move-only callables.
+*/
 
 /*!
+    \fn bool QThreadPool::waitForDone(int msecs)
     Waits up to \a msecs milliseconds for all threads to exit and removes all
     threads from the thread pool. Returns \c true if all threads were removed;
-    otherwise it returns \c false. If \a msecs is -1 (the default), the timeout
-    is ignored (waits for the last thread to exit).
+    otherwise it returns \c false. If \a msecs is -1, this function waits for
+    the last thread to exit.
 */
-bool QThreadPool::waitForDone(int msecs)
+
+/*!
+    \since 6.8
+
+    Waits until \a deadline expires for all threads to exit and removes all
+    threads from the thread pool. Returns \c true if all threads were removed;
+    otherwise it returns \c false.
+*/
+bool QThreadPool::waitForDone(QDeadlineTimer deadline)
 {
     Q_D(QThreadPool);
-    return d->waitForDone(msecs);
+    return d->waitForDone(deadline);
 }
 
 /*!
