@@ -38,6 +38,9 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
+Q_TRACE_PARAM_REPLACE(Qt::AspectRatioMode, int);
+Q_TRACE_PARAM_REPLACE(Qt::TransformationMode, int);
+
 // MSVC 19.28 does show spurious warning "C4723: potential divide by 0" for code that divides
 // by height() in release builds. Anyhow, all the code paths in this file are only executed
 // for valid QPixmap's, where height() cannot be 0. Therefore disable the warning.
@@ -286,7 +289,6 @@ QPixmap QPixmap::copy(const QRect &rect) const
 
 /*!
     \fn QPixmap::scroll(int dx, int dy, int x, int y, int width, int height, QRegion *exposed)
-    \since 4.6
 
     This convenience function is equivalent to calling QPixmap::scroll(\a dx,
     \a dy, QRect(\a x, \a y, \a width, \a height), \a exposed).
@@ -295,8 +297,6 @@ QPixmap QPixmap::copy(const QRect &rect) const
 */
 
 /*!
-    \since 4.6
-
     Scrolls the area \a rect of this pixmap by (\a dx, \a dy). The exposed
     region is left unchanged. You can optionally pass a pointer to an empty
     QRegion to get the region that is \a exposed by the scroll operation.
@@ -368,10 +368,7 @@ QPixmap &QPixmap::operator=(const QPixmap &pixmap)
 
 /*!
     \fn void QPixmap::swap(QPixmap &other)
-    \since 4.8
-
-    Swaps pixmap \a other with this pixmap. This operation is very
-    fast and never fails.
+    \memberswap{pixmap}
 */
 
 /*!
@@ -714,7 +711,7 @@ bool QPixmap::load(const QString &fileName, const char *format, Qt::ImageConvers
 
             QString key = "qt_pixmap"_L1
                     % info.absoluteFilePath()
-                    % HexString<uint>(info.lastModified().toSecsSinceEpoch())
+                    % HexString<uint>(info.lastModified(QTimeZone::UTC).toSecsSinceEpoch())
                     % HexString<quint64>(info.size())
                     % HexString<uint>(data ? data->pixelType() : QPlatformPixmap::PixmapType);
 
@@ -967,12 +964,7 @@ bool QPixmap::isDetached() const
     Passing 0 for \a flags sets all the default options. Returns \c true
     if the result is that this pixmap is not null.
 
-    Note: this function was part of Qt 3 support in Qt 4.6 and earlier.
-    It has been promoted to official API status in 4.7 to support updating
-    the pixmap's image without creating a new QPixmap as fromImage() would.
-
     \sa fromImage()
-    \since 4.7
 */
 bool QPixmap::convertFromImage(const QImage &image, Qt::ImageConversionFlags flags)
 {
@@ -1032,7 +1024,7 @@ bool QPixmap::convertFromImage(const QImage &image, Qt::ImageConversionFlags fla
     Transformations}
 
 */
-QPixmap QPixmap::scaled(const QSize& s, Qt::AspectRatioMode aspectMode, Qt::TransformationMode mode) const
+QPixmap Q_TRACE_INSTRUMENT(qtgui) QPixmap::scaled(const QSize& s, Qt::AspectRatioMode aspectMode, Qt::TransformationMode mode) const
 {
     if (isNull()) {
         qWarning("QPixmap::scaled: Pixmap is a null pixmap");
@@ -1070,7 +1062,7 @@ QPixmap QPixmap::scaled(const QSize& s, Qt::AspectRatioMode aspectMode, Qt::Tran
     \sa isNull(), {QPixmap#Pixmap Transformations}{Pixmap
     Transformations}
 */
-QPixmap QPixmap::scaledToWidth(int w, Qt::TransformationMode mode) const
+QPixmap Q_TRACE_INSTRUMENT(qtgui) QPixmap::scaledToWidth(int w, Qt::TransformationMode mode) const
 {
     if (isNull()) {
         qWarning("QPixmap::scaleWidth: Pixmap is a null pixmap");
@@ -1100,7 +1092,7 @@ QPixmap QPixmap::scaledToWidth(int w, Qt::TransformationMode mode) const
     \sa isNull(), {QPixmap#Pixmap Transformations}{Pixmap
     Transformations}
 */
-QPixmap QPixmap::scaledToHeight(int h, Qt::TransformationMode mode) const
+QPixmap Q_TRACE_INSTRUMENT(qtgui) QPixmap::scaledToHeight(int h, Qt::TransformationMode mode) const
 {
     if (isNull()) {
         qWarning("QPixmap::scaleHeight: Pixmap is a null pixmap");
@@ -1192,8 +1184,6 @@ QPixmap QPixmap::transformed(const QTransform &transform,
     there are several functions that enables transformation of the
     pixmap.
 
-    \tableofcontents
-
     \section1 Reading and Writing Image Files
 
     QPixmap provides several ways of reading an image file: The file
@@ -1277,8 +1267,9 @@ QPixmap QPixmap::transformed(const QTransform &transform,
     QPixmap using the fromImage(). If this is too expensive an
     operation, you can use QBitmap::fromImage() instead.
 
-    To convert a QPixmap to and from HICON you can use the QtWinExtras
-    functions QtWin::toHICON() and QtWin::fromHICON() respectively.
+    To convert a QPixmap to and from HICON you can use the
+    QImage::toHICON() and QImage::fromHICON() functions respectively
+    (after converting the QPixmap to a QImage, as explained above).
 
     \section1 Pixmap Transformations
 
