@@ -3,6 +3,7 @@
 
 #include "qbitmap.h"
 #include "qpixmap.h"
+#include <private/qpixmap_win_p.h>
 #include <qpa/qplatformpixmap.h>
 #include "qpixmap_raster_p.h"
 
@@ -163,7 +164,7 @@ static QImage copyImageData(const BITMAPINFOHEADER &header, const RGBQUAD *color
         Q_ASSERT(DWORD(image.sizeInBytes()) == header.biSizeImage);
         memcpy(image.bits(), data, header.biSizeImage);
         if (format == QImage::Format_RGB888)
-            image = image.rgbSwapped();
+            image = std::move(image).rgbSwapped();
         break;
     default:
         Q_UNREACHABLE();
@@ -206,7 +207,7 @@ static HBITMAP qt_createIconMask(QImage bm)
     return hbm;
 }
 
-Q_GUI_EXPORT HBITMAP qt_createIconMask(const QBitmap &bitmap)
+HBITMAP qt_createIconMask(const QBitmap &bitmap)
 {
     return qt_createIconMask(bitmap.toImage().convertToFormat(QImage::Format_Mono));
 }
@@ -224,7 +225,7 @@ static inline QImage::Format format32(int hbitmapFormat)
     return QImage::Format_ARGB32_Premultiplied;
 }
 
-Q_GUI_EXPORT HBITMAP qt_imageToWinHBITMAP(const QImage &imageIn, int hbitmapFormat = 0)
+HBITMAP qt_imageToWinHBITMAP(const QImage &imageIn, int hbitmapFormat)
 {
     if (imageIn.isNull())
         return nullptr;
@@ -304,6 +305,7 @@ Q_GUI_EXPORT HBITMAP qt_imageToWinHBITMAP(const QImage &imageIn, int hbitmapForm
         return nullptr;
     }
     if (!pixels) {
+        DeleteObject(bitmap);
         qErrnoWarning("%s, did not allocate pixel data", __FUNCTION__);
         return nullptr;
     }
@@ -323,7 +325,7 @@ Q_GUI_EXPORT HBITMAP qt_imageToWinHBITMAP(const QImage &imageIn, int hbitmapForm
     It is the caller's responsibility to free the \c HBITMAP data
     after use.
 
-    For usage with with standard GDI calls, such as \c BitBlt(), the image
+    For usage with standard GDI calls, such as \c BitBlt(), the image
     should have the format QImage::Format_RGB32.
 
     When using the resulting HBITMAP for the \c AlphaBlend() GDI function,
@@ -350,7 +352,7 @@ HBITMAP QImage::toHBITMAP() const
     return qt_imageToWinHBITMAP(*this);
 }
 
-Q_GUI_EXPORT HBITMAP qt_pixmapToWinHBITMAP(const QPixmap &p, int hbitmapFormat = 0)
+HBITMAP qt_pixmapToWinHBITMAP(const QPixmap &p, int hbitmapFormat)
 {
     if (p.isNull())
         return nullptr;
@@ -449,7 +451,7 @@ static QImage imageFromWinHBITMAP_GetDiBits(HBITMAP bitmap, bool forceQuads, int
     return copyImageData(info.bmiHeader, bmiColorTable256.bmiColors, data.data(), imageFormat);
 }
 
-Q_GUI_EXPORT QImage qt_imageFromWinHBITMAP(HBITMAP bitmap, int hbitmapFormat = 0)
+QImage qt_imageFromWinHBITMAP(HBITMAP bitmap, int hbitmapFormat)
 {
     QImage result = imageFromWinHBITMAP_DibSection(bitmap, hbitmapFormat);
     if (result.isNull())
@@ -481,7 +483,7 @@ QImage QImage::fromHBITMAP(HBITMAP hbitmap)
     return qt_imageFromWinHBITMAP(hbitmap);
 }
 
-Q_GUI_EXPORT QPixmap qt_pixmapFromWinHBITMAP(HBITMAP bitmap, int hbitmapFormat = 0)
+QPixmap qt_pixmapFromWinHBITMAP(HBITMAP bitmap, int hbitmapFormat)
 {
     return QPixmap::fromImage(imageFromWinHBITMAP_GetDiBits(bitmap, /* forceQuads */ true, hbitmapFormat));
 }
@@ -532,7 +534,7 @@ HICON QImage::toHICON(const QImage &mask) const
     return hIcon;
 }
 
-Q_GUI_EXPORT HICON qt_pixmapToWinHICON(const QPixmap &p)
+HICON qt_pixmapToWinHICON(const QPixmap &p)
 {
     QImage mask;
     QBitmap maskBitmap = p.mask();
@@ -541,7 +543,7 @@ Q_GUI_EXPORT HICON qt_pixmapToWinHICON(const QPixmap &p)
     return p.toImage().toHICON(mask);
 }
 
-Q_GUI_EXPORT QImage qt_imageFromWinHBITMAP(HDC hdc, HBITMAP bitmap, int w, int h)
+QImage qt_imageFromWinHBITMAP(HDC hdc, HBITMAP bitmap, int w, int h)
 {
     QImage image(w, h, QImage::Format_ARGB32_Premultiplied);
     if (image.isNull())
@@ -641,7 +643,7 @@ QImage QImage::fromHICON(HICON icon)
     return image;
 }
 
-Q_GUI_EXPORT QPixmap qt_pixmapFromWinHICON(HICON icon)
+QPixmap qt_pixmapFromWinHICON(HICON icon)
 {
     return QPixmap::fromImage(QImage::fromHICON(icon));
 }
