@@ -1,7 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
-#define Q_TEST_QPIXMAPCACHE
 #include "qpixmapcache.h"
 #include "qobject.h"
 #include "qdebug.h"
@@ -9,6 +8,8 @@
 #include "qthread.h"
 #include "qcoreapplication.h"
 
+using namespace std::chrono_literals;
+
 QT_BEGIN_NAMESPACE
 
 /*!
@@ -80,7 +81,6 @@ static inline bool qt_pixmapcache_thread_test()
     \brief The QPixmapCache::Key class can be used for efficient access
     to the QPixmapCache.
     \inmodule QtGui
-    \since 4.6
 
     Use QPixmapCache::insert() to receive an instance of Key generated
     by the pixmap cache. You can store the key in your own objects for
@@ -144,8 +144,8 @@ bool QPixmapCache::Key::operator ==(const Key &key) const
 
 /*!
     \fn void QPixmapCache::Key::swap(Key &)
-    \internal
     \since 5.6
+    \memberswap{key}
 */
 
 /*!
@@ -183,7 +183,6 @@ public:
     void timerEvent(QTimerEvent *) override;
     bool insert(const QString& key, const QPixmap &pixmap, int cost);
     QPixmapCache::Key insert(const QPixmap &pixmap, int cost);
-    bool replace(const QPixmapCache::Key &key, const QPixmap &pixmap, int cost);
     bool remove(const QString &key);
     bool remove(const QPixmapCache::Key &key);
 
@@ -203,7 +202,8 @@ public:
     bool flushDetachedPixmaps(bool nt);
 
 private:
-    enum { soon_time = 10000, flush_time = 30000 };
+    static constexpr auto soon_time = 10s;
+    static constexpr auto flush_time = 30s;
     int *keyArray;
     int theid;
     int ps;
@@ -217,10 +217,14 @@ QT_BEGIN_INCLUDE_NAMESPACE
 #include "qpixmapcache.moc"
 QT_END_INCLUDE_NAMESPACE
 
-size_t qHash(const QPixmapCache::Key &k, size_t seed)
+/*!
+    \fn size_t QPixmapCache::Key::qHash(const Key &key, size_t seed)
+    \since 6.6
+    \qhash{QPixmapCache::Key}
+*/
+size_t QPixmapCache::Key::hash(size_t seed) const noexcept
 {
-    const auto *keyData = QPMCache::get(k);
-    return qHash(keyData ? keyData->key : 0, seed);
+    return qHash(this->d ? this->d->key : 0, seed);
 }
 
 QPMCache::QPMCache()
@@ -253,25 +257,12 @@ bool QPMCache::flushDetachedPixmaps(bool nt)
 {
     auto mc = maxCost();
     const qsizetype currentTotal = totalCost();
+    const qsizetype oldSize = size();
     if (currentTotal)
         setMaxCost(nt ? currentTotal * 3 / 4 : currentTotal - 1);
     setMaxCost(mc);
     ps = totalCost();
-
-    bool any = false;
-    QHash<QString, QPixmapCache::Key>::iterator it = cacheKeys.begin();
-    while (it != cacheKeys.end()) {
-        const auto value = it.value();
-        if (value.isValid() && !contains(value)) {
-            releaseKey(value);
-            it = cacheKeys.erase(it);
-            any = true;
-        } else {
-            ++it;
-        }
-    }
-
-    return any;
+    return size() != oldSize;
 }
 
 void QPMCache::timerEvent(QTimerEvent *)
@@ -290,17 +281,9 @@ void QPMCache::timerEvent(QTimerEvent *)
 
 QPixmap *QPMCache::object(const QString &key) const
 {
-    QPixmapCache::Key cacheKey = cacheKeys.value(key);
-    if (!cacheKey.d || !cacheKey.d->isValid) {
-        const_cast<QPMCache *>(this)->cacheKeys.remove(key);
-        return nullptr;
-    }
-    QPixmap *ptr = QCache<QPixmapCache::Key, QPixmapCacheEntry>::object(cacheKey);
-     //We didn't find the pixmap in the cache, the key is not valid anymore
-    if (!ptr) {
-        const_cast<QPMCache *>(this)->cacheKeys.remove(key);
-    }
-    return ptr;
+    if (const auto it = cacheKeys.find(key); it != cacheKeys.cend())
+        return object(it.value());
+    return nullptr;
 }
 
 QPixmap *QPMCache::object(const QPixmapCache::Key &key) const
@@ -315,31 +298,24 @@ QPixmap *QPMCache::object(const QPixmapCache::Key &key) const
 
 bool QPMCache::insert(const QString& key, const QPixmap &pixmap, int cost)
 {
-    QPixmapCache::Key &cacheKey = cacheKeys[key];
     //If for the same key we add already a pixmap we should delete it
-    if (cacheKey.d)
-        QCache<QPixmapCache::Key, QPixmapCacheEntry>::remove(cacheKey);
+    remove(key);
 
-    //we create a new key the old one has been removed
-    cacheKey = createKey();
-
-    bool success = QCache<QPixmapCache::Key, QPixmapCacheEntry>::insert(cacheKey, new QPixmapCacheEntry(cacheKey, pixmap), cost);
-    if (success) {
-        if (!theid) {
-            theid = startTimer(flush_time);
-            t = false;
-        }
-    } else {
-        //Insertion failed we released the new allocated key
-        cacheKeys.remove(key);
+    // this will create a new key; the old one has been removed
+    auto k = insert(pixmap, cost);
+    if (k.isValid()) {
+        k.d->stringKey = key;
+        cacheKeys[key] = std::move(k);
+        return true;
     }
-    return success;
+    return false;
 }
 
 QPixmapCache::Key QPMCache::insert(const QPixmap &pixmap, int cost)
 {
-    QPixmapCache::Key cacheKey = createKey();
+    QPixmapCache::Key cacheKey = createKey(); // invalidated by ~QPixmapCacheEntry on failed insert
     bool success = QCache<QPixmapCache::Key, QPixmapCacheEntry>::insert(cacheKey, new QPixmapCacheEntry(cacheKey, pixmap), cost);
+    Q_ASSERT(success || !cacheKey.isValid());
     if (success) {
         if (!theid) {
             theid = startTimer(flush_time);
@@ -349,34 +325,10 @@ QPixmapCache::Key QPMCache::insert(const QPixmap &pixmap, int cost)
     return cacheKey;
 }
 
-bool QPMCache::replace(const QPixmapCache::Key &key, const QPixmap &pixmap, int cost)
-{
-    Q_ASSERT(key.isValid());
-    //If for the same key we had already an entry so we should delete the pixmap and use the new one
-    QCache<QPixmapCache::Key, QPixmapCacheEntry>::remove(key);
-
-    QPixmapCache::Key cacheKey = createKey();
-
-    bool success = QCache<QPixmapCache::Key, QPixmapCacheEntry>::insert(cacheKey, new QPixmapCacheEntry(cacheKey, pixmap), cost);
-    if (success) {
-        if (!theid) {
-            theid = startTimer(flush_time);
-            t = false;
-        }
-        const_cast<QPixmapCache::Key&>(key) = cacheKey;
-    }
-    return success;
-}
-
 bool QPMCache::remove(const QString &key)
 {
-    auto cacheKey = cacheKeys.constFind(key);
-    //The key was not in the cache
-    if (cacheKey == cacheKeys.constEnd())
-        return false;
-    const bool result = QCache<QPixmapCache::Key, QPixmapCacheEntry>::remove(cacheKey.value());
-    cacheKeys.erase(cacheKey);
-    return result;
+    const auto cacheKey = cacheKeys.take(key);
+    return cacheKey.isValid() && remove(cacheKey);
 }
 
 bool QPMCache::remove(const QPixmapCache::Key &key)
@@ -410,7 +362,11 @@ QPixmapCache::Key QPMCache::createKey()
 void QPMCache::releaseKey(const QPixmapCache::Key &key)
 {
     QPixmapCache::KeyData *keyData = key.d;
-    if (!keyData || keyData->key > keyArraySize || keyData->key <= 0)
+    if (!keyData)
+        return;
+    if (!keyData->stringKey.isNull())
+        cacheKeys.remove(keyData->stringKey);
+    if (keyData->key > keyArraySize || keyData->key <= 0)
         return;
     keyData->key--;
     keyArray[keyData->key] = freeKey;
@@ -463,15 +419,13 @@ QPixmapCacheEntry::~QPixmapCacheEntry()
     If the pixmap is found, the function sets \a pixmap to that pixmap and
     returns \c true; otherwise it leaves \a pixmap alone and returns \c false.
 
-    \since 4.6
-
     Example:
     \snippet code/src_gui_image_qpixmapcache.cpp 1
 */
 
 bool QPixmapCache::find(const QString &key, QPixmap *pixmap)
 {
-    if (!qt_pixmapcache_thread_test())
+    if (key.isEmpty() || !qt_pixmapcache_thread_test())
         return false;
     QPixmap *ptr = pm_cache()->object(key);
     if (ptr && pixmap)
@@ -485,8 +439,6 @@ bool QPixmapCache::find(const QString &key, QPixmap *pixmap)
     returns \c true; otherwise it leaves \a pixmap alone and returns \c false. If
     the pixmap is not found, it means that the \a key is no longer valid,
     so it will be released for the next insertion.
-
-    \since 4.6
 */
 bool QPixmapCache::find(const Key &key, QPixmap *pixmap)
 {
@@ -523,7 +475,7 @@ bool QPixmapCache::find(const Key &key, QPixmap *pixmap)
 
 bool QPixmapCache::insert(const QString &key, const QPixmap &pixmap)
 {
-    if (!qt_pixmapcache_thread_test())
+    if (key.isEmpty() || !qt_pixmapcache_thread_test())
         return false;
     return pm_cache()->insert(key, pixmap, cost(pixmap));
 }
@@ -540,8 +492,6 @@ bool QPixmapCache::insert(const QString &key, const QPixmap &pixmap)
     deleted when more space is needed.
 
     \sa setCacheLimit(), replace()
-
-    \since 4.6
 */
 QPixmapCache::Key QPixmapCache::insert(const QPixmap &pixmap)
 {
@@ -550,24 +500,23 @@ QPixmapCache::Key QPixmapCache::insert(const QPixmap &pixmap)
     return pm_cache()->insert(pixmap, cost(pixmap));
 }
 
+#if QT_DEPRECATED_SINCE(6, 6)
 /*!
+    \fn bool QPixmapCache::replace(const Key &key, const QPixmap &pixmap)
+
+    \deprecated [6.6] Use \c{remove(key); key = insert(pixmap);} instead.
+
     Replaces the pixmap associated with the given \a key with the \a pixmap
     specified. Returns \c true if the \a pixmap has been correctly inserted into
     the cache; otherwise returns \c false.
 
-    \sa setCacheLimit(), insert()
+    The passed \a key is updated to reference \a pixmap now. Other copies of \a
+    key, if any, still refer to the old pixmap, which is, however, removed from
+    the cache by this function.
 
-    \since 4.6
+    \sa setCacheLimit(), insert()
 */
-bool QPixmapCache::replace(const Key &key, const QPixmap &pixmap)
-{
-    if (!qt_pixmapcache_thread_test())
-        return false;
-    //The key is not valid anymore, a flush happened before probably
-    if (!key.d || !key.d->isValid)
-        return false;
-    return pm_cache()->replace(key, pixmap, cost(pixmap));
-}
+#endif // QT_DEPRECATED_SINCE(6, 6)
 
 /*!
     Returns the cache limit (in kilobytes).
@@ -604,7 +553,7 @@ void QPixmapCache::setCacheLimit(int n)
 */
 void QPixmapCache::remove(const QString &key)
 {
-    if (!qt_pixmapcache_thread_test())
+    if (key.isEmpty() || !qt_pixmapcache_thread_test())
         return;
     pm_cache()->remove(key);
 }
@@ -612,8 +561,6 @@ void QPixmapCache::remove(const QString &key)
 /*!
   Removes the pixmap associated with \a key from the cache and releases
   the key for a future insertion.
-
-  \since 4.6
 */
 void QPixmapCache::remove(const Key &key)
 {
@@ -642,14 +589,14 @@ void QPixmapCache::clear()
     }
 }
 
-void QPixmapCache::flushDetachedPixmaps()
+Q_AUTOTEST_EXPORT void qt_qpixmapcache_flush_detached_pixmaps() // for tst_qpixmapcache
 {
     if (!qt_pixmapcache_thread_test())
         return;
     pm_cache()->flushDetachedPixmaps(true);
 }
 
-int QPixmapCache::totalUsed()
+Q_AUTOTEST_EXPORT int qt_qpixmapcache_qpixmapcache_total_used() // for tst_qpixmapcache
 {
     if (!qt_pixmapcache_thread_test())
         return 0;
