@@ -4,13 +4,16 @@
 #ifndef QEVENT_H
 #define QEVENT_H
 
+#if 0
+#pragma qt_class(QtEvents)
+#endif
+
 #include <QtGui/qtguiglobal.h>
 
 #include <QtCore/qcoreevent.h>
 #include <QtCore/qiodevice.h>
 #include <QtCore/qlist.h>
 #include <QtCore/qnamespace.h>
-#include <QtCore/qpointer.h>
 #include <QtCore/qstring.h>
 #include <QtCore/qurl.h>
 #include <QtCore/qvariant.h>
@@ -30,8 +33,12 @@ QT_BEGIN_NAMESPACE
 class QFile;
 class QAction;
 class QMouseEvent;
+template <typename T> class QPointer;
 class QPointerEvent;
 class QScreen;
+#if QT_CONFIG(shortcut)
+class QShortcut;
+#endif
 class QTabletEvent;
 class QTouchEvent;
 #if QT_CONFIG(gestures)
@@ -64,6 +71,7 @@ protected:
 
 class Q_GUI_EXPORT QPointerEvent : public QInputEvent
 {
+    Q_GADGET
     Q_DECL_EVENT_COMMON(QPointerEvent)
 public:
     explicit QPointerEvent(Type type, const QPointingDevice *dev,
@@ -126,6 +134,7 @@ public:
 
 protected:
     friend class ::tst_QEvent;
+    friend class QMutableSinglePointEvent;
     QSinglePointEvent(Type type, const QPointingDevice *dev, const QEventPoint &point,
                       Qt::MouseButton button, Qt::MouseButtons buttons,
                       Qt::KeyboardModifiers modifiers, Qt::MouseEventSource source);
@@ -845,7 +854,10 @@ public:
 
     inline QString file() const { return m_file; }
     QUrl url() const { return m_url; }
+#if QT_DEPRECATED_SINCE(6, 6)
+    QT_DEPRECATED_VERSION_X_6_6("Interpret the string returned by file()")
     bool openFile(QFile &file, QIODevice::OpenMode flags) const;
+#endif
 private:
     QString m_file;
     QUrl m_url;
@@ -869,9 +881,12 @@ class Q_GUI_EXPORT QShortcutEvent : public QEvent
 {
     Q_DECL_EVENT_COMMON(QShortcutEvent)
 public:
+    // Note this is publicly deprecated, but should remain as internal constructor:
     QShortcutEvent(const QKeySequence &key, int id, bool ambiguous = false);
+    QShortcutEvent(const QKeySequence &key, const QShortcut *shortcut = nullptr, bool ambiguous = false);
 
     inline const QKeySequence &key() const { return m_sequence; }
+    // Note this is publicly deprecated, but should remain as internal getter:
     inline int shortcutId() const { return m_shortcutId; }
     inline bool isAmbiguous() const { return m_ambiguous; }
 protected:
@@ -929,6 +944,7 @@ public:
     bool isEndEvent() const override;
 
 protected:
+    friend class QMutableTouchEvent;
     QObject *m_target = nullptr;
     QEventPoint::States m_touchPointStates = QEventPoint::State::Unknown;
     quint32 m_reserved : 24;
@@ -1007,6 +1023,17 @@ private:
     Qt::ApplicationState m_applicationState;
 };
 
+class Q_GUI_EXPORT QChildWindowEvent : public QEvent
+{
+    Q_DECL_EVENT_COMMON(QChildWindowEvent)
+public:
+    explicit QChildWindowEvent(Type type, QWindow *childWindow);
+    QWindow *child() const { return c; }
+
+private:
+    QWindow *c;
+};
+
 QT_END_NAMESPACE
 
 #endif // QEVENT_H
