@@ -64,31 +64,6 @@ public:
                                  Qt::KeyboardModifiers mods = Qt::NoModifier,
                                  Qt::MouseEventSource source = Qt::MouseEventNotSynthesized);
 
-    static bool handleFrameStrutMouseEvent(QWindow *window, const QPointF &local,
-                                           const QPointF &global, Qt::MouseButtons state,
-                                           Qt::MouseButton button, QEvent::Type type,
-                                           Qt::KeyboardModifiers mods = Qt::NoModifier,
-                                           Qt::MouseEventSource source =
-                                           Qt::MouseEventNotSynthesized);
-    static bool handleFrameStrutMouseEvent(QWindow *window, const QPointingDevice *device,
-                                           const QPointF &local, const QPointF &global,
-                                           Qt::MouseButtons state,
-                                           Qt::MouseButton button, QEvent::Type type,
-                                           Qt::KeyboardModifiers mods = Qt::NoModifier,
-                                           Qt::MouseEventSource source =
-                                           Qt::MouseEventNotSynthesized);
-    static bool handleFrameStrutMouseEvent(QWindow *window, ulong timestamp, const QPointF &local,
-                                           const QPointF &global, Qt::MouseButtons state,
-                                           Qt::MouseButton button, QEvent::Type type,
-                                           Qt::KeyboardModifiers mods = Qt::NoModifier,
-                                           Qt::MouseEventSource source =
-                                           Qt::MouseEventNotSynthesized);
-    static bool handleFrameStrutMouseEvent(QWindow *window, ulong timestamp, const QPointingDevice *device,
-                                           const QPointF &local, const QPointF &global, Qt::MouseButtons state,
-                                           Qt::MouseButton button, QEvent::Type type,
-                                           Qt::KeyboardModifiers mods = Qt::NoModifier,
-                                           Qt::MouseEventSource source = Qt::MouseEventNotSynthesized);
-
     static bool handleShortcutEvent(QWindow *window, ulong timestamp, int k, Qt::KeyboardModifiers mods, quint32 nativeScanCode,
                                       quint32 nativeVirtualKey, quint32 nativeModifiers, const QString & text = QString(), bool autorep = false, ushort count = 1);
 
@@ -101,18 +76,18 @@ public:
                                        quint32 nativeScanCode, quint32 nativeVirtualKey,
                                        quint32 nativeModifiers,
                                        const QString& text = QString(), bool autorep = false,
-                                       ushort count = 1, bool tryShortcutOverride = true);
+                                       ushort count = 1);
     static bool handleExtendedKeyEvent(QWindow *window, ulong timestamp, QEvent::Type type, int key, Qt::KeyboardModifiers modifiers,
                                        quint32 nativeScanCode, quint32 nativeVirtualKey,
                                        quint32 nativeModifiers,
                                        const QString& text = QString(), bool autorep = false,
-                                       ushort count = 1, bool tryShortcutOverride = true);
+                                       ushort count = 1);
     static bool handleExtendedKeyEvent(QWindow *window, ulong timestamp, const QInputDevice *device,
                                        QEvent::Type type, int key, Qt::KeyboardModifiers modifiers,
                                        quint32 nativeScanCode, quint32 nativeVirtualKey,
                                        quint32 nativeModifiers,
                                        const QString& text = QString(), bool autorep = false,
-                                       ushort count = 1, bool tryShortcutOverride = true);
+                                       ushort count = 1);
     static bool handleWheelEvent(QWindow *window, const QPointF &local, const QPointF &global,
                                  QPoint pixelDelta, QPoint angleDelta,
                                  Qt::KeyboardModifiers mods = Qt::NoModifier,
@@ -182,14 +157,18 @@ public:
     template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
     static void handleLeaveEvent(QWindow *window);
     static void handleEnterLeaveEvent(QWindow *enter, QWindow *leave, const QPointF &local = QPointF(), const QPointF& global = QPointF());
+
     template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
-    static void handleWindowActivated(QWindow *window, Qt::FocusReason r = Qt::OtherFocusReason);
+    static void handleFocusWindowChanged(QWindow *window, Qt::FocusReason r = Qt::OtherFocusReason);
 
     template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
     static void handleWindowStateChanged(QWindow *window, Qt::WindowStates newState, int oldState = -1);
     template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
     static void handleWindowScreenChanged(QWindow *window, QScreen *newScreen);
 
+    template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
+    static void handleWindowDevicePixelRatioChanged(QWindow *window);
+
     template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
     static void handleSafeAreaMarginsChanged(QWindow *window);
 
@@ -252,6 +231,8 @@ public:
                                                      Qt::MouseButtons buttons = {}, int xTilt = 0, int yTilt = 0,
                                                      qreal tangentialPressure = 0, qreal rotation = 0, int z = 0,
                                                      Qt::KeyboardModifiers modifiers = Qt::NoModifier);
+
+    // The following 4 functions are deprecated (QTBUG-114560)
     static bool handleTabletEnterProximityEvent(ulong timestamp, int deviceType, int pointerType, qint64 uid);
     static void handleTabletEnterProximityEvent(int deviceType, int pointerType, qint64 uid);
     static bool handleTabletLeaveProximityEvent(ulong timestamp, int deviceType, int pointerType, qint64 uid);
@@ -267,10 +248,17 @@ public:
 #endif // QT_NO_GESTURES
 
     static void handlePlatformPanelEvent(QWindow *window);
+
 #ifndef QT_NO_CONTEXTMENU
+#if QT_GUI_REMOVED_SINCE(6, 8)
     static void handleContextMenuEvent(QWindow *window, bool mouseTriggered,
                                        const QPoint &pos, const QPoint &globalPos,
                                        Qt::KeyboardModifiers modifiers);
+#endif
+    template<typename Delivery = QWindowSystemInterface::DefaultDelivery>
+    static bool handleContextMenuEvent(QWindow *window, bool mouseTriggered,
+                                       const QPoint &pos, const QPoint &globalPos,
+                                       Qt::KeyboardModifiers modifiers);
 #endif
 #if QT_CONFIG(whatsthis)
     static void handleEnterWhatsThisEvent();
