@@ -162,6 +162,9 @@ QT_BEGIN_NAMESPACE
 
 QPixmap qt_mac_toQPixmap(const NSImage *image, const QSizeF &size)
 {
+    // ### TODO: add parameter so that we can decide whether to maintain the aspect
+    // ratio of the image (positioning the image inside the pixmap of size \a size),
+    // or whether we want to fill the resulting pixmap by stretching the image.
     const NSSize pixmapSize = NSMakeSize(size.width(), size.height());
     QPixmap pixmap(pixmapSize.width, pixmapSize.height);
     pixmap.fill(Qt::transparent);
@@ -182,6 +185,25 @@ QPixmap qt_mac_toQPixmap(const NSImage *image, const QSizeF &size)
 
 #endif // Q_OS_MACOS
 
+#ifdef QT_PLATFORM_UIKIT
+
+QImage qt_mac_toQImage(const UIImage *image, QSizeF size)
+{
+    // ### TODO: same as above
+    QImage ret(size.width(), size.height(), QImage::Format_ARGB32_Premultiplied);
+    ret.fill(Qt::transparent);
+    QMacCGContext ctx(&ret);
+    if (!ctx)
+        return QImage();
+    UIGraphicsPushContext(ctx);
+    const CGRect rect = CGRectMake(0, 0, size.width(), size.height());
+    [image drawInRect:rect];
+    UIGraphicsPopContext();
+    return ret;
+}
+
+#endif // QT_PLATFORM_UIKIT
+
 // ---------------------- Colors and Brushes ----------------------
 
 QColor qt_mac_toQColor(CGColorRef color)
@@ -206,6 +228,9 @@ QColor qt_mac_toQColor(CGColorRef color)
 #ifdef Q_OS_MACOS
 QColor qt_mac_toQColor(const NSColor *color)
 {
+    if (!color)
+        return QColor();
+
     QColor qtColor;
     switch (color.type) {
     case NSColorTypeComponentBased: {
