@@ -295,6 +295,21 @@ static int QT_WIN_CALLBACK storeFont(const LOGFONT *logFont, const TEXTMETRIC *t
     return 1;
 }
 
+bool QWindowsFontDatabaseFT::populateFamilyAliases(const QString &missingFamily)
+{
+    Q_UNUSED(missingFamily);
+
+    if (m_hasPopulatedAliases)
+        return false;
+
+    QStringList families = QFontDatabase::families();
+    for (const QString &family : families)
+        populateFamily(family);
+    m_hasPopulatedAliases = true;
+
+    return true;
+}
+
 /*
     \brief Populates the font database using EnumFontFamiliesEx().
 
@@ -363,7 +378,7 @@ void QWindowsFontDatabaseFT::populateFontDatabase()
     EnumFontFamiliesEx(dummy, &lf, populateFontFamilies, 0, 0);
     ReleaseDC(0, dummy);
     // Work around EnumFontFamiliesEx() not listing the system font
-    const QString systemDefaultFamily = QWindowsFontDatabase::systemDefaultFont().families().first();
+    const QString systemDefaultFamily = QWindowsFontDatabase::systemDefaultFont().families().constFirst();
     if (QPlatformFontDatabase::resolveFontFamilyAlias(systemDefaultFamily) == systemDefaultFamily)
         QPlatformFontDatabase::registerFontFamily(systemDefaultFamily);
 }
@@ -371,7 +386,7 @@ void QWindowsFontDatabaseFT::populateFontDatabase()
 QFontEngine * QWindowsFontDatabaseFT::fontEngine(const QFontDef &fontDef, void *handle)
 {
     QFontEngine *fe = QFreeTypeFontDatabase::fontEngine(fontDef, handle);
-    qCDebug(lcQpaFonts) << __FUNCTION__ << "FONTDEF" << fontDef.families.first() << fe << handle;
+    qCDebug(lcQpaFonts) << __FUNCTION__ << "FONTDEF" << fontDef.families.constFirst() << fe << handle;
     return fe;
 }
 
