@@ -12,10 +12,14 @@
 #include <QtCore/private/qwinregistry_p.h>
 #include <QtGui/private/qguiapplication_p.h>
 #include <qpa/qplatformintegration.h>
-#include <QtGui/private/qhighdpiscaling_p.h>
 #include <QtGui/qpainterpath.h>
 
-#include <dwrite_2.h>
+#if QT_CONFIG(directwrite3)
+#  include "qwindowsdirectwritefontdatabase_p.h"
+#  include <dwrite_3.h>
+#else
+#  include <dwrite_2.h>
+#endif
 
 #include <d2d1.h>
 
@@ -158,10 +162,13 @@ static DWRITE_MEASURING_MODE renderModeToMeasureMode(DWRITE_RENDERING_MODE rende
     }
 }
 
-static DWRITE_RENDERING_MODE hintingPreferenceToRenderingMode(const QFontDef &fontDef)
+DWRITE_RENDERING_MODE QWindowsFontEngineDirectWrite::hintingPreferenceToRenderingMode(const QFontDef &fontDef) const
 {
+    if ((fontDef.styleStrategy & QFont::NoAntialias) && glyphFormat != QFontEngine::Format_ARGB)
+        return DWRITE_RENDERING_MODE_ALIASED;
+
     QFont::HintingPreference hintingPreference = QFont::HintingPreference(fontDef.hintingPreference);
-    if (QHighDpiScaling::isActive() && hintingPreference == QFont::PreferDefaultHinting) {
+    if (!qFuzzyCompare(qApp->devicePixelRatio(), 1.0) && hintingPreference == QFont::PreferDefaultHinting) {
         // Microsoft documentation recommends using asymmetric rendering for small fonts
         // at pixel size 16 and less, and symmetric for larger fonts.
         hintingPreference = fontDef.pixelSize > 16.0
@@ -210,6 +217,12 @@ QWindowsFontEngineDirectWrite::QWindowsFontEngineDirectWrite(IDWriteFontFace *di
     m_fontEngineData->directWriteFactory->AddRef();
     m_directWriteFontFace->AddRef();
 
+    IDWriteRenderingParams *renderingParams = nullptr;
+    if (SUCCEEDED(m_fontEngineData->directWriteFactory->CreateRenderingParams(&renderingParams))) {
+        m_pixelGeometry = renderingParams->GetPixelGeometry();
+        renderingParams->Release();
+    }
+
     fontDef.pixelSize = pixelSize;
     collectMetrics();
     cache_cost = m_xHeight.toInt() * m_xHeight.toInt() * 2000;
@@ -354,12 +367,14 @@ void QWindowsFontEngineDirectWrite::collectMetrics()
         fontFile->Release();
     }
 
-    QByteArray table = getSfntTable(MAKE_TAG('h', 'h', 'e', 'a'));
+    QByteArray table = getSfntTable(QFont::Tag("hhea").value());
     const int advanceWidthMaxLocation = 10;
     if (table.size() >= advanceWidthMaxLocation + int(sizeof(quint16))) {
         quint16 advanceWidthMax = qFromBigEndian<quint16>(table.constData() + advanceWidthMaxLocation);
         m_maxAdvanceWidth = DESIGN_TO_LOGICAL(advanceWidthMax);
     }
+
+    loadKerningPairs(emSquareSize() / QFixed::fromReal(fontDef.pixelSize));
 }
 
 QFixed QWindowsFontEngineDirectWrite::underlinePosition() const
@@ -426,13 +441,13 @@ glyph_t QWindowsFontEngineDirectWrite::glyphIndex(uint ucs4) const
     return glyphIndex;
 }
 
-bool QWindowsFontEngineDirectWrite::stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs,
-                                                 int *nglyphs, QFontEngine::ShaperFlags flags) const
+int QWindowsFontEngineDirectWrite::stringToCMap(const QChar *str, int len, QGlyphLayout *glyphs,
+                                                int *nglyphs, QFontEngine::ShaperFlags flags) const
 {
     Q_ASSERT(glyphs->numGlyphs >= *nglyphs);
     if (*nglyphs < len) {
         *nglyphs = len;
-        return false;
+        return -1;
     }
 
     QVarLengthArray<UINT32> codePoints(len);
@@ -446,11 +461,15 @@ bool QWindowsFontEngineDirectWrite::stringToCMap(const QChar *str, int len, QGly
                                                          glyphIndices.data());
     if (FAILED(hr)) {
         qErrnoWarning("%s: GetGlyphIndicesW failed", __FUNCTION__);
-        return false;
+        return -1;
     }
 
-    for (int i = 0; i < actualLength; ++i)
+    int mappedGlyphs = 0;
+    for (int i = 0; i < actualLength; ++i) {
         glyphs->glyphs[i] = glyphIndices.at(i);
+        if (glyphs->glyphs[i] != 0 || isIgnorableChar(codePoints.at(i)))
+            mappedGlyphs++;
+    }
 
     *nglyphs = actualLength;
     glyphs->numGlyphs = actualLength;
@@ -458,7 +477,7 @@ bool QWindowsFontEngineDirectWrite::stringToCMap(const QChar *str, int len, QGly
     if (!(flags & GlyphIndicesOnly))
         recalcAdvances(glyphs, {});
 
-    return true;
+    return mappedGlyphs;
 }
 
 QFontEngine::FaceId QWindowsFontEngineDirectWrite::faceId() const
@@ -466,7 +485,7 @@ QFontEngine::FaceId QWindowsFontEngineDirectWrite::faceId() const
     return m_faceId;
 }
 
-void QWindowsFontEngineDirectWrite::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::ShaperFlags) const
+void QWindowsFontEngineDirectWrite::recalcAdvances(QGlyphLayout *glyphs, QFontEngine::ShaperFlags shaperFlags) const
 {
     QVarLengthArray<UINT16> glyphIndices(glyphs->numGlyphs);
 
@@ -478,11 +497,14 @@ void QWindowsFontEngineDirectWrite::recalcAdvances(QGlyphLayout *glyphs, QFontEn
 
     HRESULT hr;
     DWRITE_RENDERING_MODE renderMode = hintingPreferenceToRenderingMode(fontDef);
-    if (renderMode == DWRITE_RENDERING_MODE_GDI_CLASSIC || renderMode == DWRITE_RENDERING_MODE_GDI_NATURAL) {
+    bool needsDesignMetrics = shaperFlags & QFontEngine::DesignMetrics;
+    if (!needsDesignMetrics && (renderMode == DWRITE_RENDERING_MODE_GDI_CLASSIC
+                             || renderMode == DWRITE_RENDERING_MODE_GDI_NATURAL
+                             || renderMode == DWRITE_RENDERING_MODE_ALIASED)) {
         hr = m_directWriteFontFace->GetGdiCompatibleGlyphMetrics(float(fontDef.pixelSize),
                                                                  1.0f,
                                                                  NULL,
-                                                                 TRUE,
+                                                                 renderMode == DWRITE_RENDERING_MODE_GDI_NATURAL,
                                                                  glyphIndices.data(),
                                                                  glyphIndices.size(),
                                                                  glyphMetrics.data());
@@ -644,19 +666,23 @@ QImage QWindowsFontEngineDirectWrite::alphaMapForGlyph(glyph_t glyph,
 {
     QImage im = imageForGlyph(glyph, subPixelPosition, glyphMargin(Format_A8), t);
 
-    QImage alphaMap(im.width(), im.height(), QImage::Format_Alpha8);
+    if (!im.isNull()) {
+        QImage alphaMap(im.width(), im.height(), QImage::Format_Alpha8);
 
-    for (int y=0; y<im.height(); ++y) {
-        const uint *src = reinterpret_cast<const uint *>(im.constScanLine(y));
-        uchar *dst = alphaMap.scanLine(y);
-        for (int x=0; x<im.width(); ++x) {
-            *dst = 255 - (m_fontEngineData->pow_gamma[qGray(0xffffffff - *src)] * 255. / 2047.);
-            ++dst;
-            ++src;
+        for (int y=0; y<im.height(); ++y) {
+            const uint *src = reinterpret_cast<const uint *>(im.constScanLine(y));
+            uchar *dst = alphaMap.scanLine(y);
+            for (int x=0; x<im.width(); ++x) {
+                *dst = 255 - (m_fontEngineData->pow_gamma[qGray(0xffffffff - *src)] * 255. / 2047.);
+                ++dst;
+                ++src;
+            }
         }
-    }
 
-    return alphaMap;
+        return alphaMap;
+    } else {
+        return QFontEngine::alphaMapForGlyph(glyph, t);
+    }
 }
 
 QImage QWindowsFontEngineDirectWrite::alphaMapForGlyph(glyph_t glyph,
@@ -667,7 +693,11 @@ QImage QWindowsFontEngineDirectWrite::alphaMapForGlyph(glyph_t glyph,
 
 bool QWindowsFontEngineDirectWrite::supportsHorizontalSubPixelPositions() const
 {
-    return true;
+    DWRITE_RENDERING_MODE renderMode = hintingPreferenceToRenderingMode(fontDef);
+    return  (!isColorFont()
+            && renderMode != DWRITE_RENDERING_MODE_GDI_CLASSIC
+            && renderMode != DWRITE_RENDERING_MODE_GDI_NATURAL
+            && renderMode != DWRITE_RENDERING_MODE_ALIASED);
 }
 
 QFontEngine::Properties QWindowsFontEngineDirectWrite::properties() const
@@ -770,10 +800,15 @@ QImage QWindowsFontEngineDirectWrite::imageForGlyph(glyph_t t,
 
     if (SUCCEEDED(hr)) {
         RECT rect;
-        glyphAnalysis->GetAlphaTextureBounds(DWRITE_TEXTURE_CLEARTYPE_3x1, &rect);
+        glyphAnalysis->GetAlphaTextureBounds(renderMode == DWRITE_RENDERING_MODE_ALIASED
+                                                ? DWRITE_TEXTURE_ALIASED_1x1
+                                                : DWRITE_TEXTURE_CLEARTYPE_3x1,
+                                             &rect);
 
-        if (rect.top == rect.bottom || rect.left == rect.right)
+        if (rect.top == rect.bottom || rect.left == rect.right) {
+            qCDebug(lcQpaFonts) << __FUNCTION__ << "Cannot get alpha texture bounds. Falling back to slower rendering path.";
             return QImage();
+        }
 
         QRect boundingRect = QRect(QPoint(rect.left - margin,
                                           rect.top - margin),
@@ -803,63 +838,65 @@ QImage QWindowsFontEngineDirectWrite::imageForGlyph(glyph_t t,
             image.fill(0xffffffff);
         }
 
-        BOOL ok = true;
-
         if (SUCCEEDED(hr)) {
+            BOOL ok = true;
             while (SUCCEEDED(hr) && ok) {
-                const DWRITE_COLOR_GLYPH_RUN *colorGlyphRun = 0;
-                hr = enumerator->GetCurrentRun(&colorGlyphRun);
-                if (FAILED(hr)) { // No colored runs, only outline
-                    qErrnoWarning(hr, "%s: IDWriteColorGlyphRunEnumerator::GetCurrentRun failed", __FUNCTION__);
-                    break;
-                }
-
-                IDWriteGlyphRunAnalysis *colorGlyphsAnalysis = NULL;
-                hr = factory2->CreateGlyphRunAnalysis(
-                            &colorGlyphRun->glyphRun,
-                            &transform,
-                            renderMode,
-                            measureMode,
-                            gridFitMode,
-                            DWRITE_TEXT_ANTIALIAS_MODE_CLEARTYPE,
-                            0.0, 0.0,
-                            &colorGlyphsAnalysis
-                            );
-
-                if (FAILED(hr)) {
-                    qErrnoWarning(hr, "%s: CreateGlyphRunAnalysis failed for color run", __FUNCTION__);
-                    break;
-                }
-
-                float r, g, b, a;
-                if (colorGlyphRun->paletteIndex == 0xFFFF) {
-                    r = float(color.redF());
-                    g = float(color.greenF());
-                    b = float(color.blueF());
-                    a = float(color.alphaF());
-                } else {
-                    r = qBound(0.0f, colorGlyphRun->runColor.r, 1.0f);
-                    g = qBound(0.0f, colorGlyphRun->runColor.g, 1.0f);
-                    b = qBound(0.0f, colorGlyphRun->runColor.b, 1.0f);
-                    a = qBound(0.0f, colorGlyphRun->runColor.a, 1.0f);
-                }
-
-                if (!qFuzzyIsNull(a)) {
-                    renderGlyphRun(&image,
-                                   r,
-                                   g,
-                                   b,
-                                   a,
-                                   colorGlyphsAnalysis,
-                                   boundingRect);
-                }
-                colorGlyphsAnalysis->Release();
-
                 hr = enumerator->MoveNext(&ok);
                 if (FAILED(hr)) {
                     qErrnoWarning(hr, "%s: IDWriteColorGlyphRunEnumerator::MoveNext failed", __FUNCTION__);
                     break;
                 }
+
+                if (ok) {
+                    const DWRITE_COLOR_GLYPH_RUN *colorGlyphRun = 0;
+                    hr = enumerator->GetCurrentRun(&colorGlyphRun);
+                    if (FAILED(hr)) { // No colored runs, only outline
+                        qErrnoWarning(hr, "%s: IDWriteColorGlyphRunEnumerator::GetCurrentRun failed", __FUNCTION__);
+                        break;
+                    }
+
+                    IDWriteGlyphRunAnalysis *colorGlyphsAnalysis = NULL;
+                    hr = factory2->CreateGlyphRunAnalysis(
+                                &colorGlyphRun->glyphRun,
+                                &transform,
+                                renderMode,
+                                measureMode,
+                                gridFitMode,
+                                DWRITE_TEXT_ANTIALIAS_MODE_CLEARTYPE,
+                                0.0, 0.0,
+                                &colorGlyphsAnalysis
+                                );
+
+                    if (FAILED(hr)) {
+                        qErrnoWarning(hr, "%s: CreateGlyphRunAnalysis failed for color run", __FUNCTION__);
+                        break;
+                    }
+
+                    float r, g, b, a;
+                    if (colorGlyphRun->paletteIndex == 0xFFFF) {
+                        r = float(color.redF());
+                        g = float(color.greenF());
+                        b = float(color.blueF());
+                        a = 1.0;
+                    } else {
+                        r = qBound(0.0f, colorGlyphRun->runColor.r, 1.0f);
+                        g = qBound(0.0f, colorGlyphRun->runColor.g, 1.0f);
+                        b = qBound(0.0f, colorGlyphRun->runColor.b, 1.0f);
+                        a = qBound(0.0f, colorGlyphRun->runColor.a, 1.0f);
+                    }
+
+                    if (!qFuzzyIsNull(a)) {
+                        renderGlyphRun(&image,
+                                       r,
+                                       g,
+                                       b,
+                                       a,
+                                       colorGlyphsAnalysis,
+                                       boundingRect,
+                                       renderMode);
+                    }
+                    colorGlyphsAnalysis->Release();
+                }
             }
         } else {
             float r, g, b, a;
@@ -878,7 +915,8 @@ QImage QWindowsFontEngineDirectWrite::imageForGlyph(glyph_t t,
                            b,
                            a,
                            glyphAnalysis,
-                           boundingRect);
+                           boundingRect,
+                           renderMode);
         }
 
         glyphAnalysis->Release();
@@ -896,7 +934,8 @@ void QWindowsFontEngineDirectWrite::renderGlyphRun(QImage *destination,
                                                    float b,
                                                    float a,
                                                    IDWriteGlyphRunAnalysis *glyphAnalysis,
-                                                   const QRect &boundingRect)
+                                                   const QRect &boundingRect,
+                                                   DWRITE_RENDERING_MODE renderMode)
 {
     const int width = destination->width();
     const int height = destination->height();
@@ -917,12 +956,14 @@ void QWindowsFontEngineDirectWrite::renderGlyphRun(QImage *destination,
         BYTE *alphaValues = alphaValueArray.data();
         memset(alphaValues, 0, size);
 
-        HRESULT hr = glyphAnalysis->CreateAlphaTexture(DWRITE_TEXTURE_CLEARTYPE_3x1,
+        HRESULT hr = glyphAnalysis->CreateAlphaTexture(renderMode == DWRITE_RENDERING_MODE_ALIASED
+                                                               ? DWRITE_TEXTURE_ALIASED_1x1
+                                                               : DWRITE_TEXTURE_CLEARTYPE_3x1,
                                                        &rect,
                                                        alphaValues,
                                                        size);
         if (SUCCEEDED(hr)) {
-            if (destination->hasAlphaChannel()) {
+            if (destination->hasAlphaChannel()) { // Color glyphs
                 for (int y = 0; y < height; ++y) {
                     uint *dest = reinterpret_cast<uint *>(destination->scanLine(y));
                     BYTE *src = alphaValues + width * 3 * y;
@@ -933,6 +974,9 @@ void QWindowsFontEngineDirectWrite::renderGlyphRun(QImage *destination,
                         float blueAlpha = a * *src++ / 255.0;
                         float averageAlpha = (redAlpha + greenAlpha + blueAlpha) / 3.0;
 
+                        if (m_pixelGeometry == DWRITE_PIXEL_GEOMETRY_BGR)
+                            qSwap(redAlpha, blueAlpha);
+
                         QRgb currentRgb = dest[x];
                         dest[x] = qRgba(qRound(qRed(currentRgb) * (1.0 - averageAlpha) + averageAlpha * r),
                                         qRound(qGreen(currentRgb) * (1.0 - averageAlpha) + averageAlpha * g),
@@ -940,17 +984,30 @@ void QWindowsFontEngineDirectWrite::renderGlyphRun(QImage *destination,
                                         qRound(qAlpha(currentRgb) * (1.0 - averageAlpha) + averageAlpha * 255));
                     }
                 }
+            } else if (renderMode == DWRITE_RENDERING_MODE_ALIASED) {
+                for (int y = 0; y < height; ++y) {
+                    uint *dest = reinterpret_cast<uint *>(destination->scanLine(y));
+                    BYTE *src = alphaValues + width * y;
 
+                    for (int x = 0; x < width; ++x) {
+                        int alpha = *(src++);
+                        dest[x] = (alpha << 16) + (alpha << 8) + alpha;
+                    }
+                }
             } else {
                 for (int y = 0; y < height; ++y) {
                     uint *dest = reinterpret_cast<uint *>(destination->scanLine(y));
                     BYTE *src = alphaValues + width * 3 * y;
 
                     for (int x = 0; x < width; ++x) {
-                        dest[x] = *(src + 0) << 16
-                                | *(src + 1) << 8
-                                | *(src + 2);
+                        BYTE redAlpha   = *(src + 0);
+                        BYTE greenAlpha = *(src + 1);
+                        BYTE blueAlpha  = *(src + 2);
 
+                        if (m_pixelGeometry == DWRITE_PIXEL_GEOMETRY_BGR)
+                            qSwap(redAlpha, blueAlpha);
+
+                        dest[x] = qRgb(redAlpha, greenAlpha, blueAlpha);
                         src += 3;
                     }
                 }
@@ -973,6 +1030,12 @@ QImage QWindowsFontEngineDirectWrite::alphaRGBMapForGlyph(glyph_t t,
                                 glyphMargin(QFontEngine::Format_A32),
                                 xform);
 
+    if (mask.isNull()) {
+        mask = QFontEngine::renderedPathForGlyph(t, Qt::white);
+        if (!xform.isIdentity())
+            mask = mask.transformed(xform);
+    }
+
     return mask.depth() == 32
            ? mask
            : mask.convertToFormat(QImage::Format_RGB32);
@@ -1009,6 +1072,27 @@ void QWindowsFontEngineDirectWrite::initFontInfo(const QFontDef &request,
         fontDef.pointSize = fontDef.pixelSize * 72. / dpi;
     else if (fontDef.pixelSize == -1)
         fontDef.pixelSize = qRound(fontDef.pointSize * dpi / 72.);
+
+    m_faceId.variableAxes = request.variableAxisValues;
+
+#if QT_CONFIG(directwrite3)
+    IDWriteFontFace3 *face3 = nullptr;
+    if (SUCCEEDED(m_directWriteFontFace->QueryInterface(__uuidof(IDWriteFontFace3),
+                                                        reinterpret_cast<void **>(&face3)))) {
+        IDWriteLocalizedStrings *names;
+        if (SUCCEEDED(face3->GetFaceNames(&names))) {
+            wchar_t englishLocale[] = L"en-us";
+            fontDef.styleName = QWindowsDirectWriteFontDatabase::localeString(names, englishLocale);
+            names->Release();
+        }
+
+        // Color font
+        if (face3->GetPaletteEntryCount() > 0)
+            glyphFormat = QFontEngine::Format_ARGB;
+
+        face3->Release();
+    }
+#endif
 }
 
 QString QWindowsFontEngineDirectWrite::fontNameSubstitute(const QString &familyName)
@@ -1094,13 +1178,13 @@ glyph_metrics_t QWindowsFontEngineDirectWrite::alphaMapBoundingBox(glyph_t glyph
 
     if (SUCCEEDED(hr)) {
         RECT rect;
-        glyphAnalysis->GetAlphaTextureBounds(DWRITE_TEXTURE_CLEARTYPE_3x1, &rect);
+        glyphAnalysis->GetAlphaTextureBounds(renderMode == DWRITE_RENDERING_MODE_ALIASED ? DWRITE_TEXTURE_ALIASED_1x1 : DWRITE_TEXTURE_CLEARTYPE_3x1, &rect);
         glyphAnalysis->Release();
 
         int margin = glyphMargin(format);
 
         if (rect.left == rect.right || rect.top == rect.bottom)
-            return glyph_metrics_t();
+            return bbox;
 
         return glyph_metrics_t(rect.left,
                                rect.top,
