@@ -3,7 +3,6 @@
 
 #include "qdecompresshelper_p.h"
 
-#include <QtCore/private/qbytearray_p.h>
 #include <QtCore/qiodevice.h>
 #include <QtCore/qcoreapplication.h>
 
@@ -24,7 +23,7 @@ QT_BEGIN_NAMESPACE
 namespace {
 struct ContentEncodingMapping
 {
-    char name[8];
+    QByteArrayView name;
     QDecompressHelper::ContentEncoding encoding;
 };
 
@@ -39,10 +38,10 @@ constexpr ContentEncodingMapping contentEncodingMapping[] {
     { "deflate", QDecompressHelper::Deflate },
 };
 
-QDecompressHelper::ContentEncoding encodingFromByteArray(const QByteArray &ce) noexcept
+QDecompressHelper::ContentEncoding encodingFromByteArray(QByteArrayView ce) noexcept
 {
     for (const auto &mapping : contentEncodingMapping) {
-        if (ce.compare(QByteArrayView(mapping.name, strlen(mapping.name)), Qt::CaseInsensitive) == 0)
+        if (ce.compare(mapping.name, Qt::CaseInsensitive) == 0)
             return mapping.encoding;
     }
     return QDecompressHelper::None;
@@ -68,22 +67,19 @@ ZSTD_DStream *toZstandardPointer(void *ptr)
 #endif
 }
 
-bool QDecompressHelper::isSupportedEncoding(const QByteArray &encoding)
+bool QDecompressHelper::isSupportedEncoding(QByteArrayView encoding)
 {
     return encodingFromByteArray(encoding) != QDecompressHelper::None;
 }
 
 QByteArrayList QDecompressHelper::acceptedEncoding()
 {
-    static QByteArrayList accepted = []() {
-        QByteArrayList list;
-        list.reserve(sizeof(contentEncodingMapping) / sizeof(contentEncodingMapping[0]));
-        for (const auto &mapping : contentEncodingMapping) {
-            list << QByteArray(mapping.name);
-        }
-        return list;
-    }();
-    return accepted;
+    QByteArrayList list;
+    list.reserve(std::size(contentEncodingMapping));
+    for (const auto &mapping : contentEncodingMapping) {
+        list << mapping.name.toByteArray();
+    }
+    return list;
 }
 
 QDecompressHelper::~QDecompressHelper()
@@ -91,7 +87,7 @@ QDecompressHelper::~QDecompressHelper()
     clear();
 }
 
-bool QDecompressHelper::setEncoding(const QByteArray &encoding)
+bool QDecompressHelper::setEncoding(QByteArrayView encoding)
 {
     Q_ASSERT(contentEncoding == QDecompressHelper::None);
     if (contentEncoding != QDecompressHelper::None) {
@@ -238,8 +234,11 @@ void QDecompressHelper::feed(QByteArray &&data)
     Q_ASSERT(contentEncoding != None);
     totalCompressedBytes += data.size();
     compressedDataBuffer.append(std::move(data));
-    if (!countInternal(compressedDataBuffer[compressedDataBuffer.bufferCount() - 1]))
+    if (!countInternal(compressedDataBuffer[compressedDataBuffer.bufferCount() - 1])) {
+        if (errorStr.isEmpty() && countHelper)
+            errorStr = countHelper->errorString();
         clear(); // If our counting brother failed then so will we :|
+    }
 }
 
 /*!
@@ -251,8 +250,11 @@ void QDecompressHelper::feed(const QByteDataBuffer &buffer)
     Q_ASSERT(contentEncoding != None);
     totalCompressedBytes += buffer.byteAmount();
     compressedDataBuffer.append(buffer);
-    if (!countInternal(buffer))
+    if (!countInternal(buffer)) {
+        if (errorStr.isEmpty() && countHelper)
+            errorStr = countHelper->errorString();
         clear(); // If our counting brother failed then so will we :|
+    }
 }
 
 /*!
@@ -265,8 +267,11 @@ void QDecompressHelper::feed(QByteDataBuffer &&buffer)
     totalCompressedBytes += buffer.byteAmount();
     const QByteDataBuffer copy(buffer);
     compressedDataBuffer.append(std::move(buffer));
-    if (!countInternal(copy))
+    if (!countInternal(copy)) {
+        if (errorStr.isEmpty() && countHelper)
+            errorStr = countHelper->errorString();
         clear(); // If our counting brother failed then so will we :|
+    }
 }
 
 /*!
@@ -556,8 +561,6 @@ void QDecompressHelper::clear()
     totalBytesRead = 0;
     totalUncompressedBytes = 0;
     totalCompressedBytes = 0;
-
-    errorStr.clear();
 }
 
 qsizetype QDecompressHelper::readZLib(char *data, const qsizetype maxSize)
