@@ -36,7 +36,7 @@ public:
     static QBasicAtomicInt valid;
 };
 Q_GLOBAL_STATIC(QNetworkAccessBackendFactoryData, factoryData)
-QBasicAtomicInt QNetworkAccessBackendFactoryData::valid = Q_BASIC_ATOMIC_INITIALIZER(0);
+Q_CONSTINIT QBasicAtomicInt QNetworkAccessBackendFactoryData::valid = Q_BASIC_ATOMIC_INITIALIZER(0);
 
 class QNetworkAccessBackendPrivate : public QObjectPrivate
 {
@@ -45,7 +45,7 @@ public:
     QNetworkAccessBackend::SecurityFeatures m_securityFeatures;
     QNetworkAccessBackend::IOFeatures m_ioFeatures;
     std::shared_ptr<QNonContiguousByteDevice> uploadByteDevice;
-    QIODevice *wrappedUploadByteDevice;
+    QIODevice *wrappedUploadByteDevice = nullptr;
     QNetworkReplyImplPrivate *m_reply = nullptr;
     QNetworkAccessManagerPrivate *m_manager = nullptr;
 
@@ -567,6 +567,43 @@ void QNetworkAccessBackend::setRawHeader(const QByteArray &header, const QByteAr
     d_func()->m_reply->setRawHeader(header, value);
 }
 
+/*!
+    \since 6.8
+
+    Returns headers that are set in this QNetworkAccessBackend instance.
+
+    \sa setHeaders()
+*/
+QHttpHeaders QNetworkAccessBackend::headers() const
+{
+    return d_func()->m_reply->headers();
+}
+
+/*!
+    \since 6.8
+
+    Sets \a newHeaders as headers, overriding any previously set headers.
+
+    These headers are accessible on the QNetworkReply instance which was
+    returned when calling one of the appropriate functions on
+    QNetworkAccessManager.
+
+    \sa headers()
+*/
+void QNetworkAccessBackend::setHeaders(QHttpHeaders &&newHeaders)
+{
+    d_func()->m_reply->setHeaders(std::move(newHeaders));
+}
+
+/*!
+    \overload
+    \since 6.8
+*/
+void QNetworkAccessBackend::setHeaders(const QHttpHeaders &newHeaders)
+{
+    d_func()->m_reply->setHeaders(newHeaders);
+}
+
 /*!
     Returns the operation which was requested when calling
     QNetworkAccessManager.
