@@ -43,8 +43,8 @@
     use waitForNewConnection(), which blocks until either a
     connection is available or a timeout expires.
 
-    \sa QTcpSocket, {Fortune Server Example}, {Threaded Fortune Server Example},
-        {Loopback Example}, {Torrent Example}
+    \sa QTcpSocket, {Fortune Server}, {Threaded Fortune Server},
+        {Torrent Example}
 */
 
 /*! \fn void QTcpServer::newConnection()
@@ -498,7 +498,7 @@ bool QTcpServer::waitForNewConnection(int msec, bool *timedOut)
     if (d->state != QAbstractSocket::ListeningState)
         return false;
 
-    if (!d->socketEngine->waitForRead(msec, timedOut)) {
+    if (!d->socketEngine->waitForRead(QDeadlineTimer(msec), timedOut)) {
         d->serverSocketError = d->socketEngine->error();
         d->serverSocketErrorString = d->socketEngine->errorString();
         return false;
