@@ -27,7 +27,7 @@ QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
 
-Q_APPLICATION_STATIC(QFactoryLoader, loader, QTlsBackend_iid,
+Q_APPLICATION_STATIC(QFactoryLoader, qtlsbLoader, QTlsBackend_iid,
                      QStringLiteral("/tls"))
 
 namespace {
@@ -54,7 +54,7 @@ public:
 
     bool tryPopulateCollection()
     {
-        if (!loader())
+        if (!qtlsbLoader())
             return false;
 
         Q_CONSTINIT static QBasicMutex mutex;
@@ -63,10 +63,10 @@ public:
             return true;
 
 #if QT_CONFIG(library)
-        loader->update();
+        qtlsbLoader->update();
 #endif
         int index = 0;
-        while (loader->instance(index))
+        while (qtlsbLoader->instance(index))
             ++index;
 
         return true;
@@ -889,20 +889,28 @@ QSslCipher QTlsBackend::createCiphersuite(const QString &suiteName, QSsl::SslPro
 
 /*!
     \internal
-    Auxiliary function. Creates a new QSslCipher from \a name (which is an implementation-specific
-    string), \a protocol and \a protocolString, e.g.:
+    Auxiliary function. Creates a new QSslCipher from \a name, \a keyExchangeMethod, \a encryptionMethod,
+    \a authenticationMethod, \a bits, \a protocol version and \a protocolString.
+    For example:
     \code
-    createCipher(QStringLiteral("schannel"), QSsl::TlsV1_2, "TLSv1.2"_L1);
+    createCiphersuite("ECDHE-RSA-AES256-GCM-SHA256"_L1, "ECDH"_L1, "AES"_L1, "RSA"_L1, 256,
+                      QSsl::TlsV1_2, "TLSv1.2"_L1);
     \endcode
 */
-QSslCipher QTlsBackend::createCipher(const QString &name, QSsl::SslProtocol protocol,
-                                     const QString &protocolString)
+QSslCipher QTlsBackend::createCiphersuite(const QString &name, const QString &keyExchangeMethod,
+                                          const QString &encryptionMethod,
+                                          const QString &authenticationMethod,
+                                          int bits, QSsl::SslProtocol protocol,
+                                          const QString &protocolString)
 {
-    // Note the name 'createCipher' (not 'ciphersuite'): we don't provide
-    // information about Kx, Au, bits/supported etc.
     QSslCipher cipher;
     cipher.d->isNull = false;
     cipher.d->name = name;
+    cipher.d->bits = bits;
+    cipher.d->supportedBits = bits;
+    cipher.d->keyExchangeMethod = keyExchangeMethod;
+    cipher.d->encryptionMethod = encryptionMethod;
+    cipher.d->authenticationMethod = authenticationMethod;
     cipher.d->protocol = protocol;
     cipher.d->protocolString = protocolString;
     return cipher;
@@ -1382,8 +1390,7 @@ QByteArray TlsKey::pemHeader() const
     else if (algorithm() == QSsl::Dh)
         return QByteArrayLiteral("-----BEGIN PRIVATE KEY-----");
 
-    Q_UNREACHABLE();
-    return {};
+    Q_UNREACHABLE_RETURN({});
 }
 
 /*!
@@ -1404,8 +1411,7 @@ QByteArray TlsKey::pemFooter() const
     else if (algorithm() == QSsl::Dh)
         return QByteArrayLiteral("-----END PRIVATE KEY-----");
 
-    Q_UNREACHABLE();
-    return {};
+    Q_UNREACHABLE_RETURN({});
 }
 
 /*!
