@@ -19,7 +19,13 @@ ParsedSpecification parseSpecification(const QString &specification)
         if (arg.startsWith("/dev/"_L1)) {
             // if device is specified try to use it
             result.devices.append(arg.toString());
-        } else {
+        }
+#ifdef Q_OS_VXWORKS
+        else if (arg.startsWith("/input/"_L1)) {
+            result.devices.append(arg.toString());
+        }
+#endif
+        else {
             // build new specification without /dev/ elements
             result.spec += arg + u':';
         }
