@@ -1,42 +1,7 @@
-/****************************************************************************
-**
-** Copyright (C) 2012 BogDan Vatra <bogdan@kde.org>
-** Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
-** Contact: http://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2023 The Qt Company Ltd.
+// Copyright (C) 2012 BogDan Vatra <bogdan@kde.org>
+// Copyright (C) 2016 Olivier Goffart <ogoffart@woboq.com>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtGui/qtguiglobal.h>
 
@@ -44,6 +9,7 @@
 #include "androidjnimain.h"
 #include "qandroidplatformintegration.h"
 
+#include <qpa/qplatformwindow.h>
 #include <qpa/qwindowsysteminterface.h>
 #include <QTouchEvent>
 #include <QPointer>
@@ -53,66 +19,61 @@
 
 QT_BEGIN_NAMESPACE
 
+Q_LOGGING_CATEGORY(lcQpaInputMethods, "qt.qpa.input.methods");
+
 using namespace QtAndroid;
 
+Q_DECLARE_JNI_CLASS(QtInputInterface, "org/qtproject/qt/android/QtInputInterface")
+
 namespace QtAndroidInput
 {
     static bool m_ignoreMouseEvents = false;
+    static Qt::MouseButtons m_buttons = Qt::NoButton;
+
     static QRect m_softwareKeyboardRect;
 
     static QList<QWindowSystemInterface::TouchPoint> m_touchPoints;
 
     static QPointer<QWindow> m_mouseGrabber;
 
+
     void updateSelection(int selStart, int selEnd, int candidatesStart, int candidatesEnd)
     {
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << ">>> UPDATESELECTION" << selStart << selEnd << candidatesStart << candidatesEnd;
-#endif
-        QJniObject::callStaticMethod<void>(applicationClass(),
-                                           "updateSelection",
-                                           "(IIII)V",
-                                           selStart,
-                                           selEnd,
-                                           candidatesStart,
-                                           candidatesEnd);
+        qCDebug(lcQpaInputMethods) << ">>> UPDATESELECTION" << selStart << selEnd << candidatesStart << candidatesEnd;
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        reg->callInterface<QtJniTypes::QtInputInterface, void>("updateSelection", selStart, selEnd,
+                                                               candidatesStart, candidatesEnd);
     }
 
     void showSoftwareKeyboard(int left, int top, int width, int height, int inputHints, int enterKeyType)
     {
-        QJniObject::callStaticMethod<void>(applicationClass(),
-                                           "showSoftwareKeyboard",
-                                           "(IIIIII)V",
-                                           left,
-                                           top,
-                                           width,
-                                           height,
-                                           inputHints,
-                                           enterKeyType);
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ SHOWSOFTWAREKEYBOARD" << left << top << width << height << inputHints << enterKeyType;
-#endif
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        reg->callInterface<QtJniTypes::QtInputInterface, void>(
+                "showSoftwareKeyboard", QtAndroidPrivate::activity(),
+                left, top, width, height, inputHints,
+                enterKeyType);
+        qCDebug(lcQpaInputMethods) << "@@@ SHOWSOFTWAREKEYBOARD" << left << top << width << height << inputHints << enterKeyType;
     }
 
     void resetSoftwareKeyboard()
     {
-        QJniObject::callStaticMethod<void>(applicationClass(), "resetSoftwareKeyboard");
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug("@@@ RESETSOFTWAREKEYBOARD");
-#endif
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        reg->callInterface<QtJniTypes::QtInputInterface, void>("resetSoftwareKeyboard");
+        qCDebug(lcQpaInputMethods) << "@@@ RESETSOFTWAREKEYBOARD";
     }
 
     void hideSoftwareKeyboard()
     {
-        QJniObject::callStaticMethod<void>(applicationClass(), "hideSoftwareKeyboard");
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug("@@@ HIDESOFTWAREKEYBOARD");
-#endif
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        reg->callInterface<QtJniTypes::QtInputInterface, void>("hideSoftwareKeyboard");
+        qCDebug(lcQpaInputMethods) << "@@@ HIDESOFTWAREKEYBOARD";
     }
 
     bool isSoftwareKeyboardVisible()
     {
-        return QJniObject::callStaticMethod<jboolean>(applicationClass(), "isSoftwareKeyboardVisible");
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        return reg->callInterface<QtJniTypes::QtInputInterface, jboolean>(
+                "isSoftwareKeyboardVisible");
     }
 
     QRect softwareKeyboardRect()
@@ -122,81 +83,149 @@ namespace QtAndroidInput
 
     int getSelectHandleWidth()
     {
-        return QJniObject::callStaticMethod<jint>(applicationClass(), "getSelectHandleWidth");
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        return reg->callInterface<QtJniTypes::QtInputInterface, jint>("getSelectionHandleWidth");
     }
 
     void updateHandles(int mode, QPoint editMenuPos, uint32_t editButtons, QPoint cursor, QPoint anchor, bool rtl)
     {
-        QJniObject::callStaticMethod<void>(applicationClass(), "updateHandles", "(IIIIIIIIZ)V",
-                                           mode, editMenuPos.x(), editMenuPos.y(), editButtons,
-                                           cursor.x(), cursor.y(),
-                                           anchor.x(), anchor.y(), rtl);
+        AndroidBackendRegister *reg = QtAndroid::backendRegister();
+        reg->callInterface<QtJniTypes::QtInputInterface, void>(
+                "updateHandles", mode, editMenuPos.x(), editMenuPos.y(),
+                editButtons, cursor.x(), cursor.y(), anchor.x(), anchor.y(), rtl);
     }
 
-    static void mouseDown(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y)
+    // from https://developer.android.com/reference/android/view/MotionEvent#getButtonState()
+    enum AndroidMouseButton {
+        BUTTON_PRIMARY = 0x00000001,
+        BUTTON_SECONDARY = 0x00000002,
+        BUTTON_TERTIARY = 0x00000004,
+        BUTTON_BACK = 0x00000008,
+        BUTTON_FORWARD = 0x00000010,
+        BUTTON_STYLUS_PRIMARY = 0x00000020,
+        BUTTON_STYLUS_SECONDARY = 0x00000040,
+    };
+    Q_DECLARE_FLAGS(AndroidMouseButtons, AndroidMouseButton)
+
+    static Qt::MouseButtons toMouseButtons(jint j_buttons)
+    {
+        const auto buttons = static_cast<AndroidMouseButtons>(j_buttons);
+        Qt::MouseButtons mouseButtons;
+        if (buttons.testFlag(BUTTON_PRIMARY))
+            mouseButtons.setFlag(Qt::LeftButton);
+
+        if (buttons.testFlag(BUTTON_SECONDARY))
+            mouseButtons.setFlag(Qt::RightButton);
+
+        if (buttons.testFlag(BUTTON_TERTIARY))
+            mouseButtons.setFlag(Qt::MiddleButton);
+
+        if (buttons.testFlag(BUTTON_BACK))
+            mouseButtons.setFlag(Qt::BackButton);
+
+        if (buttons.testFlag(BUTTON_FORWARD))
+            mouseButtons.setFlag(Qt::ForwardButton);
+
+        if (buttons.testFlag(BUTTON_STYLUS_PRIMARY))
+            mouseButtons.setFlag(Qt::LeftButton);
+
+        if (buttons.testFlag(BUTTON_STYLUS_SECONDARY))
+            mouseButtons.setFlag(Qt::RightButton);
+
+        // Fall back to left button
+        if (Q_UNLIKELY(buttons != 0 && mouseButtons == Qt::NoButton)) {
+            qWarning() << "Unhandled button value:" << buttons << "Falling back to Qt::LeftButton";
+            mouseButtons = Qt::LeftButton;
+        }
+        return mouseButtons;
+    }
+
+    static void sendMouseButtonEvents(QWindow *topLevel, QPoint localPos, QPoint globalPos,
+                                      jint mouseButtonState, QEvent::Type type)
+    {
+        const Qt::MouseButtons mouseButtons = toMouseButtons(mouseButtonState);
+        const Qt::MouseButtons changedButtons = mouseButtons & ~m_buttons;
+
+        if (changedButtons == Qt::NoButton)
+            return;
+
+        static_assert (sizeof(changedButtons) <= sizeof(uint), "Qt::MouseButtons size changed. Adapt code.");
+
+        for (uint buttonInt = 0x1; static_cast<uint>(changedButtons) >= buttonInt; buttonInt <<= 1) {
+            const auto button = static_cast<Qt::MouseButton>(buttonInt);
+            if (changedButtons.testFlag(button)) {
+                QWindowSystemInterface::handleMouseEvent(topLevel, localPos, globalPos,
+                                                         mouseButtons, button, type);
+            }
+        }
+    }
+
+    static void mouseDown(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint x, jint y, jint mouseButtonState)
     {
         if (m_ignoreMouseEvents)
             return;
 
-        QPoint globalPos(x,y);
-        QWindow *tlw = topLevelWindowAt(globalPos);
-        m_mouseGrabber = tlw;
-        QPoint localPos = tlw ? (globalPos - tlw->position()) : globalPos;
-        QWindowSystemInterface::handleMouseEvent(tlw, localPos, globalPos,
-                                                 Qt::MouseButtons(Qt::LeftButton),
-                                                 Qt::LeftButton, QEvent::MouseButtonPress);
+        const QPoint globalPos(x,y);
+        QWindow *window = windowFromId(winId);
+        m_mouseGrabber = window;
+        const QPoint localPos = window && window->handle() ?
+                window->handle()->mapFromGlobal(globalPos) : globalPos;
+        sendMouseButtonEvents(window, localPos, globalPos, mouseButtonState, QEvent::MouseButtonPress);
     }
 
-    static void mouseUp(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y)
+    static void mouseUp(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint x, jint y, jint mouseButtonState)
     {
-        QPoint globalPos(x,y);
-        QWindow *tlw = m_mouseGrabber.data();
-        if (!tlw)
-            tlw = topLevelWindowAt(globalPos);
-        QPoint localPos = tlw ? (globalPos -tlw->position()) : globalPos;
-        QWindowSystemInterface::handleMouseEvent(tlw, localPos, globalPos,
-                                                 Qt::MouseButtons(Qt::NoButton),
-                                                 Qt::LeftButton, QEvent::MouseButtonRelease);
+        const QPoint globalPos(x,y);
+        QWindow *window = m_mouseGrabber.data();
+        if (!window)
+            window = windowFromId(winId);
+
+        const QPoint localPos = window && window->handle() ?
+                                    window->handle()->mapFromGlobal(globalPos) : globalPos;
+
+        sendMouseButtonEvents(window, localPos, globalPos, mouseButtonState, QEvent::MouseButtonRelease);
         m_ignoreMouseEvents = false;
-        m_mouseGrabber = 0;
+        m_mouseGrabber.clear();
     }
 
-    static void mouseMove(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y)
+    static void mouseMove(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint x, jint y)
     {
 
         if (m_ignoreMouseEvents)
             return;
 
-        QPoint globalPos(x,y);
-        QWindow *tlw = m_mouseGrabber.data();
-        if (!tlw)
-            tlw = topLevelWindowAt(globalPos);
-        QPoint localPos = tlw ? (globalPos-tlw->position()) : globalPos;
-        QWindowSystemInterface::handleMouseEvent(tlw, localPos, globalPos,
+        const QPoint globalPos(x,y);
+        QWindow *window = m_mouseGrabber.data();
+        if (!window)
+            window = windowFromId(winId);
+        const QPoint localPos = window && window->handle() ?
+                                    window->handle()->mapFromGlobal(globalPos) : globalPos;
+        QWindowSystemInterface::handleMouseEvent(window, localPos, globalPos,
                                                  Qt::MouseButtons(m_mouseGrabber ? Qt::LeftButton : Qt::NoButton),
                                                  Qt::NoButton, QEvent::MouseMove);
     }
 
-    static void mouseWheel(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y, jfloat hdelta, jfloat vdelta)
+    static void mouseWheel(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint x, jint y, jfloat hdelta, jfloat vdelta)
     {
         if (m_ignoreMouseEvents)
             return;
 
-        QPoint globalPos(x,y);
-        QWindow *tlw = m_mouseGrabber.data();
-        if (!tlw)
-            tlw = topLevelWindowAt(globalPos);
-        QPoint localPos = tlw ? (globalPos-tlw->position()) : globalPos;
-        QPoint angleDelta(hdelta * 120, vdelta * 120);
+        const QPoint globalPos(x,y);
+        QWindow *window = m_mouseGrabber.data();
+        if (!window)
+            window = windowFromId(winId);
+        const QPoint localPos = window && window->handle() ?
+                                    window->handle()->mapFromGlobal(globalPos) : globalPos;
+        const QPoint angleDelta(hdelta * 120, vdelta * 120);
 
-        QWindowSystemInterface::handleWheelEvent(tlw,
+        QWindowSystemInterface::handleWheelEvent(window,
                                                  localPos,
                                                  globalPos,
                                                  QPoint(),
                                                  angleDelta);
     }
 
-    static void longPress(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint x, jint y)
+    static void longPress(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint x, jint y)
     {
         QAndroidInputContext *inputContext = QAndroidInputContext::androidInputContext();
         if (inputContext && qGuiApp)
@@ -207,16 +236,17 @@ namespace QtAndroidInput
         if (!rightMouseFromLongPress)
             return;
         m_ignoreMouseEvents = true;
-        QPoint globalPos(x,y);
-        QWindow *tlw = topLevelWindowAt(globalPos);
-        QPoint localPos = tlw ? (globalPos-tlw->position()) : globalPos;
+        const QPoint globalPos(x,y);
+        QWindow *window = windowFromId(winId);
+        const QPoint localPos = window && window->handle() ?
+                                    window->handle()->mapFromGlobal(globalPos) : globalPos;
 
         // Click right button if no other button is already pressed.
         if (!m_mouseGrabber) {
-            QWindowSystemInterface::handleMouseEvent(tlw, localPos, globalPos,
+            QWindowSystemInterface::handleMouseEvent(window, localPos, globalPos,
                                                      Qt::MouseButtons(Qt::RightButton), Qt::RightButton,
                                                      QEvent::MouseButtonPress);
-            QWindowSystemInterface::handleMouseEvent(tlw, localPos, globalPos,
+            QWindowSystemInterface::handleMouseEvent(window, localPos, globalPos,
                                                      Qt::MouseButtons(Qt::NoButton), Qt::RightButton,
                                                      QEvent::MouseButtonRelease);
         }
@@ -227,7 +257,7 @@ namespace QtAndroidInput
         m_touchPoints.clear();
     }
 
-    static void touchAdd(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint id, jint action, jboolean /*primary*/, jint x, jint y,
+    static void touchAdd(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint id, jint action, jboolean /*primary*/, jint x, jint y,
         jfloat major, jfloat minor, jfloat rotation, jfloat pressure)
     {
         QEventPoint::State state = QEventPoint::State::Stationary;
@@ -248,18 +278,27 @@ namespace QtAndroidInput
 
         const int dw = availableWidthPixels();
         const int dh = availableHeightPixels();
+        QWindow *window = QtAndroid::windowFromId(winId);
+        if (!window) {
+            qCWarning(lcQpaInputMethods, "Touch event received for non-existing window %d", winId);
+            return;
+        }
+
+        QPointF mappedTouchPoint = window->mapToGlobal(QPointF(x, y));
         QWindowSystemInterface::TouchPoint touchPoint;
-        touchPoint.id = id;
+        // Start numbering touch points from 1
+        touchPoint.id = id + 1;
         touchPoint.pressure = pressure;
         touchPoint.rotation = qRadiansToDegrees(rotation);
-        touchPoint.normalPosition = QPointF(double(x / dw), double(y / dh));
+        touchPoint.normalPosition = QPointF((mappedTouchPoint.x() / dw),
+                                            (mappedTouchPoint.y() / dh));
         touchPoint.state = state;
-        touchPoint.area = QRectF(x - double(minor),
-                                 y - double(major),
-                                 double(minor * 2),
-                                 double(major * 2));
-        m_touchPoints.push_back(touchPoint);
+        touchPoint.area = QRectF(mappedTouchPoint.x() - double(minor * 0.5f),
+                                 mappedTouchPoint.y() - double(major * 0.5f),
+                                 double(minor),
+                                 double(major));
 
+        m_touchPoints.push_back(touchPoint);
         if (state == QEventPoint::State::Pressed) {
             QAndroidInputContext *inputContext = QAndroidInputContext::androidInputContext();
             if (inputContext && qGuiApp)
@@ -290,31 +329,35 @@ namespace QtAndroidInput
         return touchDevice;
     }
 
-    static void touchEnd(JNIEnv * /*env*/, jobject /*thiz*/, jint /*winId*/, jint /*action*/)
+    static void touchEnd(JNIEnv * /*env*/, jobject /*thiz*/, jint winId, jint /*action*/)
     {
         if (m_touchPoints.isEmpty())
             return;
 
         QMutexLocker lock(QtAndroid::platformInterfaceMutex());
-        QPointingDevice *touchDevice = getTouchDevice();
+        const QPointingDevice *touchDevice = getTouchDevice();
         if (!touchDevice)
             return;
 
-        QWindow *window = QtAndroid::topLevelWindowAt(m_touchPoints.at(0).area.center().toPoint());
+        QWindow *window = QtAndroid::windowFromId(winId);
+        if (!window)
+            return;
         QWindowSystemInterface::handleTouchEvent(window, touchDevice, m_touchPoints);
     }
 
-    static void touchCancel(JNIEnv * /*env*/, jobject /*thiz*/, jint /*winId*/)
+    static void touchCancel(JNIEnv * /*env*/, jobject /*thiz*/, jint winId)
     {
         if (m_touchPoints.isEmpty())
             return;
 
         QMutexLocker lock(QtAndroid::platformInterfaceMutex());
-        QPointingDevice *touchDevice = getTouchDevice();
+        const QPointingDevice *touchDevice = getTouchDevice();
         if (!touchDevice)
             return;
 
-        QWindow *window = QtAndroid::topLevelWindowAt(m_touchPoints.at(0).area.center().toPoint());
+        QWindow *window = QtAndroid::windowFromId(winId);
+        if (!window)
+            return;
         QWindowSystemInterface::handleTouchCancelEvent(window, touchDevice);
     }
 
@@ -327,14 +370,14 @@ namespace QtAndroidInput
 #endif // QT_CONFIG(tabletevent)
     }
 
-    static void tabletEvent(JNIEnv */*env*/, jobject /*thiz*/, jint /*winId*/, jint deviceId, jlong time, jint action,
+    static void tabletEvent(JNIEnv */*env*/, jobject /*thiz*/, jint winId, jint deviceId, jlong time, jint action,
         jint pointerType, jint buttonState, jfloat x, jfloat y, jfloat pressure)
     {
 #if QT_CONFIG(tabletevent)
-        QPointF globalPosF(x, y);
-        QPoint globalPos((int)x, (int)y);
-        QWindow *tlw = topLevelWindowAt(globalPos);
-        QPointF localPos = tlw ? (globalPosF - tlw->position()) : globalPosF;
+        const QPointF globalPosF(x, y);
+        QWindow *window = windowFromId(winId);
+        const QPointF localPos = window && window->handle() ?
+                                    window->handle()->mapFromGlobalF(globalPosF) : globalPosF;
 
         // Galaxy Note with plain Android:
         // 0 1 0    stylus press
@@ -354,6 +397,7 @@ namespace QtAndroidInput
         Qt::MouseButtons buttons = Qt::NoButton;
         switch (action) {
         case 1:     // ACTION_UP
+        case 6:     // ACTION_POINTER_UP, happens if stylus is not the primary pointer
         case 212:   // stylus release while side-button held on Galaxy Note 4
             buttons = Qt::NoButton;
             break;
@@ -365,11 +409,9 @@ namespace QtAndroidInput
             break;
         }
 
-#ifdef QT_DEBUG_ANDROID_STYLUS
-        qDebug() << action << pointerType << buttonState << '@' << x << y << "pressure" << pressure << ": buttons" << buttons;
-#endif
+        qCDebug(lcQpaInputMethods) << action << pointerType << buttonState << '@' << x << y << "pressure" << pressure << ": buttons" << buttons;
 
-        QWindowSystemInterface::handleTabletEvent(tlw, ulong(time),
+        QWindowSystemInterface::handleTabletEvent(window, ulong(time),
             localPos, globalPosF, int(QInputDevice::DeviceType::Stylus), pointerType,
             buttons, pressure, 0, 0, 0., 0., 0, deviceId, Qt::NoModifier);
 #endif // QT_CONFIG(tabletevent)
@@ -833,9 +875,7 @@ namespace QtAndroidInput
                 QMetaObject::invokeMethod(inputContext, "hideSelectionHandles", Qt::QueuedConnection);
             }
         }
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ KEYBOARDVISIBILITYCHANGED" << inputContext;
-#endif
+        qCDebug(lcQpaInputMethods) << "@@@ KEYBOARDVISIBILITYCHANGED" << inputContext;
     }
 
     static void keyboardGeometryChanged(JNIEnv */*env*/, jobject /*thiz*/, jint x, jint y, jint w, jint h)
@@ -848,16 +888,12 @@ namespace QtAndroidInput
         if (inputContext && qGuiApp)
             inputContext->emitKeyboardRectChanged();
 
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ KEYBOARDRECTCHANGED" << m_softwareKeyboardRect;
-#endif
+        qCDebug(lcQpaInputMethods) << "@@@ KEYBOARDRECTCHANGED" << m_softwareKeyboardRect;
     }
 
     static void handleLocationChanged(JNIEnv */*env*/, jobject /*thiz*/, int id, int x, int y)
     {
-#ifdef QT_DEBUG_ANDROID_IM_PROTOCOL
-        qDebug() << "@@@ handleLocationChanged" << id << x << y;
-#endif
+        qCDebug(lcQpaInputMethods) << "@@@ handleLocationChanged" << id << x << y;
         QAndroidInputContext *inputContext = QAndroidInputContext::androidInputContext();
         if (inputContext && qGuiApp)
             QMetaObject::invokeMethod(inputContext, "handleLocationChanged", Qt::BlockingQueuedConnection,
@@ -865,13 +901,14 @@ namespace QtAndroidInput
 
     }
 
-    static JNINativeMethod methods[] = {
+
+    static const JNINativeMethod methods[] = {
         {"touchBegin","(I)V",(void*)touchBegin},
         {"touchAdd","(IIIZIIFFFF)V",(void*)touchAdd},
         {"touchEnd","(II)V",(void*)touchEnd},
         {"touchCancel", "(I)V", (void *)touchCancel},
-        {"mouseDown", "(III)V", (void *)mouseDown},
-        {"mouseUp", "(III)V", (void *)mouseUp},
+        {"mouseDown", "(IIII)V", (void *)mouseDown},
+        {"mouseUp", "(IIII)V", (void *)mouseUp},
         {"mouseMove", "(III)V", (void *)mouseMove},
         {"mouseWheel", "(IIIFF)V", (void *)mouseWheel},
         {"longPress", "(III)V", (void *)longPress},
@@ -881,14 +918,13 @@ namespace QtAndroidInput
         {"keyUp", "(IIIZ)V", (void *)keyUp},
         {"keyboardVisibilityChanged", "(Z)V", (void *)keyboardVisibilityChanged},
         {"keyboardGeometryChanged", "(IIII)V", (void *)keyboardGeometryChanged},
-        {"handleLocationChanged", "(III)V", (void *)handleLocationChanged}
+        {"handleLocationChanged", "(III)V", (void *)handleLocationChanged},
     };
 
-    bool registerNatives(JNIEnv *env)
+    bool registerNatives(QJniEnvironment &env)
     {
-        jclass appClass = QtAndroid::applicationClass();
-
-        if (env->RegisterNatives(appClass, methods, sizeof(methods) / sizeof(methods[0])) < 0) {
+        if (!env.registerNativeMethods(QtJniTypes::Traits<QtJniTypes::QtInputDelegate>::className(),
+                                 methods, sizeof(methods) / sizeof(methods[0]))) {
             __android_log_print(ANDROID_LOG_FATAL,"Qt", "RegisterNatives failed");
             return false;
         }
