@@ -39,9 +39,9 @@ class QDebug;
 // QCocoaWindow
 //
 // QCocoaWindow is an NSView (not an NSWindow!) in the sense
-// that it relies on a NSView for all event handling and
-// graphics output and does not require a NSWindow, except for
-// for the window-related functions like setWindowTitle.
+// that it relies on an NSView for all event handling and
+// graphics output and does not require an NSWindow, except for
+// the window-related functions like setWindowTitle.
 //
 // As a consequence of this it is possible to embed the QCocoaWindow
 // in an NSView hierarchy by getting a pointer to the "backing"
@@ -80,6 +80,8 @@ public:
     QRect normalGeometry() const override;
     void setCocoaGeometry(const QRect &rect);
 
+    QMargins safeAreaMargins() const override;
+
     void setVisible(bool visible) override;
     void setWindowFlags(Qt::WindowFlags flags) override;
     void setWindowState(Qt::WindowStates state) override;
@@ -122,6 +124,7 @@ public:
 
     Q_NOTIFICATION_HANDLER(NSWindowDidMoveNotification) void windowDidMove();
     Q_NOTIFICATION_HANDLER(NSWindowDidResizeNotification) void windowDidResize();
+    Q_NOTIFICATION_HANDLER(NSWindowWillStartLiveResizeNotification) void windowWillStartLiveResize();
     Q_NOTIFICATION_HANDLER(NSWindowDidEndLiveResizeNotification) void windowDidEndLiveResize();
     Q_NOTIFICATION_HANDLER(NSWindowDidBecomeKeyNotification) void windowDidBecomeKey();
     Q_NOTIFICATION_HANDLER(NSWindowDidResignKeyNotification) void windowDidResignKey();
@@ -158,18 +161,19 @@ public:
     void setWindowCursor(NSCursor *cursor);
 
     void registerTouch(bool enable);
-    void setContentBorderThickness(int topThickness, int bottomThickness);
+
     void registerContentBorderArea(quintptr identifier, int upper, int lower);
     void setContentBorderAreaEnabled(quintptr identifier, bool enable);
     void setContentBorderEnabled(bool enable) override;
     bool testContentBorderAreaPosition(int position) const;
     void applyContentBorderThickness(NSWindow *window = nullptr);
-    void updateNSToolbar();
 
     qreal devicePixelRatio() const override;
     QWindow *childWindowAt(QPoint windowPoint);
     bool shouldRefuseKeyWindowAndFirstResponder();
 
+    bool windowEvent(QEvent *event) override;
+
     QPoint bottomLeftClippedByNSWindowOffset() const override;
 
     void updateNormalGeometry();
@@ -185,6 +189,8 @@ public:
     Q_DECLARE_FLAGS(RecreationReasons, RecreationReason)
     Q_FLAG(RecreationReasons)
 
+    bool inLiveResize() const override;
+
 protected:
     void recreateWindowIfNeeded();
     QCocoaNSWindow *createNSWindow(bool shouldBePanel);
@@ -219,32 +225,34 @@ public: // for QNSView
     static void setupPopupMonitor();
     static void removePopupMonitor();
 
-    NSView *m_view;
-    QCocoaNSWindow *m_nsWindow;
+    NSView *m_view = nil;
+    QCocoaNSWindow *m_nsWindow = nil;
 
-    Qt::WindowStates m_lastReportedWindowState;
-    Qt::WindowModality m_windowModality;
+    Qt::WindowStates m_lastReportedWindowState = Qt::WindowNoState;
+    Qt::WindowModality m_windowModality = Qt::NonModal;
 
     static QPointer<QCocoaWindow> s_windowUnderMouse;
 
-    bool m_initialized;
-    bool m_inSetVisible;
-    bool m_inSetGeometry;
-    bool m_inSetStyleMask;
-    QCocoaMenuBar *m_menubar;
+    bool m_initialized = false;
+    bool m_inSetVisible = false;
+    bool m_inSetGeometry = false;
+    bool m_inSetStyleMask = false;
+    bool m_inLiveResize = false;
 
-    bool m_frameStrutEventsEnabled;
+    QCocoaMenuBar *m_menubar = nullptr;
+
+    bool m_frameStrutEventsEnabled = false;
     QRect m_exposedRect;
     QRect m_normalGeometry;
-    int m_registerTouchCount;
-    bool m_resizableTransientParent;
+    int m_registerTouchCount = 0;
+    bool m_resizableTransientParent = false;
+
+    QMargins m_lastReportedSafeAreaMargins;
 
     static const int NoAlertRequest;
-    NSInteger m_alertRequest;
+    NSInteger m_alertRequest = NoAlertRequest;
 
-    bool m_drawContentBorderGradient;
-    int m_topContentBorderThickness;
-    int m_bottomContentBorderThickness;
+    bool m_drawContentBorderGradient = false;
 
     struct BorderRange {
         BorderRange(quintptr i, int u, int l) : identifier(i), upper(u), lower(l) { }
