@@ -1,6 +1,8 @@
 // Copyright (C) 2013 BlackBerry Limited. All rights reserved.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
+#undef QT_NO_FOREACH // this file contains unported legacy Q_FOREACH uses
+
 #include "qqnxglobal.h"
 
 #include "qqnxintegration.h"
@@ -30,11 +32,12 @@
 #if QT_CONFIG(qqnx_pps)
 #  include "qqnxbuttoneventnotifier.h"
 #  include "qqnxclipboard.h"
-#  if QT_CONFIG(qqnx_imf)
-#    include "qqnxinputcontext_imf.h"
-#  else
-#    include "qqnxinputcontext_noimf.h"
-#  endif
+#endif
+
+#if QT_CONFIG(qqnx_imf)
+#  include "qqnxinputcontext_imf.h"
+#else
+#  include "qqnxinputcontext_noimf.h"
 #endif
 
 #include <qpa/qplatforminputcontextfactory_p.h>
@@ -47,6 +50,7 @@
 #include <qpa/qwindowsysteminterface.h>
 
 #include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/private/qrhibackingstore_p.h>
 
 #if !defined(QT_NO_OPENGL)
 #include "qqnxglcontext.h"
@@ -62,14 +66,10 @@
 #include <QtCore/QFile>
 #include <errno.h>
 
-#if defined(QQNXINTEGRATION_DEBUG)
-#define qIntegrationDebug qDebug
-#else
-#define qIntegrationDebug QT_NO_QDEBUG_MACRO
-#endif
-
 QT_BEGIN_NAMESPACE
 
+// Q_LOGGING_CATEGORY(lcQpaQnx, "qt.qpa.qnx");
+
 using namespace Qt::StringLiterals;
 
 QQnxIntegration *QQnxIntegration::ms_instance;
@@ -98,7 +98,7 @@ static inline QQnxIntegration::Options parseOptions(const QStringList &paramList
 
 static inline int getContextCapabilities(const QStringList &paramList)
 {
-    QString contextCapabilitiesPrefix = QStringLiteral("screen-context-capabilities=");
+    constexpr auto contextCapabilitiesPrefix = "screen-context-capabilities="_L1;
     int contextCapabilities = SCREEN_APPLICATION_CONTEXT;
     for (const QString &param : paramList) {
         if (param.startsWith(contextCapabilitiesPrefix)) {
@@ -118,9 +118,9 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
     , m_screenEventThread(0)
     , m_navigatorEventHandler(new QQnxNavigatorEventHandler())
     , m_virtualKeyboard(0)
+    , m_inputContext(0)
 #if QT_CONFIG(qqnx_pps)
     , m_navigatorEventNotifier(0)
-    , m_inputContext(0)
     , m_buttonsNotifier(new QQnxButtonEventNotifier())
 #endif
     , m_qpaInputContext(0)
@@ -142,7 +142,7 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
 {
     ms_instance = this;
     m_options = parseOptions(paramList);
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // Open connection to QNX composition manager
     if (screen_create_context(&m_screenContext, getContextCapabilities(paramList))) {
@@ -219,7 +219,7 @@ QQnxIntegration::QQnxIntegration(const QStringList &paramList)
 
 QQnxIntegration::~QQnxIntegration()
 {
-    qIntegrationDebug("platform plugin shutdown begin");
+    qCDebug(lcQpaQnx) << "Platform plugin shutdown begin";
     delete m_nativeInterface;
 
 #if QT_CONFIG(draganddrop)
@@ -276,12 +276,12 @@ QQnxIntegration::~QQnxIntegration()
 
     ms_instance = nullptr;
 
-    qIntegrationDebug("platform plugin shutdown end");
+    qCDebug(lcQpaQnx) << "Platform plugin shutdown end";
 }
 
 bool QQnxIntegration::hasCapability(QPlatformIntegration::Capability cap) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     switch (cap) {
     case MultipleWindows:
     case ForeignWindows:
@@ -312,7 +312,7 @@ QPlatformWindow *QQnxIntegration::createForeignWindow(QWindow *window, WId nativ
 
 QPlatformWindow *QQnxIntegration::createPlatformWindow(QWindow *window) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QSurface::SurfaceType surfaceType = window->surfaceType();
     const bool needRootWindow = options() & RootWindow;
     switch (surfaceType) {
@@ -330,14 +330,25 @@ QPlatformWindow *QQnxIntegration::createPlatformWindow(QWindow *window) const
 
 QPlatformBackingStore *QQnxIntegration::createPlatformBackingStore(QWindow *window) const
 {
-    qIntegrationDebug();
-    return new QQnxRasterBackingStore(window);
+    QSurface::SurfaceType surfaceType = window->surfaceType();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO << surfaceType;
+    switch (surfaceType) {
+    case QSurface::RasterSurface:
+        return new QQnxRasterBackingStore(window);
+#if !defined(QT_NO_OPENGL)
+    // Return a QRhiBackingStore for non-raster surface windows
+    case QSurface::OpenGLSurface:
+        return new QRhiBackingStore(window);
+#endif
+    default:
+        return nullptr;
+    }
 }
 
 #if !defined(QT_NO_OPENGL)
 QPlatformOpenGLContext *QQnxIntegration::createPlatformOpenGLContext(QOpenGLContext *context) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // Get color channel sizes from window format
     QSurfaceFormat format = context->format();
@@ -393,19 +404,17 @@ QPlatformOpenGLContext *QQnxIntegration::createPlatformOpenGLContext(QOpenGLCont
 }
 #endif
 
-#if QT_CONFIG(qqnx_pps)
 QPlatformInputContext *QQnxIntegration::inputContext() const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     if (m_qpaInputContext)
         return m_qpaInputContext;
     return m_inputContext;
 }
-#endif
 
 void QQnxIntegration::moveToScreen(QWindow *window, int screen)
 {
-    qIntegrationDebug() << "w =" << window << ", s =" << screen;
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO << "w =" << window << ", s =" << screen;
 
     // get platform window used by widget
     QQnxWindow *platformWindow = static_cast<QQnxWindow *>(window->handle());
@@ -419,7 +428,7 @@ void QQnxIntegration::moveToScreen(QWindow *window, int screen)
 
 QAbstractEventDispatcher *QQnxIntegration::createEventDispatcher() const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // We transfer ownersip of the event-dispatcher to QtCoreApplication
     QAbstractEventDispatcher *eventDispatcher = m_eventDispatcher;
@@ -436,7 +445,7 @@ QPlatformNativeInterface *QQnxIntegration::nativeInterface() const
 #if !defined(QT_NO_CLIPBOARD)
 QPlatformClipboard *QQnxIntegration::clipboard() const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
 #if QT_CONFIG(qqnx_pps)
     if (!m_clipboard)
@@ -455,7 +464,7 @@ QPlatformDrag *QQnxIntegration::drag() const
 
 QVariant QQnxIntegration::styleHint(QPlatformIntegration::StyleHint hint) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     if ((hint == ShowIsFullScreen) && (m_options & FullScreenApplication))
         return true;
 
@@ -469,7 +478,7 @@ QPlatformServices * QQnxIntegration::services() const
 
 QWindow *QQnxIntegration::window(screen_window_t qnxWindow) const
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QMutexLocker locker(&m_windowMapperMutex);
     Q_UNUSED(locker);
     return m_windowMapper.value(qnxWindow, 0);
@@ -477,7 +486,7 @@ QWindow *QQnxIntegration::window(screen_window_t qnxWindow) const
 
 void QQnxIntegration::addWindow(screen_window_t qnxWindow, QWindow *window)
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QMutexLocker locker(&m_windowMapperMutex);
     Q_UNUSED(locker);
     m_windowMapper.insert(qnxWindow, window);
@@ -485,7 +494,7 @@ void QQnxIntegration::addWindow(screen_window_t qnxWindow, QWindow *window)
 
 void QQnxIntegration::removeWindow(screen_window_t qnxWindow)
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     QMutexLocker locker(&m_windowMapperMutex);
     Q_UNUSED(locker);
     m_windowMapper.remove(qnxWindow);
@@ -594,7 +603,7 @@ QList<screen_display_t *> QQnxIntegration::sortDisplays(screen_display_t *availa
 
 void QQnxIntegration::createDisplays()
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
     // Query number of displays
     int displayCount = 0;
     int result = screen_get_context_property_iv(m_screenContext, SCREEN_PROPERTY_DISPLAY_COUNT,
@@ -624,11 +633,12 @@ void QQnxIntegration::createDisplays()
         Q_SCREEN_CHECKERROR(result, "Failed to query display attachment");
 
         if (!isAttached) {
-            qIntegrationDebug("Skipping non-attached display %d", i);
+            qCDebug(lcQpaQnx) << "Skipping non-attached display " << i;
             continue;
         }
 
-        qIntegrationDebug("Creating screen for display %d", i);
+        qCDebug(lcQpaQnx) << "Creating screen for display " << i;
+
         createDisplay(*orderedDisplays[i], /*isPrimary=*/false);
     } // of displays iteration
 }
@@ -662,7 +672,8 @@ void QQnxIntegration::removeDisplay(QQnxScreen *screen)
 
 void QQnxIntegration::destroyDisplays()
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
+
     Q_FOREACH (QQnxScreen *screen, m_screens) {
         QWindowSystemInterface::handleScreenRemoved(screen);
     }
@@ -713,7 +724,7 @@ bool QQnxIntegration::supportsNavigatorEvents() const
 #if QT_CONFIG(opengl)
 void QQnxIntegration::createEglDisplay()
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // Initialize connection to EGL
     m_eglDisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
@@ -727,7 +738,7 @@ void QQnxIntegration::createEglDisplay()
 
 void QQnxIntegration::destroyEglDisplay()
 {
-    qIntegrationDebug();
+    qCDebug(lcQpaQnx) << Q_FUNC_INFO;
 
     // Close connection to EGL
     eglTerminate(m_eglDisplay);
