@@ -6,6 +6,7 @@
 #include "qwindowswindow.h"
 
 #include <QtGui/qwindow.h>
+#include <QtGui/private/qpixmap_win_p.h>
 #include <QtCore/qdebug.h>
 #include <QtCore/qvariant.h>
 #include <QtCore/qmetaobject.h>
@@ -229,14 +230,14 @@ void QWindowsMenuItem::setIcon(const QIcon &icon)
         updateBitmap();
 }
 
-Q_GUI_EXPORT HBITMAP qt_pixmapToWinHBITMAP(const QPixmap &p, int hbitmapFormat = 0);
-
 void QWindowsMenuItem::updateBitmap()
 {
     freeBitmap();
     if (!m_icon.isNull()) {
         const int size = m_iconSize ? m_iconSize : GetSystemMetrics(SM_CYMENUCHECK);
-        m_hbitmap = qt_pixmapToWinHBITMAP(m_icon.pixmap(QSize(size, size)), 1);
+        // native Win32 menus don't support high-DPI versions of icons, so always
+        // use the pixmap for a 1.0 scale factor.
+        m_hbitmap = qt_pixmapToWinHBITMAP(m_icon.pixmap(QSize(size, size), 1.0), 1);
     }
     MENUITEMINFO itemInfo;
     menuItemInfoInit(itemInfo);
