@@ -44,7 +44,6 @@ QT_BEGIN_NAMESPACE
 using namespace Qt::StringLiterals;
 
 Q_LOGGING_CATEGORY(lcQpaXInput, "qt.qpa.input")
-Q_LOGGING_CATEGORY(lcQpaXInputDevices, "qt.qpa.input.devices")
 Q_LOGGING_CATEGORY(lcQpaXInputEvents, "qt.qpa.input.events")
 Q_LOGGING_CATEGORY(lcQpaScreen, "qt.qpa.screen")
 Q_LOGGING_CATEGORY(lcQpaEvents, "qt.qpa.events")
@@ -92,9 +91,9 @@ QXcbConnection::QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGra
     const int focusInDelay = 100;
     m_focusInTimer.setSingleShot(true);
     m_focusInTimer.setInterval(focusInDelay);
-    m_focusInTimer.callOnTimeout([]() {
+    m_focusInTimer.callOnTimeout(this, []() {
         // No FocusIn events for us, proceed with FocusOut normally.
-        QWindowSystemInterface::handleWindowActivated(nullptr, Qt::ActiveWindowFocusReason);
+        QWindowSystemInterface::handleFocusWindowChanged(nullptr, Qt::ActiveWindowFocusReason);
     });
 
     sync();
@@ -145,12 +144,12 @@ void QXcbConnection::removeWindowEventListener(xcb_window_t id)
 
 QXcbWindowEventListener *QXcbConnection::windowEventListenerFromId(xcb_window_t id)
 {
-    return m_mapper.value(id, 0);
+    return m_mapper.value(id, nullptr);
 }
 
 QXcbWindow *QXcbConnection::platformWindowFromId(xcb_window_t id)
 {
-    QXcbWindowEventListener *listener = m_mapper.value(id, 0);
+    QXcbWindowEventListener *listener = m_mapper.value(id, nullptr);
     if (listener)
         return listener->toWindow();
     return nullptr;
@@ -602,12 +601,12 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         if (clientMessage->format != 32)
             return;
 #if QT_CONFIG(draganddrop)
-        if (clientMessage->type == atom(QXcbAtom::XdndStatus))
+        if (clientMessage->type == atom(QXcbAtom::AtomXdndStatus))
             drag()->handleStatus(clientMessage);
-        else if (clientMessage->type == atom(QXcbAtom::XdndFinished))
+        else if (clientMessage->type == atom(QXcbAtom::AtomXdndFinished))
             drag()->handleFinished(clientMessage);
 #endif
-        if (m_systemTrayTracker && clientMessage->type == atom(QXcbAtom::MANAGER))
+        if (m_systemTrayTracker && clientMessage->type == atom(QXcbAtom::AtomMANAGER))
             m_systemTrayTracker->notifyManagerClientMessageEvent(clientMessage);
         HANDLE_PLATFORM_WINDOW_EVENT(xcb_client_message_event_t, window, handleClientMessageEvent);
     }
@@ -658,7 +657,7 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         setTime(selectionRequest->time);
 #endif
 #if QT_CONFIG(draganddrop)
-        if (selectionRequest->selection == atom(QXcbAtom::XdndSelection))
+        if (selectionRequest->selection == atom(QXcbAtom::AtomXdndSelection))
             m_drag->handleSelectionRequest(selectionRequest);
         else
 #endif
@@ -686,11 +685,11 @@ void QXcbConnection::handleXcbEvent(xcb_generic_event_t *event)
         if (m_clipboard->handlePropertyNotify(event))
             break;
 #endif
-        if (propertyNotify->atom == atom(QXcbAtom::_NET_WORKAREA)) {
+        if (propertyNotify->atom == atom(QXcbAtom::Atom_NET_WORKAREA)) {
             QXcbVirtualDesktop *virtualDesktop = virtualDesktopForRootWindow(propertyNotify->window);
             if (virtualDesktop)
                 virtualDesktop->updateWorkArea();
-        } else if (propertyNotify->atom == atom(QXcbAtom::_NET_SUPPORTED)) {
+        } else if (propertyNotify->atom == atom(QXcbAtom::Atom_NET_SUPPORTED)) {
             m_wmSupport->updateNetWMAtoms();
         } else {
             HANDLE_PLATFORM_WINDOW_EVENT(xcb_property_notify_event_t, window, handlePropertyNotifyEvent);
@@ -787,13 +786,13 @@ void QXcbConnection::setStartupId(const QByteArray &nextId)
             xcb_change_property(xcb_connection(),
                                 XCB_PROP_MODE_REPLACE,
                                 clientLeader(),
-                                atom(QXcbAtom::_NET_STARTUP_ID),
-                                atom(QXcbAtom::UTF8_STRING),
+                                atom(QXcbAtom::Atom_NET_STARTUP_ID),
+                                atom(QXcbAtom::AtomUTF8_STRING),
                                 8,
                                 nextId.size(),
                                 nextId.constData());
         else
-            xcb_delete_property(xcb_connection(), clientLeader(), atom(QXcbAtom::_NET_STARTUP_ID));
+            xcb_delete_property(xcb_connection(), clientLeader(), atom(QXcbAtom::Atom_NET_STARTUP_ID));
     }
 }
 
@@ -822,8 +821,8 @@ xcb_timestamp_t QXcbConnection::getTimestamp()
 {
     // send a dummy event to myself to get the timestamp from X server.
     xcb_window_t window = rootWindow();
-    xcb_atom_t dummyAtom = atom(QXcbAtom::CLIP_TEMPORARY);
-    xcb_change_property(xcb_connection(), XCB_PROP_MODE_APPEND, window, dummyAtom,
+    xcb_atom_t dummyAtom = atom(QXcbAtom::Atom_QT_GET_TIMESTAMP);
+    xcb_change_property(xcb_connection(), XCB_PROP_MODE_REPLACE, window, dummyAtom,
                         XCB_ATOM_INTEGER, 32, 0, nullptr);
 
     connection()->flush();
@@ -855,8 +854,6 @@ xcb_timestamp_t QXcbConnection::getTimestamp()
     xcb_timestamp_t timestamp = pn->time;
     free(event);
 
-    xcb_delete_property(xcb_connection(), window, dummyAtom);
-
     return timestamp;
 }
 
@@ -923,7 +920,7 @@ xcb_window_t QXcbConnection::clientLeader()
         xcb_change_property(xcb_connection(),
                             XCB_PROP_MODE_REPLACE,
                             m_clientLeader,
-                            atom(QXcbAtom::WM_CLIENT_LEADER),
+                            atom(QXcbAtom::AtomWM_CLIENT_LEADER),
                             XCB_ATOM_WINDOW,
                             32,
                             1,
@@ -936,7 +933,7 @@ xcb_window_t QXcbConnection::clientLeader()
             xcb_change_property(xcb_connection(),
                                 XCB_PROP_MODE_REPLACE,
                                 m_clientLeader,
-                                atom(QXcbAtom::SM_CLIENT_ID),
+                                atom(QXcbAtom::AtomSM_CLIENT_ID),
                                 XCB_ATOM_STRING,
                                 8,
                                 session.size(),
@@ -1057,8 +1054,8 @@ bool QXcbConnection::isUserInputEvent(xcb_generic_event_t *event) const
 
     if (eventType == XCB_CLIENT_MESSAGE) {
         auto clientMessage = reinterpret_cast<const xcb_client_message_event_t *>(event);
-        if (clientMessage->format == 32 && clientMessage->type == atom(QXcbAtom::WM_PROTOCOLS))
-            if (clientMessage->data.data32[0] == atom(QXcbAtom::WM_DELETE_WINDOW))
+        if (clientMessage->format == 32 && clientMessage->type == atom(QXcbAtom::AtomWM_PROTOCOLS))
+            if (clientMessage->data.data32[0] == atom(QXcbAtom::AtomWM_DELETE_WINDOW))
                 isInputEvent = true;
     }
 
@@ -1143,13 +1140,6 @@ Qt::MouseButtons QXcbConnection::queryMouseButtons() const
     return translateMouseButtons(stateMask);
 }
 
-Qt::KeyboardModifiers QXcbConnection::queryKeyboardModifiers() const
-{
-    int stateMask = 0;
-    QXcbCursor::queryPointer(connection(), nullptr, nullptr, &stateMask);
-    return keyboard()->translateModifiers(stateMask);
-}
-
 QXcbGlIntegration *QXcbConnection::glIntegration() const
 {
     if (m_glIntegrationInitialized)
