@@ -6,14 +6,14 @@
 #include "qsizepolicy.h"
 #include "qwidget.h"
 
-#include "private/qvariant_p.h"
 #include <private/qmetatype_p.h>
 
 QT_BEGIN_NAMESPACE
 
 namespace {
 
-static const struct : QMetaTypeModuleHelper
+// NOLINTNEXTLINE(cppcoreguidelines-virtual-class-destructor): this is not a base class
+static constexpr struct : QMetaTypeModuleHelper
 {
     const QtPrivate::QMetaTypeInterface *interfaceForType(int type) const override {
         switch (type) {
