@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include "qapplication.h"
 #include "qcursor.h"
@@ -44,12 +8,16 @@
 #include "qscrollbar.h"
 #include "qstyle.h"
 #include "qstyleoption.h"
+#include "qstylepainter.h"
 #if QT_CONFIG(menu)
 #include "qmenu.h"
 #endif
-#include <QtCore/qelapsedtimer.h>
 
-#ifndef QT_NO_ACCESSIBILITY
+#include <QtCore/qelapsedtimer.h>
+#include <QtCore/qpointer.h>
+
+
+#if QT_CONFIG(accessibility)
 #include "qaccessible.h"
 #endif
 #include <limits.h>
@@ -187,7 +155,7 @@ QT_BEGIN_NAMESPACE
     Most GUI styles use the pageStep() value to calculate the size of the
     slider.
 
-    \sa QScrollArea, QSlider, QDial, QSpinBox, {fowler}{GUI Design Handbook: Scroll Bar}, {Sliders Example}
+    \sa QScrollArea, QSlider, QDial, QSpinBox, {Sliders Example}
 */
 
 bool QScrollBarPrivate::updateHoverControl(const QPoint &pos)
@@ -224,7 +192,9 @@ void QScrollBarPrivate::setTransient(bool value)
     if (transient != value) {
         transient = value;
         if (q->isVisible()) {
-            if (q->style()->styleHint(QStyle::SH_ScrollBar_Transient, nullptr, q))
+            QStyleOptionSlider opt;
+            q->initStyleOption(&opt);
+            if (q->style()->styleHint(QStyle::SH_ScrollBar_Transient, &opt, q))
                 q->update();
         } else if (!transient) {
             q->show();
@@ -235,7 +205,9 @@ void QScrollBarPrivate::setTransient(bool value)
 void QScrollBarPrivate::flash()
 {
     Q_Q(QScrollBar);
-    if (!flashed && q->style()->styleHint(QStyle::SH_ScrollBar_Transient, nullptr, q)) {
+    QStyleOptionSlider opt;
+    q->initStyleOption(&opt);
+    if (!flashed && q->style()->styleHint(QStyle::SH_ScrollBar_Transient, &opt, q)) {
         flashed = true;
         if (!q->isVisible())
             q->show();
@@ -319,7 +291,7 @@ void QScrollBar::initStyleOption(QStyleOptionSlider *option) const
     option->upsideDown = d->invertedAppearance;
     if (d->orientation == Qt::Horizontal)
         option->state |= QStyle::State_Horizontal;
-    if ((d->flashed || !d->transient) && style()->styleHint(QStyle::SH_ScrollBar_Transient, nullptr, this))
+    if ((d->flashed || !d->transient) && style()->styleHint(QStyle::SH_ScrollBar_Transient, option, this))
         option->state |= QStyle::State_On;
 }
 
@@ -376,7 +348,9 @@ void QScrollBarPrivate::init()
     invertedControls = true;
     pressedControl = hoverControl = QStyle::SC_None;
     pointerOutsidePressedControl = false;
-    transient = q->style()->styleHint(QStyle::SH_ScrollBar_Transient, nullptr, q);
+    QStyleOption opt;
+    opt.initFrom(q);
+    transient = q->style()->styleHint(QStyle::SH_ScrollBar_Transient, &opt, q);
     flashed = false;
     flashTimer = 0;
     q->setFocusPolicy(Qt::NoFocus);
@@ -470,12 +444,17 @@ bool QScrollBar::event(QEvent *event)
         if (const QHoverEvent *he = static_cast<const QHoverEvent *>(event))
             d_func()->updateHoverControl(he->position().toPoint());
         break;
-    case QEvent::StyleChange:
-        d_func()->setTransient(style()->styleHint(QStyle::SH_ScrollBar_Transient, nullptr, this));
+    case QEvent::StyleChange: {
+        QStyleOptionSlider opt;
+        initStyleOption(&opt);
+        d_func()->setTransient(style()->styleHint(QStyle::SH_ScrollBar_Transient, &opt, this));
         break;
+    }
     case QEvent::Timer:
         if (static_cast<QTimerEvent *>(event)->timerId() == d->flashTimer) {
-            if (d->flashed && style()->styleHint(QStyle::SH_ScrollBar_Transient, nullptr, this)) {
+            QStyleOptionSlider opt;
+            initStyleOption(&opt);
+            if (d->flashed && style()->styleHint(QStyle::SH_ScrollBar_Transient, &opt, this)) {
                 d->flashed = false;
                 update();
             }
@@ -525,7 +504,7 @@ void QScrollBar::wheelEvent(QWheelEvent *event)
 void QScrollBar::paintEvent(QPaintEvent *)
 {
     Q_D(QScrollBar);
-    QPainter p(this);
+    QStylePainter p(this);
     QStyleOptionSlider opt;
     initStyleOption(&opt);
     opt.subControls = QStyle::SC_All;
@@ -536,7 +515,7 @@ void QScrollBar::paintEvent(QPaintEvent *)
     } else {
         opt.activeSubControls = (QStyle::SubControl)d->hoverControl;
     }
-    style()->drawComplexControl(QStyle::CC_ScrollBar, &opt, &p, this);
+    p.drawComplexControl(QStyle::CC_ScrollBar, opt);
 }
 
 /*!
