@@ -1,10 +1,10 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
+#include <QtTest/private/qcomparisontesthelper_p.h>
 #include <QStandardPaths>
 #include <QScopeGuard>
-#include <QScopedValueRollback>
 
 #include <qfile.h>
 #include <qdir.h>
@@ -38,14 +38,15 @@
 #include <Foundation/Foundation.h>
 #endif
 
+#if QT_CONFIG(process)
+#include <QProcess>
+#endif
+
 #if defined(Q_OS_VXWORKS)
 #define Q_NO_SYMLINKS
 #endif
 
 #if defined(Q_OS_WIN)
-QT_BEGIN_NAMESPACE
-extern Q_CORE_EXPORT int qt_ntfs_permission_lookup;
-QT_END_NAMESPACE
 bool IsUserAdmin();
 #endif
 
@@ -176,6 +177,7 @@ private slots:
 
     void systemFiles();
 
+    void compareCompiles();
     void compare_data();
     void compare();
 
@@ -184,6 +186,7 @@ private slots:
 
     void fileTimes_data();
     void fileTimes();
+    void setFileTimes();
     void fakeFileTimes_data();
     void fakeFileTimes();
 
@@ -204,7 +207,7 @@ private slots:
 
     void isHidden_data();
     void isHidden();
-#if defined(Q_OS_MAC)
+#if defined(Q_OS_DARWIN)
     void isHiddenFromFinder();
 #endif
 
@@ -241,6 +244,12 @@ private slots:
     void nonExistingFile();
 
     void stdfilesystem();
+    void readSymLink();
+
+#if defined(Q_OS_DARWIN)
+    void fileSystemCaseSensitivity_data();
+    void fileSystemCaseSensitivity();
+#endif
 
 private:
     const QString m_currentDir;
@@ -353,7 +362,7 @@ void tst_QFileInfo::isDir_data()
     QFile::remove("brokenlink.lnk");
     QFile::remove("dummyfile");
     QFile file3("dummyfile");
-    file3.open(QIODevice::WriteOnly);
+    QVERIFY(file3.open(QIODevice::WriteOnly));
     if (file3.link("brokenlink.lnk")) {
         file3.remove();
         QFileInfo info3("brokenlink.lnk");
@@ -658,7 +667,7 @@ void tst_QFileInfo::canonicalFilePath()
         QFile file(QDir::currentPath());
         if (file.link(link)) {
             QFile tempfile("tempfile.txt");
-            tempfile.open(QIODevice::ReadWrite);
+            QVERIFY(tempfile.open(QIODevice::ReadWrite));
             tempfile.write("This file is generated by the QFileInfo autotest.");
             QVERIFY(tempfile.flush());
             tempfile.close();
@@ -764,7 +773,7 @@ void tst_QFileInfo::bundleName_data()
 
     QTest::newRow("root") << "/" << "";
     QTest::newRow("etc") << "/etc" << "";
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
     QTest::newRow("safari") << "/Applications/Safari.app" << "Safari";
 #endif
 }
@@ -995,6 +1004,11 @@ void tst_QFileInfo::systemFiles()
     QVERIFY(fi.birthTime() <= fi.lastModified());
 }
 
+void tst_QFileInfo::compareCompiles()
+{
+    QTestPrivate::testEqualityOperatorsCompile<QFileInfo>();
+}
+
 void tst_QFileInfo::compare_data()
 {
     QTest::addColumn<QString>("file1");
@@ -1021,8 +1035,8 @@ void tst_QFileInfo::compare_data()
         << m_sourceFile
 #if defined(Q_OS_WIN)
         << true;
-#elif defined(Q_OS_MAC)
-        << !pathconf(QDir::currentPath().toLatin1().constData(), _PC_CASE_SENSITIVE);
+#elif defined(Q_OS_DARWIN)
+        << (pathconf(QDir::currentPath().toLatin1().constData(), _PC_CASE_SENSITIVE) != 1);
 #else
         << false;
 #endif
@@ -1030,7 +1044,7 @@ void tst_QFileInfo::compare_data()
 
 void tst_QFileInfo::compare()
 {
-#if defined(Q_OS_MAC)
+#if defined(Q_OS_DARWIN)
     if (qstrcmp(QTest::currentDataTag(), "casesense1") == 0)
         QSKIP("Qt thinks all UNIX filesystems are case sensitive, see QTBUG-28246");
 #endif
@@ -1039,7 +1053,7 @@ void tst_QFileInfo::compare()
     QFETCH(QString, file2);
     QFETCH(bool, same);
     QFileInfo fi1(file1), fi2(file2);
-    QCOMPARE(fi1 == fi2, same);
+    QT_TEST_EQUALITY_OPS(fi1, fi2, same);
 }
 
 void tst_QFileInfo::consistent_data()
@@ -1107,8 +1121,8 @@ void tst_QFileInfo::fileTimes()
     {
         // try to guess if file times on this filesystem round to the second
         QFileInfo cwd(".");
-        if (cwd.lastModified().toMSecsSinceEpoch() % 1000 == 0
-                && cwd.lastRead().toMSecsSinceEpoch() % 1000 == 0) {
+        if (cwd.lastModified(QTimeZone::UTC).toMSecsSinceEpoch() % 1000 == 0
+                && cwd.lastRead(QTimeZone::UTC).toMSecsSinceEpoch() % 1000 == 0) {
             fsClockSkew = sleepTime = 1000;
 
             noAccessTime = qIsLikelyToBeFat(fileName);
@@ -1128,46 +1142,46 @@ void tst_QFileInfo::fileTimes()
     QDateTime birthTime, writeTime, metadataChangeTime, readTime;
 
     // --- Create file and write to it
-    beforeBirth = QDateTime::currentDateTime().addMSecs(-fsClockSkew);
+    beforeBirth = QDateTime::currentDateTimeUtc().addMSecs(-fsClockSkew);
     {
         QFile file(fileName);
         QVERIFY(file.open(QFile::WriteOnly | QFile::Text));
         QFileInfo fileInfo(fileName);
-        birthTime = fileInfo.birthTime();
+        birthTime = fileInfo.birthTime(QTimeZone::UTC);
         QVERIFY2(!birthTime.isValid() || birthTime > beforeBirth,
                  datePairString(birthTime, beforeBirth));
 
         QTest::qSleep(sleepTime);
-        beforeWrite = QDateTime::currentDateTime().addMSecs(-fsClockSkew);
+        beforeWrite = QDateTime::currentDateTimeUtc().addMSecs(-fsClockSkew);
         QTextStream ts(&file);
         ts << fileName << Qt::endl;
     }
     {
         QFileInfo fileInfo(fileName);
-        writeTime = fileInfo.lastModified();
+        writeTime = fileInfo.lastModified(QTimeZone::UTC);
         QVERIFY2(writeTime > beforeWrite, datePairString(writeTime, beforeWrite));
-        QCOMPARE(fileInfo.birthTime(), birthTime); // mustn't have changed
+        QCOMPARE(fileInfo.birthTime(QTimeZone::UTC), birthTime); // mustn't have changed
     }
 
     // --- Change the file's metadata
     QTest::qSleep(sleepTime);
-    beforeMetadataChange = QDateTime::currentDateTime().addMSecs(-fsClockSkew);
+    beforeMetadataChange = QDateTime::currentDateTimeUtc().addMSecs(-fsClockSkew);
     {
         QFile file(fileName);
         file.setPermissions(file.permissions());
     }
     {
         QFileInfo fileInfo(fileName);
-        metadataChangeTime = fileInfo.metadataChangeTime();
+        metadataChangeTime = fileInfo.metadataChangeTime(QTimeZone::UTC);
         QVERIFY2(metadataChangeTime > beforeMetadataChange,
                  datePairString(metadataChangeTime, beforeMetadataChange));
         QVERIFY(metadataChangeTime >= writeTime); // not all filesystems can store both times
-        QCOMPARE(fileInfo.birthTime(), birthTime); // mustn't have changed
+        QCOMPARE(fileInfo.birthTime(QTimeZone::UTC), birthTime); // mustn't have changed
     }
 
     // --- Read the file
     QTest::qSleep(sleepTime);
-    beforeRead = QDateTime::currentDateTime().addMSecs(-fsClockSkew);
+    beforeRead = QDateTime::currentDateTimeUtc().addMSecs(-fsClockSkew);
     {
         QFile file(fileName);
         QVERIFY(file.open(QFile::ReadOnly | QFile::Text));
@@ -1177,9 +1191,9 @@ void tst_QFileInfo::fileTimes()
     }
 
     QFileInfo fileInfo(fileName);
-    readTime = fileInfo.lastRead();
-    QCOMPARE(fileInfo.lastModified(), writeTime); // mustn't have changed
-    QCOMPARE(fileInfo.birthTime(), birthTime); // mustn't have changed
+    readTime = fileInfo.lastRead(QTimeZone::UTC);
+    QCOMPARE(fileInfo.lastModified(QTimeZone::UTC), writeTime); // mustn't have changed
+    QCOMPARE(fileInfo.birthTime(QTimeZone::UTC), birthTime); // mustn't have changed
     QVERIFY(readTime.isValid());
 
 #if defined(Q_OS_QNX) || defined(Q_OS_ANDROID)
@@ -1191,8 +1205,8 @@ void tst_QFileInfo::fileTimes()
     const auto disabledAccessTimes =
         QWinRegistryKey(HKEY_LOCAL_MACHINE,
                         LR"(SYSTEM\CurrentControlSet\Control\FileSystem)")
-        .dwordValue(L"NtfsDisableLastAccessUpdate");
-    if (disabledAccessTimes.second && disabledAccessTimes.first != 0)
+        .value<DWORD>(L"NtfsDisableLastAccessUpdate");
+    if (disabledAccessTimes.value_or(0) != 0)
         noAccessTime = true;
 #endif
 
@@ -1203,6 +1217,28 @@ void tst_QFileInfo::fileTimes()
     QVERIFY(writeTime < beforeRead);
 }
 
+void tst_QFileInfo::setFileTimes()
+{
+    QByteArray data("OLE\nOLE\nOLE");
+    QTemporaryFile file;
+
+    QVERIFY(file.open());
+    QCOMPARE(file.write(data), data.size());
+    QCOMPARE(file.size(), data.size());
+
+    QDateTime before = QDateTime::currentDateTimeUtc().addMSecs(-5000);
+
+    QVERIFY(file.setFileTime(before, QFile::FileModificationTime));
+    const QDateTime mtime = file.fileTime(QFile::FileModificationTime).toUTC();
+    if (mtime.time().msec() == 0)
+    {
+        const QTime beforeTime = before.time();
+        const QTime beforeTimeWithMSCutOff{beforeTime.hour(), beforeTime.minute(), beforeTime.second(), 0};
+        before.setTime(beforeTimeWithMSCutOff);
+    }
+    QCOMPARE(mtime, before);
+}
+
 void tst_QFileInfo::fakeFileTimes_data()
 {
     QTest::addColumn<QDateTime>("when");
@@ -1220,7 +1256,7 @@ void tst_QFileInfo::fakeFileTimes()
     QFETCH(QDateTime, when);
 
     QFile file("faketimefile.txt");
-    file.open(QIODevice::WriteOnly);
+    QVERIFY(file.open(QIODevice::WriteOnly));
     file.write("\n", 1);
     file.close();
 
@@ -1229,12 +1265,12 @@ void tst_QFileInfo::fakeFileTimes()
       the file is open at the time.  Of course, when writing, close() changes
       modification time, so need to re-open for read in order to setFileTime().
      */
-    file.open(QIODevice::ReadOnly);
+    QVERIFY(file.open(QIODevice::ReadOnly));
     bool ok = file.setFileTime(when, QFileDevice::FileModificationTime);
     file.close();
 
     if (ok)
-        QCOMPARE(QFileInfo(file.fileName()).lastModified(), when);
+        QCOMPARE(QFileInfo(file.fileName()).lastModified(QTimeZone::UTC), when);
     else
         QSKIP("Unable to set file metadata to contrived values");
 }
@@ -1251,7 +1287,7 @@ void tst_QFileInfo::isSymLink_data()
     QVERIFY(file1.link("link.lnk"));
 
     QFile file2("dummyfile");
-    file2.open(QIODevice::WriteOnly);
+    QVERIFY(file2.open(QIODevice::WriteOnly));
     QVERIFY(file2.link("brokenlink.lnk"));
     file2.remove();
 
@@ -1262,6 +1298,7 @@ void tst_QFileInfo::isSymLink_data()
     QTest::newRow("existent file") << m_sourceFile << false << "";
     QTest::newRow("link") << "link.lnk" << true << QFileInfo(m_sourceFile).absoluteFilePath();
     QTest::newRow("broken link") << "brokenlink.lnk" << true << QFileInfo("dummyfile").absoluteFilePath();
+    QTest::newRow("nonexistent") << "thispathdoesntexist.lnk" << false << QString();
 
 #ifndef Q_OS_WIN
     QDir::current().mkdir("relative");
@@ -1387,6 +1424,8 @@ void tst_QFileInfo::isSymbolicLink_data()
         << regularFile.fileName() << false;
     QTest::newRow("directory")
         << QDir::currentPath() << false;
+    QTest::newRow("nonexistent")
+        << "thispathdoesntexist.lnk" << false;
 
 #ifndef Q_NO_SYMLINKS
 #if defined(Q_OS_WIN)
@@ -1437,7 +1476,7 @@ void tst_QFileInfo::link_data()
 
     QFile file1(m_sourceFile);
     QFile file2("dummyfile");
-    file2.open(QIODevice::WriteOnly);
+    QVERIFY(file2.open(QIODevice::WriteOnly));
 
     QTest::newRow("existent file") << m_sourceFile << false << false << "";
 #if defined(Q_OS_WIN)
@@ -1505,9 +1544,9 @@ void tst_QFileInfo::isHidden_data()
 {
     QTest::addColumn<QString>("path");
     QTest::addColumn<bool>("isHidden");
-    foreach (const QFileInfo& info, QDir::drives()) {
+    const auto drives = QDir::drives();
+    for (const QFileInfo& info : drives)
         QTest::newRow(qPrintable("drive." + info.path())) << info.path() << false;
-    }
 
 #if defined(Q_OS_WIN)
     QVERIFY(QDir("./hidden-directory").exists() || QDir().mkdir("./hidden-directory"));
@@ -1522,14 +1561,14 @@ void tst_QFileInfo::isHidden_data()
     QTest::newRow("/path/to/.hidden-directory/..") << QDir::currentPath() + QString("/.hidden-directory/..") << true;
 #endif
 
-#if defined(Q_OS_MAC)
+#if defined(Q_OS_DARWIN)
     // /bin has the hidden attribute on OS X
     QTest::newRow("/bin/") << QString::fromLatin1("/bin/") << true;
 #elif !defined(Q_OS_WIN)
     QTest::newRow("/bin/") << QString::fromLatin1("/bin/") << false;
 #endif
 
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
     QTest::newRow("mac_etc") << QString::fromLatin1("/etc") << true;
     QTest::newRow("mac_private_etc") << QString::fromLatin1("/private/etc") << false;
     QTest::newRow("mac_Applications") << QString::fromLatin1("/Applications") << false;
@@ -1545,7 +1584,7 @@ void tst_QFileInfo::isHidden()
     QCOMPARE(fi.isHidden(), isHidden);
 }
 
-#if defined(Q_OS_MAC)
+#if defined(Q_OS_DARWIN)
 void tst_QFileInfo::isHiddenFromFinder()
 {
     const char *filename = "test_foobar.txt";
@@ -1571,7 +1610,7 @@ void tst_QFileInfo::isBundle_data()
     QTest::addColumn<QString>("path");
     QTest::addColumn<bool>("isBundle");
     QTest::newRow("root") << QString::fromLatin1("/") << false;
-#ifdef Q_OS_MAC
+#ifdef Q_OS_DARWIN
     QTest::newRow("mac_Applications") << QString::fromLatin1("/Applications") << false;
     QTest::newRow("mac_Applications") << QString::fromLatin1("/Applications/Safari.app") << true;
 #endif
@@ -1625,14 +1664,14 @@ void tst_QFileInfo::refresh()
     file.flush();
 
     QFileInfo info(file);
-    QDateTime lastModified = info.lastModified();
+    QDateTime lastModified = info.lastModified(QTimeZone::UTC);
     QCOMPARE(info.size(), qint64(7));
 
     QTest::qSleep(sleepTime);
 
     QCOMPARE(file.write("JOJOJO"), qint64(6));
     file.flush();
-    QCOMPARE(info.lastModified(), lastModified);
+    QCOMPARE(info.lastModified(QTimeZone::UTC), lastModified);
 
     QCOMPARE(info.size(), qint64(7));
 #if defined(Q_OS_WIN)
@@ -1640,7 +1679,7 @@ void tst_QFileInfo::refresh()
 #endif
     info.refresh();
     QCOMPARE(info.size(), qint64(13));
-    QVERIFY(info.lastModified() > lastModified);
+    QVERIFY(info.lastModified(QTimeZone::UTC) > lastModified);
 
     QFileInfo info2 = info;
     QCOMPARE(info2.size(), info.size());
@@ -1874,13 +1913,28 @@ void tst_QFileInfo::brokenShortcut()
 void tst_QFileInfo::isWritable()
 {
     QFile tempfile("tempfile.txt");
-    tempfile.open(QIODevice::WriteOnly);
+    QVERIFY(tempfile.open(QIODevice::WriteOnly));
     tempfile.write("This file is generated by the QFileInfo autotest.");
     tempfile.close();
 
+    // isWritable returns true when file is writable
     QVERIFY(QFileInfo("tempfile.txt").isWritable());
+
+    QFile::setPermissions("tempfile.txt",
+                          QFileDevice::ReadOwner | QFileDevice::ReadUser | QFileDevice::ReadGroup);
+
+    // isWritable returns false when read-only, unless root
+    bool isRoot = false;
+#if defined (Q_OS_UNIX)
+    isRoot = (geteuid() == 0);
+#endif
+    QCOMPARE(QFileInfo("tempfile.txt").isWritable(), isRoot);
+
     tempfile.remove();
 
+    // isWritable returns false when file does not exist
+    QVERIFY(!QFileInfo("tempfile.txt").isWritable());
+
 #if defined(Q_OS_WIN)
     QFileInfo fi("c:\\pagefile.sys");
     QVERIFY2(fi.exists(), msgDoesNotExist(fi.absoluteFilePath()).constData());
@@ -1888,8 +1942,7 @@ void tst_QFileInfo::isWritable()
 #endif
 
 #if defined (Q_OS_WIN)
-    QScopedValueRollback<int> ntfsMode(qt_ntfs_permission_lookup);
-    qt_ntfs_permission_lookup = 1;
+    QNtfsPermissionCheckGuard permissionGuard;
     QFileInfo fi2(QFile::decodeName(qgetenv("SystemRoot") + "/system.ini"));
     QVERIFY(fi2.exists());
     QCOMPARE(fi2.isWritable(), IsUserAdmin());
@@ -1995,7 +2048,7 @@ private:
 
 void tst_QFileInfo::testDecomposedUnicodeNames()
 {
-#ifndef Q_OS_MAC
+#ifndef Q_OS_DARWIN
     QSKIP("This is a OS X only test (unless you know more about filesystems, then maybe you should try it ;)");
 #else
     QFETCH(QString, filePath);
@@ -2139,7 +2192,7 @@ void tst_QFileInfo::owner()
                 NetApiBufferFree(pBuf);
         }
     }
-    qt_ntfs_permission_lookup = 1;
+    QNtfsPermissionCheckGuard permissionGuard;
 #endif
     if (userName.isEmpty())
         QSKIP("Can't retrieve the user name");
@@ -2156,9 +2209,6 @@ void tst_QFileInfo::owner()
     QCOMPARE(fi.owner(), userName);
 
     QFile::remove(fileName);
-#if defined(Q_OS_WIN)
-    qt_ntfs_permission_lookup = 0;
-#endif
 }
 
 void tst_QFileInfo::group()
@@ -2242,10 +2292,10 @@ static void stateCheck(const QFileInfo &info, const QString &dirname, const QStr
 
     QCOMPARE(info.permissions(), QFile::Permissions());
 
-    QVERIFY(!info.birthTime().isValid());
-    QVERIFY(!info.metadataChangeTime().isValid());
-    QVERIFY(!info.lastRead().isValid());
-    QVERIFY(!info.lastModified().isValid());
+    QVERIFY(!info.birthTime(QTimeZone::UTC).isValid());
+    QVERIFY(!info.metadataChangeTime(QTimeZone::UTC).isValid());
+    QVERIFY(!info.lastRead(QTimeZone::UTC).isValid());
+    QVERIFY(!info.lastModified(QTimeZone::UTC).isValid());
 };
 
 void tst_QFileInfo::invalidState_data()
@@ -2294,13 +2344,15 @@ void tst_QFileInfo::stdfilesystem()
         // We compare using absoluteFilePath since QFileInfo::operator== ends up using
         // canonicalFilePath which evaluates to empty-string for non-existent paths causing
         // these tests to always succeed.
-#define COMPARE_CONSTRUCTION(filepath)                                                 \
-        QCOMPARE(QFileInfo(fs::path(filepath)).absoluteFilePath(),                     \
-                 QFileInfo(QString::fromLocal8Bit(filepath)).absoluteFilePath());      \
-        QCOMPARE(QFileInfo(base, fs::path(filepath)).absoluteFilePath(),               \
-                 QFileInfo(base, QString::fromLocal8Bit(filepath)).absoluteFilePath())
-
         QDir base{ "../" }; // Used for the QFileInfo(QDir, <path>) ctor
+        auto doCompare = [&base](const char *filepath) {
+            QCOMPARE(QFileInfo(fs::path(filepath)).absoluteFilePath(),
+                     QFileInfo(QString::fromLocal8Bit(filepath)).absoluteFilePath());
+            QCOMPARE(QFileInfo(base, fs::path(filepath)).absoluteFilePath(),
+                     QFileInfo(base, QString::fromLocal8Bit(filepath)).absoluteFilePath());
+        };
+#define COMPARE_CONSTRUCTION(filepath)                                                 \
+    doCompare(filepath); if (QTest::currentTestFailed()) return
 
         COMPARE_CONSTRUCTION("./file");
 
@@ -2313,7 +2365,22 @@ void tst_QFileInfo::stdfilesystem()
         COMPARE_CONSTRUCTION("/path/TO/file.txt");
         COMPARE_CONSTRUCTION("./path/TO/file.txt");
         COMPARE_CONSTRUCTION("../file.txt");
+#if !(defined(__GLIBCXX__) && defined(Q_OS_WIN32))
+        // libstdc++ bug on Windows - https://gcc.gnu.org/bugzilla/show_bug.cgi?id=111244
         COMPARE_CONSTRUCTION("./filæ.txt");
+#endif
+
+        // Test unicode strings
+        QCOMPARE(QFileInfo(fs::path(u"./filæ.txt")).absoluteFilePath(),
+                 QFileInfo(u"./filæ.txt"_s).absoluteFilePath());
+        QCOMPARE(QFileInfo(base, fs::path(u"./filæ.txt")).absoluteFilePath(),
+                 QFileInfo(base, u"./filæ.txt"_s).absoluteFilePath());
+#ifdef __cpp_char8_t
+        QCOMPARE(QFileInfo(fs::path(u8"./filæ.txt")).absoluteFilePath(),
+                 QFileInfo(u8"./filæ.txt").absoluteFilePath());
+        QCOMPARE(QFileInfo(base, fs::path(u8"./filæ.txt")).absoluteFilePath(),
+                 QFileInfo(base, u8"./filæ.txt").absoluteFilePath());
+#endif
 
 #undef COMPARE_CONSTRUCTION
         {
@@ -2375,5 +2442,83 @@ void tst_QFileInfo::stdfilesystem()
 #endif
 }
 
+void tst_QFileInfo::readSymLink()
+{
+    QString symLinkName("./a.link");
+    const auto tidier = qScopeGuard([symLinkName]() { QFile::remove(symLinkName); });
+
+#ifdef Q_OS_WIN
+    QVERIFY2(CreateSymbolicLink(L"a.link", L"..\\..\\a",   SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE)
+             != 0,
+             "Failed to create symlink for test");
+#else
+    QVERIFY2(QFile::link("../../a", symLinkName), "Failed to create symlink for test");
+#endif
+    QFileInfo info(symLinkName);
+    QCOMPARE(info.readSymLink(), QString("../../a"));
+}
+
+#if defined(Q_OS_DARWIN)
+void tst_QFileInfo::fileSystemCaseSensitivity_data()
+{
+    QTest::addColumn<QString>("fileSystemType");
+    QTest::addColumn<QString>("volumeName");
+    QTest::addColumn<Qt::CaseSensitivity>("caseSensitivity");
+
+    QTest::newRow("APFS") << "APFS" << "apfs" << Qt::CaseInsensitive;
+    QTest::newRow("APFS case-sensitive") << "Case-sensitive APFS" << "apfs_case_sensitive" << Qt::CaseSensitive;
+    QTest::newRow("HFS+") << "HFS+" << "hfs" << Qt::CaseInsensitive;
+    QTest::newRow("HFS+ case-sensitive") << "Case-sensitive HFS+" << "hfs_case_sensitive" << Qt::CaseSensitive;
+    QTest::newRow("FAT32") << "MS-DOS FAT32" << "fat32" << Qt::CaseInsensitive;
+    QTest::newRow("ExFAT") << "ExFAT" << "exfat" << Qt::CaseInsensitive;
+}
+
+void tst_QFileInfo::fileSystemCaseSensitivity()
+{
+#if !QT_CONFIG(process)
+    QSKIP("No QProcess available");
+#else
+    QTemporaryDir tmpDir;
+    QVERIFY2(tmpDir.isValid(), qPrintable(tmpDir.errorString()));
+
+    QFETCH(QString, fileSystemType);
+    QFETCH(QString, volumeName);
+
+    QString imageName = tmpDir.filePath(QString("%2.sparseimage").arg(volumeName));
+
+    QVERIFY(QProcess::execute("hdiutil", { "create", "-quiet",
+        "-type", "SPARSE", "-size", "50m", "-fs", fileSystemType,
+        "-volname", volumeName, imageName }) == 0);
+
+    QVERIFY(QProcess::execute("hdiutil", { "attach", "-quiet",
+        "-mountroot", tmpDir.path(), imageName }) == 0);
+
+    QDir mountPoint(tmpDir.filePath(volumeName));
+    QVERIFY(mountPoint.exists());
+
+    auto cleanup = qScopeGuard([&] {
+        QVERIFY(QProcess::execute("hdiutil", { "detach",
+            "-quiet", mountPoint.absolutePath() }) == 0);
+    });
+
+    QFileInfo lowerCase(mountPoint.filePath("foo"));
+    {
+        QFile file(lowerCase.filePath());
+        QVERIFY(file.open(QFile::WriteOnly));
+    }
+
+    QFileInfo upperCase(mountPoint.filePath("FOO"));
+    {
+        QFile file(upperCase.filePath());
+        QVERIFY(file.open(QFile::WriteOnly));
+    }
+
+    QFETCH(Qt::CaseSensitivity, caseSensitivity);
+    QCOMPARE(lowerCase == upperCase, caseSensitivity == Qt::CaseInsensitive);
+
+#endif // QT_CONFIG(process)
+}
+#endif // defined(Q_OS_DARWIN)
+
 QTEST_MAIN(tst_QFileInfo)
 #include "tst_qfileinfo.moc"
