@@ -1,10 +1,12 @@
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QByteArrayView>
 
 #include <QTest>
 
+#include <QtCore/qspan.h>
+
 // for negative testing (can't convert from)
 #include <deque>
 #include <list>
@@ -76,6 +78,26 @@ static_assert(CanConvert<const std::array<char, 1> >);
 static_assert(CanConvert<      std::array<char, 1>&>);
 static_assert(CanConvert<const std::array<char, 1>&>);
 
+static_assert(CanConvert<      QSpan<char> >);
+static_assert(CanConvert<const QSpan<char> >);
+static_assert(CanConvert<      QSpan<char>&>);
+static_assert(CanConvert<const QSpan<char>&>);
+
+static_assert(CanConvert<      QSpan<char, 42> >);
+static_assert(CanConvert<const QSpan<char, 42> >);
+static_assert(CanConvert<      QSpan<char, 42>&>);
+static_assert(CanConvert<const QSpan<char, 42>&>);
+
+static_assert(CanConvert<      QSpan<std::byte> >);
+static_assert(CanConvert<const QSpan<std::byte> >);
+static_assert(CanConvert<      QSpan<std::byte>&>);
+static_assert(CanConvert<const QSpan<std::byte>&>);
+
+static_assert(CanConvert<      QSpan<std::byte, 42> >);
+static_assert(CanConvert<const QSpan<std::byte, 42> >);
+static_assert(CanConvert<      QSpan<std::byte, 42>&>);
+static_assert(CanConvert<const QSpan<std::byte, 42>&>);
+
 static_assert(!CanConvert<std::deque<char>>);
 static_assert(!CanConvert<std::list<char>>);
 
@@ -171,6 +193,7 @@ private slots:
 
     void comparison() const;
     void compare() const;
+    void std_stringview_conversion();
 
 private:
     template <typename Data>
@@ -196,6 +219,10 @@ void tst_QByteArrayView::constExpr() const
         static_assert(bv.isEmpty());
         static_assert(bv.data() == nullptr);
 
+        constexpr std::string_view sv = bv;
+        static_assert(sv.size() == 0);
+        static_assert(sv.data() == nullptr);
+
         constexpr QByteArrayView bv2(bv.data(), bv.data() + bv.size());
         static_assert(bv2.isNull());
         static_assert(bv2.empty());
@@ -208,6 +235,10 @@ void tst_QByteArrayView::constExpr() const
         static_assert(bv.isEmpty());
         static_assert(bv.data() != nullptr);
 
+        constexpr std::string_view sv = bv;
+        static_assert(sv.size() == bv.size());
+        static_assert(sv.data() == bv.data());
+
         constexpr QByteArrayView bv2(bv.data(), bv.data() + bv.size());
         static_assert(!bv2.isNull());
         static_assert(bv2.empty());
@@ -240,15 +271,22 @@ void tst_QByteArrayView::constExpr() const
         static_assert(bv.rbegin()  != bv.rend());
         static_assert(bv.crbegin() != bv.crend());
 
+        constexpr std::string_view sv = bv;
+        static_assert(sv.size() == bv.size());
+        static_assert(sv.data() == bv.data());
+#ifdef AMBIGUOUS_CALL // QTBUG-108805
+        static_assert(sv == bv);
+        static_assert(bv == sv);
+#endif
+
         constexpr QByteArrayView bv2(bv.data(), bv.data() + bv.size());
         static_assert(!bv2.isNull());
         static_assert(!bv2.empty());
         static_assert(bv2.size() == 5);
     }
-#if !defined(Q_CC_GNU) || defined(Q_CC_CLANG)
+#if !defined(Q_CC_GNU_ONLY) || !defined(QT_SANITIZE_UNDEFINED)
     // Below checks are disabled because of a compilation issue with GCC and
     // -fsanitize=undefined. See https://gcc.gnu.org/bugzilla/show_bug.cgi?id=71962.
-    // Note: Q_CC_GNU is also defined for Clang, so we need to check that too.
     {
         static constexpr char hello[] = "Hello";
         constexpr QByteArrayView bv(hello);
@@ -265,6 +303,13 @@ void tst_QByteArrayView::constExpr() const
         static_assert(bv.at(4)   == 'o');
         static_assert(bv.back()  == 'o');
         static_assert(bv.last()  == 'o');
+
+        constexpr std::string_view sv = bv;
+        static_assert(bv.size() == sv.size());
+#ifdef AMBIGUOUS_CALL // QTBUG-108805
+        static_assert(bv == sv);
+        static_assert(sv == bv);
+#endif
     }
     {
         static constexpr char hello[] = { 'H', 'e', 'l', 'l', 'o' };
@@ -282,6 +327,16 @@ void tst_QByteArrayView::constExpr() const
         static_assert(bv.at(4)   == 'o');
         static_assert(bv.back()  == 'o');
         static_assert(bv.last()  == 'o');
+
+        constexpr auto bv2 = QByteArrayView::fromArray(hello);
+        QCOMPARE_EQ(bv, bv2);
+
+        constexpr std::string_view sv = bv;
+        static_assert(bv.size() == sv.size());
+#ifdef AMBIGUOUS_CALL // QTBUG-108805
+        static_assert(bv == sv);
+        static_assert(sv == bv);
+#endif
     }
 #endif
     {
@@ -290,6 +345,42 @@ void tst_QByteArrayView::constExpr() const
         static_assert(bv.isNull());
         static_assert(bv.isEmpty());
         static_assert(bv.size() == 0);
+
+        constexpr std::string_view sv = bv;
+        static_assert(sv.size() == 0);
+        static_assert(sv.data() == nullptr);
+    }
+    {
+        constexpr QByteArrayView bv(QLatin1StringView("Hello"));
+        static_assert(bv.size() == 5);
+        static_assert(!bv.empty());
+        static_assert(!bv.isEmpty());
+        static_assert(!bv.isNull());
+        static_assert(*bv.data() == 'H');
+        static_assert(bv[0]      == 'H');
+        static_assert(bv.at(0)   == 'H');
+        static_assert(bv.front() == 'H');
+        static_assert(bv.first() == 'H');
+        static_assert(bv[4]      == 'o');
+        static_assert(bv.at(4)   == 'o');
+        static_assert(bv.back()  == 'o');
+        static_assert(bv.last()  == 'o');
+    }
+    {
+        constexpr QByteArrayView bv(QUtf8StringView("Hello"));
+        static_assert(bv.size() == 5);
+        static_assert(!bv.empty());
+        static_assert(!bv.isEmpty());
+        static_assert(!bv.isNull());
+        static_assert(*bv.data() == 'H');
+        static_assert(bv[0]      == 'H');
+        static_assert(bv.at(0)   == 'H');
+        static_assert(bv.front() == 'H');
+        static_assert(bv.first() == 'H');
+        static_assert(bv[4]      == 'o');
+        static_assert(bv.at(4)   == 'o');
+        static_assert(bv.back()  == 'o');
+        static_assert(bv.last()  == 'o');
     }
 }
 
@@ -349,7 +440,7 @@ void tst_QByteArrayView::fromArray() const
 {
     static constexpr char hello[] = "Hello\0abc\0\0.";
 
-    constexpr QByteArrayView bv = QByteArrayView::fromArray(hello);
+    const QByteArrayView bv = QByteArrayView::fromArray(hello);
     QCOMPARE(bv.size(), 13);
     QVERIFY(!bv.empty());
     QVERIFY(!bv.isEmpty());
@@ -432,7 +523,7 @@ void tst_QByteArrayView::fromQByteArray() const
     QByteArray empty = "";
 
     QVERIFY(QByteArrayView(null).isNull());
-    QVERIFY(!qToByteArrayViewIgnoringNull(null).isNull());
+    QVERIFY(qToByteArrayViewIgnoringNull(null).isNull());
 
     QVERIFY(QByteArrayView(null).isEmpty());
     QVERIFY(qToByteArrayViewIgnoringNull(null).isEmpty());
@@ -450,7 +541,7 @@ namespace help {
 template <typename T>
 size_t size(const T &t) { return size_t(t.size()); }
 template <typename T>
-size_t size(const T *t) { return std::char_traits<T>::length(t); }
+size_t size(const T *t) { return QtPrivate::lengthHelperPointer(t); }
 
 template <typename T>
 decltype(auto)             cbegin(const T &t) { return t.begin(); }
@@ -632,5 +723,30 @@ void tst_QByteArrayView::compare() const
     QVERIFY(alpha.compare(beta, Qt::CaseSensitive) > 0);
 }
 
+void tst_QByteArrayView::std_stringview_conversion()
+{
+    static_assert(std::is_convertible_v<QByteArrayView, std::string_view>);
+
+    QByteArrayView bav;
+    std::string_view sv(bav);
+    QCOMPARE(sv, std::string_view());
+
+    bav = "";
+    sv = bav;
+    QCOMPARE(bav.size(), 0);
+    QCOMPARE(sv.size(), size_t(0));
+    QCOMPARE(sv, std::string_view());
+
+    bav = "Hello";
+    sv = bav;
+    QCOMPARE(sv, std::string_view("Hello"));
+
+    bav = QByteArrayView::fromArray("Hello\0world");
+    sv = bav;
+    QCOMPARE(bav.size(), 12);
+    QCOMPARE(sv.size(), size_t(12));
+    QCOMPARE(sv, std::string_view("Hello\0world", 12));
+}
+
 QTEST_APPLESS_MAIN(tst_QByteArrayView)
 #include "tst_qbytearrayview.moc"
