@@ -1,14 +1,24 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-#include <QCoreApplication>
-#include <QDebug>
-#include <QElapsedTimer>
-#include <QSignalSpy>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
+#include <QtTest/qtest.h>
+#include <QtTest/qsignalspy.h>
+
+#include <QtConcurrent/qtconcurrentfilter.h>
+#include <QtConcurrent/qtconcurrentmap.h>
+#include <QtConcurrent/qtconcurrentrun.h>
+
+#include <QtCore/qcoreapplication.h>
+#include <QtCore/qdebug.h>
+#include <QtCore/qelapsedtimer.h>
+#include <QtCore/qfuturesynchronizer.h>
+#include <QtCore/qfuturewatcher.h>
+#include <QtCore/qset.h>
 
-#include <QtConcurrent>
 #include <private/qfutureinterface_p.h>
 
 using namespace QtConcurrent;
+using namespace std::chrono_literals;
 
 #include <QTest>
 
@@ -25,6 +35,7 @@ private slots:
     void cancelAndFinish();
     void resultAt();
     void resultReadyAt();
+    void orderedResultReadyAt();
     void futureSignals();
     void watchFinishedFuture();
     void watchCanceledFuture();
@@ -279,6 +290,28 @@ void tst_QFutureWatcher::resultReadyAt()
     QVERIFY(resultSpy.wait());
 }
 
+void tst_QFutureWatcher::orderedResultReadyAt()
+{
+    for (int i = 0; i < 1000; ++i) {
+        QObject context;
+        QFuture<QString> f = run([](QPromise<QString> &fi) {
+            fi.addResult("First");
+            fi.addResult("Second");
+        });
+        QList<int> actualIndices;
+
+        QFutureWatcher<QString> watcher;
+        connect(&watcher, &QFutureWatcherBase::resultReadyAt, &context,
+                [&actualIndices](int index) { actualIndices.append(index); });
+        watcher.setFuture(f);
+        f.waitForFinished();
+        QCoreApplication::processEvents();
+        const QList<int> expectedIndices{0, 1};
+        QCOMPARE(actualIndices.size(), expectedIndices.size());
+        QCOMPARE(actualIndices, expectedIndices);
+    }
+}
+
 class SignalSlotObject : public QObject
 {
 Q_OBJECT
@@ -900,13 +933,13 @@ QT_WARNING_POP
     QFuture<int> future = QtConcurrent::mapped(&pool, values, [&](int value) {
         ++count;
         // Sleep, to make sure not all threads will start at once.
-        QThread::msleep(50);
+        QThread::sleep(50ms);
         return value;
     });
     watcher.setFuture(future);
 
     // Allow some threads to start before suspending.
-    QThread::msleep(200);
+    QThread::sleep(200ms);
 
     watcher.suspend();
     watcher.suspend();
@@ -921,7 +954,7 @@ QT_WARNING_POP
     QCOMPARE(resultReadyAfterPaused, count);
 
     // Make sure no more results are reported before resuming.
-    QThread::msleep(200);
+    QThread::sleep(200ms);
     QCOMPARE(resultReadyAfterPaused, resultReadySpy.size());
     resultReadySpy.clear();
 
@@ -1139,7 +1172,7 @@ public:
 
 void tst_QFutureWatcher::warnRace()
 {
-#ifndef Q_OS_MAC //I don't know why it is not working on mac
+#ifndef Q_OS_DARWIN // I don't know why it is not working on mac
 #ifndef QT_NO_DEBUG
     QTest::ignoreMessage(QtWarningMsg, "QFutureWatcher::connect: connecting after calling setFuture() is likely to produce race");
 #endif
