@@ -1,5 +1,5 @@
 // Copyright (C) 2017 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtGui/QVulkanInstance>
 #include <QtGui/QVulkanFunctions>
@@ -27,10 +27,6 @@ private slots:
 
 void tst_QVulkan::vulkanInstance()
 {
-#ifdef Q_OS_ANDROID
-    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
-        QSKIP("Fails on Android 12 (QTBUG-111236)");
-#endif
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
@@ -67,10 +63,6 @@ void tst_QVulkan::vulkanInstance()
 
 void tst_QVulkan::vulkanCheckSupported()
 {
-#ifdef Q_OS_ANDROID
-    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
-        QSKIP("Fails on Android 12 (QTBUG-111236)");
-#endif
     // Test the early calls to supportedLayers/extensions/apiVersion that need
     // the library and some basics, but do not initialize the instance.
     QVulkanInstance inst;
@@ -89,8 +81,8 @@ void tst_QVulkan::vulkanCheckSupported()
 
     if (inst.create()) { // skip the rest when Vulkan is not supported at all
         QVERIFY(!ve.isEmpty());
-        QVERIFY(ve == inst.supportedExtensions());
-        QVERIFY(supportedApiVersion.majorVersion() >= 1);
+        QCOMPARE(ve, inst.supportedExtensions());
+        QCOMPARE_GE(supportedApiVersion.majorVersion(), 1);
     }
 }
 
@@ -214,10 +206,6 @@ void tst_QVulkan::vulkanPlainWindow()
 
 void tst_QVulkan::vulkanVersionRequest()
 {
-#ifdef Q_OS_ANDROID
-    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
-        QSKIP("Fails on Android 12 (QTBUG-111236)");
-#endif
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
@@ -264,10 +252,6 @@ static void waitForUnexposed(QWindow *w)
 
 void tst_QVulkan::vulkanWindow()
 {
-#ifdef Q_OS_ANDROID
-    if (QNativeInterface::QAndroidApplication::sdkVersion() >= 31)
-        QSKIP("Fails on Android 12 (QTBUG-111236)");
-#endif
     QVulkanInstance inst;
     if (!inst.create())
         QSKIP("Vulkan init failed; skip");
@@ -314,7 +298,7 @@ void tst_QVulkan::vulkanWindow()
     QVERIFY(w.graphicsCommandPool() != VK_NULL_HANDLE);
     QVERIFY(w.defaultRenderPass() != VK_NULL_HANDLE);
 
-    QVERIFY(w.concurrentFrameCount() > 0);
+    QCOMPARE_GT(w.concurrentFrameCount(), 0);
     QVERIFY(w.concurrentFrameCount() <= QVulkanWindow::MAX_CONCURRENT_FRAME_COUNT);
 }
 
@@ -455,20 +439,20 @@ void tst_QVulkan::vulkanWindowRenderer()
     if (w.availablePhysicalDevices().isEmpty())
         QSKIP("No Vulkan physical devices; skip");
 
-    QVERIFY(testVulkan.preInitResCount == 1);
-    QVERIFY(testVulkan.initResCount == 1);
-    QVERIFY(testVulkan.initSwcResCount == 1);
+    QCOMPARE(testVulkan.preInitResCount, 1);
+    QCOMPARE(testVulkan.initResCount, 1);
+    QCOMPARE(testVulkan.initSwcResCount, 1);
     // this has to be QTRY due to the async update in QVulkanWindowPrivate::ensureStarted()
     QTRY_VERIFY(testVulkan.startNextFrameCount >= 1);
 
     QVERIFY(!w.swapChainImageSize().isEmpty());
-    QVERIFY(w.colorFormat() != VK_FORMAT_UNDEFINED);
-    QVERIFY(w.depthStencilFormat() != VK_FORMAT_UNDEFINED);
+    QCOMPARE_NE(w.colorFormat(), VK_FORMAT_UNDEFINED);
+    QCOMPARE_NE(w.depthStencilFormat(), VK_FORMAT_UNDEFINED);
 
     w.destroy();
     waitForUnexposed(&w);
-    QVERIFY(testVulkan.releaseSwcResCount == 1);
-    QVERIFY(testVulkan.releaseResCount == 1);
+    QCOMPARE(testVulkan.releaseSwcResCount, 1);
+    QCOMPARE(testVulkan.releaseResCount, 1);
 }
 
 void tst_QVulkan::vulkanWindowGrab()
@@ -522,9 +506,9 @@ void tst_QVulkan::vulkanWindowGrab()
     int greenFuzz = qAbs(qGreen(a) - qGreen(refPixel));
     int blueFuzz = qAbs(qBlue(a) - qBlue(refPixel));
 
-    QVERIFY(redFuzz <= 1);
-    QVERIFY(blueFuzz <= 1);
-    QVERIFY(greenFuzz <= 1);
+    QCOMPARE_LE(redFuzz, 1);
+    QCOMPARE_LE(blueFuzz, 1);
+    QCOMPARE_LE(greenFuzz, 1);
 
     w.destroy();
 }
