@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 
 #include <QTest>
@@ -7,21 +7,12 @@
 
 #include "../../kernel/qsqldatabase/tst_databases.h"
 
-QString reltest1;
-QString reltest2;
-QString reltest3;
-QString reltest4;
-QString reltest5;
-
 class tst_QSqlRelationalTableModel : public QObject
 {
     Q_OBJECT
 
 public:
-    void recreateTestTables(QSqlDatabase);
-    tst_QSqlRelationalTableModel();
-
-    tst_Databases dbs;
+    using QObject::QObject;
 
 public slots:
     void initTestCase_data();
@@ -54,27 +45,46 @@ private slots:
     void selectAfterUpdate();
     void relationOnFirstColumn();
     void setRelation();
+    void setMultipleRelations();
 
 private:
-    void dropTestTables( QSqlDatabase db );
+    void fixupTableNamesForDb(const QSqlDatabase &db);
+    void recreateTestTables(const QSqlDatabase &db);
+    void dropTestTables(const QSqlDatabase &db);
+    static QString escapeTableName(const QSqlDatabase &db, const QString &name)
+    {
+        QString _name = name;
+        const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
+        if (dbType == QSqlDriver::Oracle ||
+            dbType == QSqlDriver::DB2)
+            _name = name.toUpper();
+        return db.driver()->escapeIdentifier(_name, QSqlDriver::TableName);
+    }
+    static QString escapeFieldName(const QSqlDatabase &db, const QString &name)
+    {
+        QString _name = name;
+        const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
+        if (dbType == QSqlDriver::Interbase ||
+            dbType == QSqlDriver::Oracle ||
+            dbType == QSqlDriver::DB2)
+            _name = name.toUpper();
+        return db.driver()->escapeIdentifier(_name, QSqlDriver::FieldName);
+    }
+    QString reltest1;
+    QString reltest2;
+    QString reltest3;
+    QString reltest4;
+    QString reltest5;
+    tst_Databases dbs;
 };
 
-tst_QSqlRelationalTableModel::tst_QSqlRelationalTableModel()
+void tst_QSqlRelationalTableModel::fixupTableNamesForDb(const QSqlDatabase &db)
 {
-    static QSqlDatabase static_qtest_db_1 = QSqlDatabase();
-    reltest1 = qTableName("reltest1", __FILE__, static_qtest_db_1);
-
-    static QSqlDatabase static_qtest_db_2 = QSqlDatabase();
-    reltest2 = qTableName("reltest2", __FILE__, static_qtest_db_2);
-
-    static QSqlDatabase static_qtest_db_3 = QSqlDatabase();
-    reltest3 = qTableName("reltest3", __FILE__, static_qtest_db_3);
-
-    static QSqlDatabase static_qtest_db_4 = QSqlDatabase();
-    reltest4 = qTableName("reltest4", __FILE__, static_qtest_db_4);
-
-    static QSqlDatabase static_qtest_db_5 = QSqlDatabase();
-    reltest5 = qTableName("reltest5", __FILE__, static_qtest_db_5);
+    reltest1 = qTableName("reltest1", __FILE__, db);
+    reltest2 = qTableName("reltest2", __FILE__, db);
+    reltest3 = qTableName("reltest3", __FILE__, db);
+    reltest4 = qTableName("reltest4", __FILE__, db);
+    reltest5 = qTableName("reltest5", __FILE__, db);
 }
 
 void tst_QSqlRelationalTableModel::initTestCase_data()
@@ -84,7 +94,7 @@ void tst_QSqlRelationalTableModel::initTestCase_data()
         QSKIP("No database drivers are available in this Qt configuration");
 }
 
-void tst_QSqlRelationalTableModel::recreateTestTables(QSqlDatabase db)
+void tst_QSqlRelationalTableModel::recreateTestTables(const QSqlDatabase &db)
 {
     dropTestTables(db);
 
@@ -114,32 +124,36 @@ void tst_QSqlRelationalTableModel::recreateTestTables(QSqlDatabase db)
     QVERIFY_SQL( q, exec("insert into " + reltest5 + " values('herr', 'Hr')"));
     QVERIFY_SQL( q, exec("insert into " + reltest5 + " values('mister', 'Mr')"));
 
-    if (testWhiteSpaceNames(db.driverName())) {
-        const auto reltest6 = qTableName("rel test6", __FILE__, db);
-        QVERIFY_SQL( q, exec("create table " + reltest6 + " (id int not null primary key, " + db.driver()->escapeIdentifier("city key", QSqlDriver::FieldName) +
-                    " int, " + db.driver()->escapeIdentifier("extra field", QSqlDriver::FieldName) + " int)"));
-        QVERIFY_SQL( q, exec("insert into " + reltest6 + " values(1, 1,9)"));
-        QVERIFY_SQL( q, exec("insert into " + reltest6 + " values(2, 2,8)"));
+    const auto reltest6 = qTableName("rel test6", __FILE__, db);
+    const auto cityKeyStr = db.driver()->escapeIdentifier("city key", QSqlDriver::FieldName);
+    const auto extraFieldStr = db.driver()->escapeIdentifier("extra field", QSqlDriver::FieldName);
+    QVERIFY_SQL( q, exec("create table " + reltest6 + " (id int not null primary key, " + cityKeyStr +
+                " int, " + extraFieldStr + " int)"));
+    QVERIFY_SQL( q, exec("insert into " + reltest6 + " values(1, 1,9)"));
+    QVERIFY_SQL( q, exec("insert into " + reltest6 + " values(2, 2,8)"));
 
-        const auto reltest7 = qTableName("rel test7", __FILE__, db);
-        QVERIFY_SQL( q, exec("create table " + reltest7 + " (" + db.driver()->escapeIdentifier("city id", QSqlDriver::TableName) + " int not null primary key, " + db.driver()->escapeIdentifier("city name", QSqlDriver::FieldName) + " varchar(20))"));
-        QVERIFY_SQL( q, exec("insert into " + reltest7 + " values(1, 'New York')"));
-        QVERIFY_SQL( q, exec("insert into " + reltest7 + " values(2, 'Washington')"));
-    }
+    const auto reltest7 = qTableName("rel test7", __FILE__, db);
+    const auto cityIdStr = db.driver()->escapeIdentifier("city id", QSqlDriver::TableName);
+    const auto cityNameStr = db.driver()->escapeIdentifier("city name", QSqlDriver::FieldName);
+    QVERIFY_SQL( q, exec("create table " + reltest7 + " (" + cityIdStr + " int not null primary key, " +
+                         cityNameStr + " varchar(20))"));
+    QVERIFY_SQL( q, exec("insert into " + reltest7 + " values(1, 'New York')"));
+    QVERIFY_SQL( q, exec("insert into " + reltest7 + " values(2, 'Washington')"));
 }
 
 void tst_QSqlRelationalTableModel::initTestCase()
 {
-    foreach (const QString &dbname, dbs.dbNames) {
-        QSqlDatabase db=QSqlDatabase::database(dbname);
+    for (const QString &dbName : std::as_const(dbs.dbNames)) {
+        QSqlDatabase db = QSqlDatabase::database(dbName);
+        QSqlQuery q(db);
         QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
         if (dbType == QSqlDriver::Interbase) {
-            db.exec("SET DIALECT 3");
+            q.exec("SET DIALECT 3");
         } else if (dbType == QSqlDriver::MSSqlServer) {
-            db.exec("SET ANSI_DEFAULTS ON");
-            db.exec("SET IMPLICIT_TRANSACTIONS OFF");
+            q.exec("SET ANSI_DEFAULTS ON");
+            q.exec("SET IMPLICIT_TRANSACTIONS OFF");
         } else if (dbType == QSqlDriver::PostgreSQL) {
-            db.exec("set client_min_messages='warning'");
+            q.exec("set client_min_messages='warning'");
         }
         recreateTestTables(db);
     }
@@ -147,30 +161,27 @@ void tst_QSqlRelationalTableModel::initTestCase()
 
 void tst_QSqlRelationalTableModel::cleanupTestCase()
 {
-    foreach (const QString &dbName, dbs.dbNames) {
+    for (const QString &dbName : std::as_const(dbs.dbNames)) {
         QSqlDatabase db = QSqlDatabase::database(dbName);
-        CHECK_DATABASE( db );
-        dropTestTables( QSqlDatabase::database(dbName) );
+        CHECK_DATABASE(db);
+        dropTestTables(db);
     }
     dbs.close();
 }
 
-void tst_QSqlRelationalTableModel::dropTestTables( QSqlDatabase db )
+void tst_QSqlRelationalTableModel::dropTestTables(const QSqlDatabase &db)
 {
-    QStringList tableNames;
-    tableNames << reltest1
-            << reltest2
-            << reltest3
-            << reltest4
-            << reltest5
-            << qTableName("rel test6", __FILE__, db)
-            << qTableName("rel test7", __FILE__, db)
-            << qTableName("CASETEST1", db)
-            << qTableName("casetest1", db);
+    fixupTableNamesForDb(db);
+    QStringList tableNames{reltest1, reltest2, reltest3, reltest4, reltest5,
+                           qTableName("rel test6", __FILE__, db),
+                           qTableName("rel test7", __FILE__, db),
+                           qTableName("CASETEST1", __FILE__, db),
+                           qTableName("casetest1", __FILE__, db)};
     tst_Databases::safeDropTables( db, tableNames );
 
-    db.exec("DROP SCHEMA " + qTableName("QTBUG_5373", __FILE__, db) + " CASCADE");
-    db.exec("DROP SCHEMA " + qTableName("QTBUG_5373_s2", __FILE__, db) + " CASCADE");
+    QSqlQuery q(db);
+    q.exec("DROP SCHEMA " + qTableName("QTBUG_5373", __FILE__, db) + " CASCADE");
+    q.exec("DROP SCHEMA " + qTableName("QTBUG_5373_s2", __FILE__, db) + " CASCADE");
 }
 
 void tst_QSqlRelationalTableModel::init()
@@ -186,6 +197,7 @@ void tst_QSqlRelationalTableModel::data()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QSqlRelationalTableModel model(0, db);
 
@@ -227,6 +239,7 @@ void tst_QSqlRelationalTableModel::setData()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
     // set the values using OnRowChange Strategy
@@ -434,6 +447,7 @@ void tst_QSqlRelationalTableModel::insertRecord()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QSqlRelationalTableModel model(0, db);
 
@@ -442,6 +456,29 @@ void tst_QSqlRelationalTableModel::insertRecord()
     model.setSort(0, Qt::AscendingOrder);
     QVERIFY_SQL(model, select());
 
+    constexpr auto fkTitleKey = 4711;
+    constexpr auto fkTitleVal = "new title";
+    {
+        auto relModel = model.relationModel(2);
+        // make sure populateDictionary() is called
+        relModel->select();
+
+        QSqlRecord rec;
+        QSqlField f1("id", QMetaType(QMetaType::Int));
+        QSqlField f2("title", QMetaType(QMetaType::QString));
+
+        f1.setValue(fkTitleKey);
+        f2.setValue(fkTitleVal);
+
+        f1.setGenerated(true);
+        f2.setGenerated(true);
+
+        rec.append(f1);
+        rec.append(f2);
+
+        QVERIFY(relModel->insertRecord(-1, rec));
+    }
+
     QSqlRecord rec;
     QSqlField f1("id", QMetaType(QMetaType::Int));
     QSqlField f2("name", QMetaType(QMetaType::QString));
@@ -450,7 +487,7 @@ void tst_QSqlRelationalTableModel::insertRecord()
 
     f1.setValue(7);
     f2.setValue("test");
-    f3.setValue(1);
+    f3.setValue(fkTitleKey);
     f4.setValue(2);
 
     f1.setGenerated(true);
@@ -467,7 +504,7 @@ void tst_QSqlRelationalTableModel::insertRecord()
 
     QCOMPARE(model.data(model.index(4, 0)).toInt(), 7);
     QCOMPARE(model.data(model.index(4, 1)).toString(), QString("test"));
-    QCOMPARE(model.data(model.index(4, 2)).toString(), QString("herr"));
+    QCOMPARE(model.data(model.index(4, 2)).toString(), QString(fkTitleVal));
 
     // In LeftJoin mode, two additional rows are fetched
     model.setJoinMode(QSqlRelationalTableModel::LeftJoin);
@@ -475,7 +512,7 @@ void tst_QSqlRelationalTableModel::insertRecord()
 
     QCOMPARE(model.data(model.index(6, 0)).toInt(), 7);
     QCOMPARE(model.data(model.index(6, 1)).toString(), QString("test"));
-    QCOMPARE(model.data(model.index(6, 2)).toString(), QString("herr"));
+    QCOMPARE(model.data(model.index(6, 2)).toString(), QString(fkTitleVal));
 }
 
 void tst_QSqlRelationalTableModel::setRecord()
@@ -544,6 +581,7 @@ void tst_QSqlRelationalTableModel::insertWithStrategies()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
     QSqlRelationalTableModel model(0, db);
@@ -652,56 +690,33 @@ void tst_QSqlRelationalTableModel::removeColumn()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
-    recreateTestTables(db);
 
-    QSqlRelationalTableModel model(0, db);
+    for (const auto mode : {QSqlRelationalTableModel::InnerJoin, QSqlRelationalTableModel::LeftJoin}) {
+        recreateTestTables(db);
 
-    model.setTable(reltest1);
-    model.setRelation(2, QSqlRelation(reltest2, "id", "title"));
-    QVERIFY_SQL(model, select());
+        QSqlRelationalTableModel model(0, db);
 
-    QVERIFY_SQL(model, removeColumn(3));
-    QVERIFY_SQL(model, select());
+        model.setTable(reltest1);
+        model.setRelation(2, QSqlRelation(reltest2, "id", "title"));
+        model.setJoinMode(mode);
+        QVERIFY_SQL(model, select());
 
-    QCOMPARE(model.columnCount(), 3);
+        QVERIFY_SQL(model, removeColumn(3));
+        QVERIFY_SQL(model, select());
 
-    QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
-    QCOMPARE(model.data(model.index(0, 1)).toString(), QString("harry"));
-    QCOMPARE(model.data(model.index(0, 2)).toString(), QString("herr"));
-    QCOMPARE(model.data(model.index(0, 3)), QVariant());
+        QCOMPARE(model.columnCount(), 3);
 
-    // try removing more than one column
-    QVERIFY_SQL(model, removeColumns(1, 2));
-    QCOMPARE(model.columnCount(), 1);
-    QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
-    QCOMPARE(model.data(model.index(0, 1)), QVariant());
+        QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
+        QCOMPARE(model.data(model.index(0, 1)).toString(), QString("harry"));
+        QCOMPARE(model.data(model.index(0, 2)).toString(), QString("herr"));
+        QCOMPARE(model.data(model.index(0, 3)), QVariant());
 
-    // try in LeftJoin mode the same tests
-    CHECK_DATABASE(db);
-    recreateTestTables(db);
-
-    QSqlRelationalTableModel lmodel(0, db);
-
-    lmodel.setTable(reltest1);
-    lmodel.setRelation(2, QSqlRelation(reltest2, "id", "title"));
-    lmodel.setJoinMode(QSqlRelationalTableModel::LeftJoin);
-    QVERIFY_SQL(lmodel, select());
-
-    QVERIFY_SQL(lmodel, removeColumn(3));
-    QVERIFY_SQL(lmodel, select());
-
-    QCOMPARE(lmodel.columnCount(), 3);
-
-    QCOMPARE(lmodel.data(lmodel.index(0, 0)).toInt(), 1);
-    QCOMPARE(lmodel.data(lmodel.index(0, 1)).toString(), QString("harry"));
-    QCOMPARE(lmodel.data(lmodel.index(0, 2)).toString(), QString("herr"));
-    QCOMPARE(lmodel.data(lmodel.index(0, 3)), QVariant());
-
-    // try removing more than one column
-    QVERIFY_SQL(lmodel, removeColumns(1, 2));
-    QCOMPARE(lmodel.columnCount(), 1);
-    QCOMPARE(lmodel.data(lmodel.index(0, 0)).toInt(), 1);
-    QCOMPARE(lmodel.data(lmodel.index(0, 1)), QVariant());
+        // try removing more than one column
+        QVERIFY_SQL(model, removeColumns(1, 2));
+        QCOMPARE(model.columnCount(), 1);
+        QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
+        QCOMPARE(model.data(model.index(0, 1)), QVariant());
+    }
 }
 
 void tst_QSqlRelationalTableModel::filter()
@@ -736,6 +751,7 @@ void tst_QSqlRelationalTableModel::sort()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
     QSqlRelationalTableModel model(0, db);
@@ -781,7 +797,7 @@ void tst_QSqlRelationalTableModel::sort()
     QStringList stringsInDatabaseOrder;
     // PostgreSQL puts the null ones (from the table with the original value) first in descending order
     // which translate to empty strings in the related table
-    if (dbType == QSqlDriver::PostgreSQL)
+    if (dbType == QSqlDriver::PostgreSQL || dbType == QSqlDriver::MimerSQL)
         stringsInDatabaseOrder << "" << "" << "mister" << "mister" << "herr" << "herr";
     else
         stringsInDatabaseOrder << "mister" << "mister" << "herr" << "herr" << "" << "";
@@ -794,7 +810,7 @@ void tst_QSqlRelationalTableModel::sort()
     // PostgreSQL puts the null ones (from the table with the original value) first in descending order
     // which translate to empty strings in the related table
     stringsInDatabaseOrder.clear();
-    if (dbType == QSqlDriver::PostgreSQL)
+    if (dbType == QSqlDriver::PostgreSQL || dbType == QSqlDriver::MimerSQL)
         stringsInDatabaseOrder << "herr" << "mister" << "mister" << "mister" << "mister" << "";
     else if (dbType != QSqlDriver::Sybase)
         stringsInDatabaseOrder << "" << "herr" << "mister" << "mister" << "mister" << "mister";
@@ -873,6 +889,7 @@ void tst_QSqlRelationalTableModel::revert()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QSqlRelationalTableModel model(0, db);
 
@@ -910,6 +927,7 @@ void tst_QSqlRelationalTableModel::clearDisplayValuesCache()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
     QSqlRelationalTableModel model(0, db);
@@ -964,6 +982,7 @@ void tst_QSqlRelationalTableModel::insertRecordDuplicateFieldNames()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
     QSqlRelationalTableModel model(0, db);
@@ -975,13 +994,12 @@ void tst_QSqlRelationalTableModel::insertRecordDuplicateFieldNames()
     model.setRelation(2, QSqlRelation(reltest4, "id", "name"));
     QVERIFY_SQL(model, select());
 
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-        QCOMPARE(model.record(1).value((reltest4+QLatin1String("_name_2")).toUpper()).toString(),
-            QString("Trondheim"));
-    } else {
-        QCOMPARE(model.record(1).value((reltest4+QLatin1String("_name_2"))).toString(),
-            QString("Trondheim"));
-    }
+    QString reltest4Unescaped = qTableName("reltest4", __FILE__, db, false);
+    QString fieldName = reltest4Unescaped + QLatin1String("_name_2");
+    if (dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2)
+        fieldName = fieldName.toUpper();
+    fieldName.truncate(db.driver()->maximumIdentifierLength(QSqlDriver::TableName));
+    QCOMPARE(model.record(1).value(fieldName).toString(), QLatin1String("Trondheim"));
 
     QSqlRecord rec = model.record();
     rec.setValue(0, 3);
@@ -997,10 +1015,7 @@ void tst_QSqlRelationalTableModel::insertRecordDuplicateFieldNames()
     }
 
     // The duplicate field names is aliased because it's comes from the relation's display column.
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2)
-        QCOMPARE(rec.fieldName(2), (reltest4+QLatin1String("_name_2")).toUpper());
-    else
-        QCOMPARE(rec.fieldName(2), reltest4+QLatin1String("_name_2"));
+    QCOMPARE(rec.fieldName(2), fieldName);
 
     QVERIFY(model.insertRecord(-1, rec));
     QCOMPARE(model.data(model.index(2, 2)).toString(), QString("Oslo"));
@@ -1013,6 +1028,7 @@ void tst_QSqlRelationalTableModel::invalidData()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QSqlRelationalTableModel model(0, db);
     model.setTable(reltest1);
@@ -1043,6 +1059,7 @@ void tst_QSqlRelationalTableModel::relationModel()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QSqlRelationalTableModel model(0, db);
     model.setTable(reltest1);
@@ -1086,60 +1103,64 @@ void tst_QSqlRelationalTableModel::casing()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::SQLite || dbType == QSqlDriver::MSSqlServer)
+    if (dbType == QSqlDriver::SQLite || dbType == QSqlDriver::MSSqlServer)
         QSKIP("The casing test for this database is irrelevant since this database does not treat different cases as separate entities");
 
     QSqlQuery q(db);
-    QVERIFY_SQL( q, exec("create table " + qTableName("CASETEST1", db).toUpper() +
+    const QString caseTestUpper = qTableName("CASETEST1", __FILE__, db).toUpper();
+    const QString caseTestLower = qTableName("casetest1", __FILE__, db);
+    tst_Databases::safeDropTables(db, {caseTestUpper, caseTestLower});
+    QVERIFY_SQL( q, exec("create table " + caseTestUpper +
                 " (id int not null primary key, name varchar(20), title_key int, another_title_key int)"));
 
-    if (!q.exec("create table " + qTableName("casetest1", db) +
+    if (!q.exec("create table " + caseTestLower +
                 " (ident int not null primary key, name varchar(20), title_key int)"))
         QSKIP("The casing test for this database is irrelevant since this database does not treat different cases as separate entities");
 
-    QVERIFY_SQL( q, exec("insert into " + qTableName("CASETEST1", db).toUpper() + " values(1, 'harry', 1, 2)"));
-    QVERIFY_SQL( q, exec("insert into " + qTableName("CASETEST1", db).toUpper() + " values(2, 'trond', 2, 1)"));
-    QVERIFY_SQL( q, exec("insert into " + qTableName("CASETEST1", db).toUpper() + " values(3, 'vohi', 1, 2)"));
-    QVERIFY_SQL( q, exec("insert into " + qTableName("CASETEST1", db).toUpper() + " values(4, 'boris', 2, 2)"));
-    QVERIFY_SQL( q, exec("insert into " + qTableName("casetest1", db) + " values(1, 'jerry', 1)"));
-    QVERIFY_SQL( q, exec("insert into " + qTableName("casetest1", db) + " values(2, 'george', 2)"));
-    QVERIFY_SQL( q, exec("insert into " + qTableName("casetest1", db) + " values(4, 'kramer', 2)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestUpper + " values(1, 'harry', 1, 2)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestUpper + " values(2, 'trond', 2, 1)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestUpper + " values(3, 'vohi', 1, 2)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestUpper + " values(4, 'boris', 2, 2)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestLower + " values(1, 'jerry', 1)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestLower + " values(2, 'george', 2)"));
+    QVERIFY_SQL( q, exec("insert into " + caseTestLower + " values(4, 'kramer', 2)"));
 
     if (dbType == QSqlDriver::Oracle) {
         //try an owner that doesn't exist
-        QSqlRecord rec = db.driver()->record("doug." + qTableName("CASETEST1", db).toUpper());
+        QSqlRecord rec = db.driver()->record("doug." + caseTestUpper);
         QCOMPARE( rec.count(), 0);
 
         //try an owner that does exist
-        rec = db.driver()->record(db.userName() + QLatin1Char('.') + qTableName("CASETEST1", db).toUpper());
+        rec = db.driver()->record(db.userName() + QLatin1Char('.') + caseTestUpper);
         QCOMPARE( rec.count(), 4);
     }
-    QSqlRecord rec = db.driver()->record(qTableName("CASETEST1", db).toUpper());
+    QSqlRecord rec = db.driver()->record(caseTestUpper);
     QCOMPARE( rec.count(), 4);
 
-    rec = db.driver()->record(qTableName("casetest1", db));
+    rec = db.driver()->record(caseTestLower);
     QCOMPARE( rec.count(), 3);
 
     QSqlTableModel upperCaseModel(0, db);
-    upperCaseModel.setTable(qTableName("CASETEST1", db).toUpper());
+    upperCaseModel.setTable(caseTestUpper);
 
-    QCOMPARE(upperCaseModel.tableName(), qTableName("CASETEST1", db).toUpper());
+    QCOMPARE(upperCaseModel.tableName(), caseTestUpper);
 
     QVERIFY_SQL(upperCaseModel, select());
 
     QCOMPARE(upperCaseModel.rowCount(), 4);
 
     QSqlTableModel lowerCaseModel(0, db);
-    lowerCaseModel.setTable(qTableName("casetest1", db));
-    QCOMPARE(lowerCaseModel.tableName(), qTableName("casetest1", db));
+    lowerCaseModel.setTable(caseTestLower);
+    QCOMPARE(lowerCaseModel.tableName(), caseTestLower);
     QVERIFY_SQL(lowerCaseModel, select());
 
     QCOMPARE(lowerCaseModel.rowCount(), 3);
 
     QSqlRelationalTableModel model(0, db);
-    model.setTable(qTableName("CASETEST1", db).toUpper());
+    model.setTable(caseTestUpper);
     model.setRelation(2, QSqlRelation(reltest2, "id", "title"));
     QVERIFY_SQL(model, select());
 
@@ -1153,7 +1174,7 @@ void tst_QSqlRelationalTableModel::escapedRelations()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
-    const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
+    fixupTableNamesForDb(db);
 
     recreateTestTables(db);
 
@@ -1161,16 +1182,7 @@ void tst_QSqlRelationalTableModel::escapedRelations()
     model.setTable(reltest1);
 
     //try with relation table name quoted
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-        model.setRelation(2, QSqlRelation(db.driver()->escapeIdentifier(reltest2.toUpper(),QSqlDriver::TableName),
-                            "id",
-                            "title"));
-    } else {
-        model.setRelation(2, QSqlRelation(db.driver()->escapeIdentifier(reltest2,QSqlDriver::TableName),
-                            "id",
-                            "title"));
-
-    }
+    model.setRelation(2, QSqlRelation(escapeTableName(db, reltest2), "id", "title"));
     QVERIFY_SQL(model, select());
 
     QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
@@ -1185,16 +1197,8 @@ void tst_QSqlRelationalTableModel::escapedRelations()
     QCOMPARE(model.data(model.index(0, 2)).toString(), QString("herr"));
 
     //try with index column quoted
+    model.setRelation(2, QSqlRelation(reltest2, escapeFieldName(db, "id"), "title"));
     model.setJoinMode(QSqlRelationalTableModel::InnerJoin);
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-        model.setRelation(2, QSqlRelation(reltest2,
-                            db.driver()->escapeIdentifier("id", QSqlDriver::FieldName).toUpper(),
-                            "title"));
-    } else {
-        model.setRelation(2, QSqlRelation(reltest2,
-                            db.driver()->escapeIdentifier("id", QSqlDriver::FieldName),
-                            "title"));
-    }
     QVERIFY_SQL(model, select());
 
     QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
@@ -1209,18 +1213,8 @@ void tst_QSqlRelationalTableModel::escapedRelations()
     QCOMPARE(model.data(model.index(0, 2)).toString(), QString("herr"));
 
     //try with display column quoted
+    model.setRelation(2, QSqlRelation(reltest2, "id", escapeFieldName(db, "title")));
     model.setJoinMode(QSqlRelationalTableModel::InnerJoin);
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-
-        model.setRelation(2, QSqlRelation(reltest2,
-                            "id",
-                            db.driver()->escapeIdentifier("title", QSqlDriver::FieldName).toUpper()));
-    } else {
-        model.setRelation(2, QSqlRelation(reltest2,
-                            "id",
-                            db.driver()->escapeIdentifier("title", QSqlDriver::FieldName)));
-    }
-
     QVERIFY_SQL(model, select());
 
     QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
@@ -1235,16 +1229,10 @@ void tst_QSqlRelationalTableModel::escapedRelations()
     QCOMPARE(model.data(model.index(0, 2)).toString(), QString("herr"));
 
     //try with tablename and index and display columns quoted in the relation
+    model.setRelation(2, QSqlRelation(escapeTableName(db, reltest2),
+                                      escapeFieldName(db, "id"),
+                                      escapeFieldName(db, "title")));
     model.setJoinMode(QSqlRelationalTableModel::InnerJoin);
-    if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-        model.setRelation(2, QSqlRelation(reltest2,
-                            "id",
-                            db.driver()->escapeIdentifier("title", QSqlDriver::FieldName).toUpper()));
-    } else {
-        model.setRelation(2, QSqlRelation(reltest2,
-                            "id",
-                            db.driver()->escapeIdentifier("title", QSqlDriver::FieldName)));
-    }
     QVERIFY_SQL(model, select());
 
     QCOMPARE(model.data(model.index(0, 0)).toInt(), 1);
@@ -1264,17 +1252,13 @@ void tst_QSqlRelationalTableModel::escapedTableName()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
-    const QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
+    fixupTableNamesForDb(db);
 
     // set the values using OnRowChange Strategy with an escaped tablename
     {
         QSqlRelationalTableModel model(0, db);
 
-        if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-            model.setTable(db.driver()->escapeIdentifier(reltest1.toUpper(), QSqlDriver::TableName));
-        } else {
-            model.setTable(db.driver()->escapeIdentifier(reltest1, QSqlDriver::TableName));
-        }
+        model.setTable(escapeTableName(db, reltest1));
         model.setSort(0, Qt::AscendingOrder);
         model.setRelation(2, QSqlRelation(reltest2, "id", "title"));
         QVERIFY_SQL(model, select());
@@ -1317,11 +1301,7 @@ void tst_QSqlRelationalTableModel::escapedTableName()
     {
         QSqlRelationalTableModel model(0, db);
 
-        if (dbType == QSqlDriver::Interbase || dbType == QSqlDriver::Oracle || dbType == QSqlDriver::DB2) {
-            model.setTable(db.driver()->escapeIdentifier(reltest1.toUpper(), QSqlDriver::TableName));
-        } else {
-            model.setTable(db.driver()->escapeIdentifier(reltest1, QSqlDriver::TableName));
-        }
+        model.setTable(escapeTableName(db, reltest1));
         model.setSort(0, Qt::AscendingOrder);
         model.setRelation(2, QSqlRelation(reltest2, "id", "title"));
         model.setJoinMode(QSqlRelationalTableModel::LeftJoin);
@@ -1368,9 +1348,8 @@ void tst_QSqlRelationalTableModel::whiteSpaceInIdentifiers()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
-    if (!testWhiteSpaceNames(db.driverName()))
-        QSKIP("White space test irrelevant for driver");
     QSqlRelationalTableModel model(0, db);
     model.setTable(qTableName("rel test6", __FILE__, db));
     model.setSort(0, Qt::DescendingOrder);
@@ -1453,6 +1432,7 @@ void tst_QSqlRelationalTableModel::psqlSchemaTest()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
     QSqlDriver::DbmsType dbType = tst_Databases::getDatabaseType(db);
 
     if (dbType != QSqlDriver::PostgreSQL)
@@ -1480,6 +1460,7 @@ void tst_QSqlRelationalTableModel::selectAfterUpdate()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QSqlRelationalTableModel model(0, db);
     model.setTable(reltest1);
@@ -1505,6 +1486,7 @@ void tst_QSqlRelationalTableModel::relationOnFirstColumn()
     QFETCH_GLOBAL(QString, dbName);
     QSqlDatabase db = QSqlDatabase::database(dbName);
     CHECK_DATABASE(db);
+    fixupTableNamesForDb(db);
 
     QString testTable1 = qTableName("QTBUG_20038_test1", __FILE__, db);
     QString testTable2 = qTableName("QTBUG_20038_test2", __FILE__, db);
@@ -1519,7 +1501,14 @@ void tst_QSqlRelationalTableModel::relationOnFirstColumn()
     QVERIFY_SQL(q, exec("INSERT INTO " + testTable1 + " (id1, val1) VALUES(3, 30);"));
 
     //prepare test2 table
-    QVERIFY_SQL(q, exec("CREATE TABLE " + testTable2 + " (id INTEGER PRIMARY KEY, name TEXT);"));
+    if (tst_Databases::getDatabaseType(db) == QSqlDriver::MimerSQL) {
+        QVERIFY_SQL(q,
+                    exec("CREATE TABLE " + testTable2
+                         + " (id INTEGER PRIMARY KEY, name NVARCHAR(100));"));
+    } else {
+        QVERIFY_SQL(q,
+                    exec("CREATE TABLE " + testTable2 + " (id INTEGER PRIMARY KEY, name VARCHAR(100));"));
+    }
     QVERIFY_SQL(q, exec("DELETE FROM " + testTable2 + QLatin1Char(';')));
     QVERIFY_SQL(q, exec("INSERT INTO " + testTable2 + " (id, name) VALUES (10, 'Hervanta');"));
     QVERIFY_SQL(q, exec("INSERT INTO " + testTable2 + " (id, name) VALUES (20, 'Keskusta');"));
@@ -1575,5 +1564,24 @@ void tst_QSqlRelationalTableModel::setRelation()
     QCOMPARE(model.data(model.index(0, 2)), QVariant(1));
 }
 
+void tst_QSqlRelationalTableModel::setMultipleRelations()
+{
+    QFETCH_GLOBAL(QString, dbName);
+    QSqlDatabase db = QSqlDatabase::database(dbName);
+    CHECK_DATABASE(db);
+    recreateTestTables(db);
+
+    QSqlRelationalTableModel model(0, db);
+    model.setTable(reltest1);
+    QVERIFY_SQL(model, select());
+    model.setRelation(2, QSqlRelation(reltest2, "id", "title"));
+    model.data(model.index(0, 2));  // initialize model for QSqlRelation above
+    // id must be big enough that the internal QList needs to be reallocated
+    model.setRelation(100, QSqlRelation(reltest2, "id", "title"));
+    QSqlTableModel *relationModel = model.relationModel(2);
+    QVERIFY(relationModel);
+    QVERIFY(relationModel->select());
+}
+
 QTEST_MAIN(tst_QSqlRelationalTableModel)
 #include "tst_qsqlrelationaltablemodel.moc"
