@@ -1,7 +1,9 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtNetwork>
+#include <QtNetwork/qsslsocket.h>
+
+#include <QtCore/qcoreapplication.h>
 
 int main(int argc, char ** argv)
 {
