@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QTest>
 #include <QtTest/private/qtesthelpers_p.h>
@@ -27,6 +27,9 @@
 #include <qpa/qplatformtheme.h>
 #include <qpa/qplatformintegration.h>
 
+#include <QtWidgets/private/qapplication_p.h>
+#include <QtWidgets/private/qmenu_p.h>
+
 using namespace QTestPrivate;
 
 Q_DECLARE_METATYPE(Qt::Key);
@@ -110,6 +113,12 @@ private slots:
     void tearOffMenuNotDisplayed();
     void QTBUG_61039_menu_shortcuts();
     void screenOrientationChangedCloseMenu();
+    void deleteWhenTriggered();
+
+    void nestedTearOffDetached();
+    void closeMenuOnClickIfMouseHasntMoved();
+
+    void invisibleActions();
 
 protected slots:
     void onActivated(QAction*);
@@ -481,7 +490,7 @@ void tst_QMenu::focus()
     QPushButton button("Push me", &window);
     centerOnScreen(&window);
     window.show();
-    qApp->setActiveWindow(&window);
+    QApplicationPrivate::setActiveWindow(&window);
 
     QVERIFY(button.hasFocus());
     QCOMPARE(QApplication::focusWidget(), (QWidget *)&button);
@@ -525,7 +534,6 @@ void tst_QMenu::overrideMenuAction()
     m->addAction(aQuit);
 
     w.show();
-    QApplication::setActiveWindow(&w);
     w.setFocus();
     QVERIFY(QTest::qWaitForWindowActive(&w));
     QVERIFY(w.hasFocus());
@@ -1140,14 +1148,18 @@ void tst_QMenu::pushButtonPopulateOnAboutToShow()
         QSKIP("Your window manager won't allow a window against the bottom of the screen");
     }
 
-    QTimer::singleShot(300, buttonMenu, SLOT(hide()));
     QTest::mouseClick(&b, Qt::LeftButton, Qt::NoModifier, b.rect().center());
+    QVERIFY(QTest::qWaitForWindowExposed(buttonMenu));
+    QTest::qWait(300);
+    buttonMenu->hide();
     QVERIFY2(!buttonMenu->geometry().intersects(b.geometry()), msgGeometryIntersects(buttonMenu->geometry(), b.geometry()));
 
     // note: we're assuming that, if we previously got the desired geometry, we'll get it here too
     b.move(10, screen.bottom()-buttonMenu->height()-5);
-    QTimer::singleShot(300, buttonMenu, SLOT(hide()));
     QTest::mouseClick(&b, Qt::LeftButton, Qt::NoModifier, b.rect().center());
+    QVERIFY(QTest::qWaitForWindowExposed(buttonMenu));
+    QTest::qWait(300);
+    buttonMenu->hide();
     QVERIFY2(!buttonMenu->geometry().intersects(b.geometry()), msgGeometryIntersects(buttonMenu->geometry(), b.geometry()));
 }
 
@@ -1603,9 +1615,8 @@ void tst_QMenu::transientParent()
     QWindow *topLevel = window.windowHandle();
     QVERIFY(topLevel);
 
-    QApplication::setActiveWindow(&window);
     window.setFocus();
-    QVERIFY(QTest::qWaitForWindowActive(&window));
+    QVERIFY(QTest::qWaitForWindowFocused(&window));
     QVERIFY(window.hasFocus());
 
     QTest::keyPress(&window, Qt::Key_F, Qt::AltModifier);
@@ -2011,5 +2022,162 @@ void tst_QMenu::screenOrientationChangedCloseMenu()
      QTRY_COMPARE(menu.isVisible(),false);
 }
 
+/*
+    Verify that deleting the menu in a slot connected to an
+    action's triggered signal doesn't crash.
+    QTBUG-106718
+*/
+void tst_QMenu::deleteWhenTriggered()
+{
+    QPointer<QMenu> menu = new QMenu;
+    QAction *action = menu->addAction("Action", [&menu]{
+        delete menu;
+    });
+    menu->popup(QGuiApplication::primaryScreen()->availableGeometry().center());
+    menu->setActiveAction(action);
+    QTest::keyClick(menu, Qt::Key_Return);
+    QTRY_VERIFY(!menu);
+}
+
+/*
+    QMenu uses the caused-stack to create the parent/child relationship
+    for tear-off menus. Since QTornOffMenu set the DeleteOnClose flag, closing a
+    tear-off in the parent chain will result in a null-pointer in the caused-stack.
+    Verify that we don't crash when traversing the chain, as reported in QTBUG-112217.
+
+    The test has to open the submenus by hovering of the menu action, otherwise
+    the caused-stack remains empty and the issue doesn't reproduce. Due to QMenu's
+    timing and "sloppiness", we need to move the mouse within the action, with some
+    waiting and event processing in between to trigger the opening of the submenu.
+    If this fails we skip, as we then can't test what we are trying to test.
+*/
+void tst_QMenu::nestedTearOffDetached()
+{
+    // Since QTornOffMenu is not declared in qmenuprivate.h we can't access the
+    // object even through QMenuPrivate. So use an event filter to watch out for
+    // a QTornOffMenu showing.
+    class TearOffWatcher : public QObject
+    {
+    public:
+        QMenu *tornOffMenu = nullptr;
+    protected:
+        bool eventFilter(QObject *receiver, QEvent *event) override
+        {
+            if (event->type() == QEvent::Show && receiver->inherits("QTornOffMenu"))
+                tornOffMenu = qobject_cast<QMenu *>(receiver);
+            return QObject::eventFilter(receiver, event);
+        }
+    } watcher;
+    qApp->installEventFilter(&watcher);
+
+    QWidget widget;
+    QMenu *menu = new QMenu("Context", &widget);
+
+    MenuMetrics mm(menu);
+    const int tearOffOffset = mm.fw + mm.vmargin + mm.tearOffHeight / 2;
+
+    QMenu *subMenu = menu->addMenu("SubMenu");
+    menu->setTearOffEnabled(true);
+    QMenu *subSubMenu = subMenu->addMenu("SubSubMenu");
+    subMenu->setTearOffEnabled(true);
+    subSubMenu->addAction("Action!");
+    subSubMenu->setTearOffEnabled(true);
+
+    widget.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&widget));
+
+    // open and tear off context menu
+    menu->popup(widget.geometry().center());
+    QTest::mouseClick(menu, Qt::LeftButton, {}, QPoint(menu->width() / 2, tearOffOffset));
+
+    QMenu *menuTorn = watcher.tornOffMenu;
+    watcher.tornOffMenu = nullptr;
+    QVERIFY(menuTorn);
+    QVERIFY(QTest::qWaitForWindowExposed(menuTorn));
+
+    // open second menu and tear-off
+    QTest::mouseMove(menuTorn, menuTorn->actionGeometry(subMenu->menuAction()).topLeft());
+    QTest::qWait(100);
+    QTest::mouseMove(menuTorn, menuTorn->actionGeometry(subMenu->menuAction()).center());
+    if (!QTest::qWaitFor([subMenu]{ return subMenu->isVisible(); }))
+        QSKIP("Menu failed to show, skipping test");
+
+    QTest::mouseClick(subMenu, Qt::LeftButton, {}, QPoint(subMenu->width() / 2, tearOffOffset));
+    menuTorn = watcher.tornOffMenu;
+    QVERIFY(menuTorn);
+    QVERIFY(QTest::qWaitForWindowExposed(menuTorn));
+    // close the top level tear off
+    menu->hideTearOffMenu();
+    // open third menu and tear-off
+    QTest::mouseMove(menuTorn, menuTorn->actionGeometry(subSubMenu->menuAction()).topLeft());
+    QTest::qWait(100);
+    QTest::mouseMove(menuTorn, menuTorn->actionGeometry(subSubMenu->menuAction()).center());
+    QTRY_VERIFY(subSubMenu->isVisible());
+    QTest::mouseClick(subSubMenu, Qt::LeftButton, {}, QPoint(subSubMenu->width() / 2, tearOffOffset));
+}
+
+/*!
+    Test that a menu will close if you do a mouse click on top of
+    it without having moved the mouse.
+    (QTBUG-128359).
+*/
+void tst_QMenu::closeMenuOnClickIfMouseHasntMoved()
+{
+    QWidget w;
+    w.resize(100, 100);
+    w.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&w));
+
+    QMenu contextMenu;
+    for (int i = 0; i < 5; ++i) {
+        QAction *action = contextMenu.addAction(QStringLiteral("action"));
+        connect(action, &QAction::triggered, []{ QFAIL("No menu item should trigger"); });
+    }
+
+    const QPoint pos = w.rect().center();
+    const QPoint globalPos = w.mapToGlobal(pos);
+    // Move the mouse inside the window
+    QTest::mouseMove(&w, pos);
+    // Move the menu a bit up, so that a menu item falls underneath the
+    // mouse (similar to the code attached to the bug report: QTBUG-128359).
+    contextMenu.popup(globalPos - QPoint(0, 20));
+    QVERIFY(QTest::qWaitForWindowExposed(&contextMenu));
+    // Do a mouse click without having moved the cursor. This
+    // should close the menu, even if it's underneath the mouse.
+    QTest::mouseClick(&contextMenu, Qt::RightButton, {}, contextMenu.mapFromGlobal(pos));
+}
+
+void tst_QMenu::invisibleActions()
+{
+    QWidget window;
+    window.resize(100, 100);
+    window.show();
+
+    const QPoint globalPos = window.mapToGlobal(window.rect().center());
+
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    QMenu contextMenu;
+    QList<QAction *> actions;
+    for (int i = 0; i < 5; ++i)
+        actions << contextMenu.addAction("action");
+    QVERIFY(contextMenu.sizeHint().isValid());
+
+    contextMenu.popup(globalPos);
+    QVERIFY(contextMenu.isVisible());
+
+    contextMenu.close();
+
+    for (const auto &action : actions)
+        action->setVisible(false);
+
+    contextMenu.popup(globalPos);
+    QCOMPARE(contextMenu.isVisible(), contextMenu.sizeHint().isValid());
+
+    // if it wasn't shown previously, then exec() shouldn't do anything either
+    if (!contextMenu.isVisible())
+        QVERIFY(!contextMenu.exec());
+}
+
 QTEST_MAIN(tst_QMenu)
 #include "tst_qmenu.moc"
