@@ -1,5 +1,5 @@
 // Copyright (C) 2020 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QLocale>
 #include <QTest>
@@ -237,21 +237,16 @@ void tst_QLocale::fromScriptLand_data()
     ROW("Guru-IN", GurmukhiScript, India);
     ROW("Latn-MA", LatinScript, Morocco);
     ROW("Tfng-MA", TifinaghScript, Morocco);
-    ROW("Cyrl-BA", CyrillicScript, BosniaAndHerzegowina);
     ROW("Cyrl-RS", CyrillicScript, Serbia);
-    ROW("Latn-BA", LatinScript, BosniaAndHerzegowina);
     ROW("Latn-ME", LatinScript, Montenegro);
     ROW("Arab-AF", ArabicScript, Afghanistan);
     ROW("Cyrl-UZ", CyrillicScript, Uzbekistan);
     ROW("Latn-UZ", LatinScript, Uzbekistan);
-    ROW("Latn-LR", LatinScript, Liberia);
     ROW("Vaii-LR", VaiScript, Liberia);
     ROW("Hans-CN", SimplifiedHanScript, China);
     ROW("Hant-HK", TraditionalHanScript, HongKong);
-    ROW("Hans-CN", SimplifiedHanScript, China);
     ROW("Hans-HK", SimplifiedHanScript, HongKong);
     ROW("Hans-SG", SimplifiedHanScript, Singapore);
-    ROW("Hant-HK", TraditionalHanScript, HongKong);
     ROW("Hant-TW", TraditionalHanScript, Taiwan);
 #undef ROW
 }
@@ -455,10 +450,6 @@ void tst_QLocale::toLongLong()
     QBENCHMARK {
         actual = loc.toLongLong(text, &ok);
     }
-    QEXPECT_FAIL("ar_EG: +403", "Code wrongly assumes single character, QTBUG-107801", Abort);
-    QEXPECT_FAIL("ar_EG: -403", "Code wrongly assumes single character, QTBUG-107801", Abort);
-    QEXPECT_FAIL("fa_IR: +403", "Code wrongly assumes single character, QTBUG-107801", Abort);
-    QEXPECT_FAIL("fa_IR: -403", "Code wrongly assumes single character, QTBUG-107801", Abort);
     QCOMPARE(ok, good);
     QCOMPARE(actual, expected);
 }
@@ -488,8 +479,6 @@ void tst_QLocale::toULongLong()
     QBENCHMARK {
         actual = loc.toULongLong(text, &ok);
     }
-    QEXPECT_FAIL("ar_EG: +403", "Code wrongly assumes single character, QTBUG-107801", Abort);
-    QEXPECT_FAIL("fa_IR: +403", "Code wrongly assumes single character, QTBUG-107801", Abort);
     QCOMPARE(ok, good);
     QCOMPARE(actual, expected);
 }
@@ -566,13 +555,13 @@ void tst_QLocale::toDouble_data()
     QTest::newRow("se_NO: 4x-3") // Only first character of exponent
             << u"4\u00b7\u2212" "03"_s << u"se_NO"_s << false << 0.0;
     QTest::newRow("ar_EG: 4e-3") // Arabic, Egypt
-            << u"\u0664\u0627\u0633\u061c-\u0660\u0663"_s << u"ar_EG"_s << true << 4e-3;
+            << u"\u0664\u0623\u0633\u061c-\u0660\u0663"_s << u"ar_EG"_s << true << 4e-3;
     QTest::newRow("ar_EG: 4x-3") // Only first character of exponent
-            << u"\u0664\u0627\u061c-\u0660\u0663"_s << u"ar_EG"_s << false << 0.0;
+            << u"\u0664\u0623\u061c-\u0660\u0663"_s << u"ar_EG"_s << false << 0.0;
     QTest::newRow("ar_EG: 4e!3") // Only first character of sign
-            << u"\u0664\u0627\u0633\u061c\u0660\u0663"_s << u"ar_EG"_s << false << 0.0;
+            << u"\u0664\u0623\u0633\u061c\u0660\u0663"_s << u"ar_EG"_s << false << 0.0;
     QTest::newRow("ar_EG: 4x!3") // Only first character of sign and exponent
-            << u"\u0664\u0627\u061c\u0660\u0663"_s << u"ar_EG"_s << false << 0.0;
+            << u"\u0664\u0623\u061c\u0660\u0663"_s << u"ar_EG"_s << false << 0.0;
 }
 
 void tst_QLocale::toDouble()
@@ -588,9 +577,6 @@ void tst_QLocale::toDouble()
     QBENCHMARK {
         actual = loc.toDouble(text, &ok);
     }
-    QEXPECT_FAIL("sv_SE: 4e-3", "Code wrongly assumes single character, QTBUG-107801", Abort);
-    QEXPECT_FAIL("se_NO: 4e-3", "Code wrongly assumes single character, QTBUG-107801", Abort);
-    QEXPECT_FAIL("ar_EG: 4e-3", "Code wrongly assumes single character, QTBUG-107801", Abort);
     QEXPECT_FAIL("en: &infin;", "Localized infinity support missing: QTBUG-95460", Abort);
     QEXPECT_FAIL("ga: Nuimh", "Localized NaN support missing: QTBUG-95460", Abort);
     QCOMPARE(ok, good);
